#!/usr/bin/python
# Copyright 2017 Cumulus Networks Inc, all rights reserved
#
# asic manager
#
#
#
#
#
#
#

import sys
import argparse
import getopt
import os
import subprocess
import json
import math
import time
import logging
import logging.handlers
import traceback
import datetime
import importlib

import cumulus.datapath_utils
import cumulus.datapath_config_parse

global dp_utils
dp_utils = cumulus.datapath_utils.DatapathUtils()

# initialize the global logger
global logger
logger = logging.getLogger('asic_manager')
fmt = logging.Formatter(fmt='%(name)s %(levelname)s: %(message)s')
handler = logging.handlers.SysLogHandler('/dev/log')
handler.setFormatter(fmt)
logger.setLevel(logging.INFO)
logger.addHandler(handler)

# ----------------------------------------------------------
#
class ChipNotSupportedError(Exception):

    def __init__(self, chip_name):
        self.chip_name = chip_name

# ----------------------------------------------------------
#
class AsicManager(object):

    asic_is_supported = False

    def __init__(self, config_manager):
        self.config_manager = config_manager

    def read_state(context, state_desc):
        print 'read state function stubbed on on this platform'

    def clear_manager(context):
        print 'clear function stubbed on on this platform'

# ----------------------------------------------------------
#
class ReadResults(object):
    def __init__(self):
        self.trigger_flags = 0
        self.start_time = datetime.datetime.now()

# ----------------------------------------------------------
#
class StateDesc(object):
    snapshot_flag = 0x1
    log_flag      = 0x2
    collect_flag  = 0x4
    def __init__(self, port_group):

        self.name = port_group.name
        self.action_flags = 0
        for action in port_group.action_list:
            if action == 'snapshot':
                self.action_flags |= self.snapshot_flag
                self.snapshot = port_group.snapshot
            if action == 'log':
                self.action_flags |= self.log_flag
                self.log = port_group.log
            if action == 'collect':
                self.action_flags |= self.collect_flag
                self.collect = port_group.collect

        self.stat_type     = port_group.stat_type
        self.trigger_type  = port_group.trigger_type
        self.port_list     = port_group.linux_port_list
        self.switch_priority_list = port_group.cos_list
        self.curr_snapshot_idx = 0
        self.report_time   = False

# ----------------------------------------------------------
#
class ChipHandle(object):
    def __init__(self, module_name, manager_class_name):
        self.module_name  = module_name
        self.class_name   = manager_class_name

# ----------------------------------------------------------
def _init_config_manager(chip, config_manager, configuration_file):

    if configuration_file == 'default':
        file_dict = dp_utils.get_config_files(chip)
        file_name = file_dict.get('monitor', None)
    else:
        file_name = configuration_file
    config_manager.init_config()
    try:
        config_manager.read_config_file('monitor', file_name)
    except Exception as e:
        raise RuntimeError('configuration file %s was not read correctly' % file_name)
    config_manager.check_config()
    config_manager.process_config()

# ----------------------------------------------------------
#
def write_results_to_file(results, output_file):

    # convert output to json and write to the file
    f = open(output_file, 'w')
    output_dict = {}
    date_info = {}
    date_info['start_datetime'] = str(results.start_time)
    date_info['end_datetime'] = str(results.end_time)
    output_dict['timestamp_info'] = date_info
    output_dict['histogram_info'] = None
    output_dict['buffer_info']    = None
    output_dict['packet_info']    = None
    if len(results.histogram_info.keys()) > 0:
        output_dict['histogram_info'] = results.histogram_info
    if len(results.buffer_info) > 0:
        output_dict['buffer_info'] = results.buffer_info
    if len(results.packet_info.keys()) > 0:
        output_dict['packet_info'] = results.packet_info
    json_output = json.dumps(output_dict)
    f.write(json_output)
    f.close()

# ----------------------------------------------------------
#
def initialize_asic_manager(configuration_file='default') :

    # select the read function
    map = { 'TridentTwo_56850_Chip'     : ChipHandle('cumulus.asic_manager', 'AsicManager'),
            'TridentTwo_56854_Chip'     : ChipHandle('cumulus.asic_manager', 'AsicManager'),
            'TridentTwoPlus_56860_Chip' : ChipHandle('cumulus.asic_manager', 'AsicManager'),
            'TridentTwoPlus_56864_Chip' : ChipHandle('cumulus.asic_manager', 'AsicManager'),
            'Helix4Chip'                : ChipHandle('cumulus.asic_manager', 'AsicManager'),
            'TomahawkChip'              : ChipHandle('cumulus.asic_manager', 'AsicManager'),
            'SpectrumChip'              : ChipHandle('cumulus.mlx_asic_manager', 'MlxAsicManager') }

    chip = dp_utils.get_chip()
    if chip == None :
        raise RuntimeError('chip not found')
    chip_name = chip.__class__.__name__
    if chip_name not in map :
        raise ChipNotSupportedError(chip_name)
    try:
        module_name    = importlib.import_module(map[chip_name].module_name)
        manager_class  = getattr(module_name, map[chip_name].class_name)
    except Exception as e:
        raise
    if manager_class.asic_is_supported == False:
        raise ChipNotSupportedError(chip_name)

    # parse the asic monitor config file
    config_manager = cumulus.datapath_config_parse.ConfigManager(chip)
    if config_manager == None:
        raise RuntimeError('Chip %s config manager object not created' % chip_name)

    _init_config_manager(chip, config_manager, configuration_file)

    # create the asic manager
    asic_manager   = manager_class(config_manager)
    return asic_manager
