#
# Copyright (C) 2017 Cumulus Networks, Inc.
#

#----------------------------------------------------------------------------
#
#   Accton OMP800 chassis
#
#   Do not instantiate this class or import this module directly.
#   Use the parent module's probe() function.  See __init__.py.
#
#----------------------------------------------------------------------------

from cumulus.chassis.base import chassisBase

class chassisImpl(chassisBase):
    """
    Functions specific to the Accton OMP800 Chassis
    """

    # Constants
    _SYSFS_DEVICE       = "/sys/bus/i2c/devices/12-0060"
    _SYSFS_CARD_TYPE    = _SYSFS_DEVICE + "/card_type_id"
    _SYSFS_SLOT         = _SYSFS_DEVICE + "/slot_id"
    _SYSFS_CPU          = _SYSFS_DEVICE + "/cpu_id"

    def __init__(self):
        chassisBase.__init__(self)
        try:
            card_type = self._readint(self._SYSFS_CARD_TYPE)
            hw_slot   = self._readint(self._SYSFS_SLOT)
            cpu       = self._readint(self._SYSFS_CPU)
            sw_slot   = ((1-card_type) * 16) + (hw_slot * 2) + cpu + 1
        except (IOError, ValueError, TypeError) as e:
            raise RuntimeError("Could not determine chassis slot number: %s" %
                               str(e))
        self.slot = sw_slot

    def GetSlotNum(self):
        """
        Return the slot number of the module on which this code is running.
        Fabric cards are 1 to 16.  Line cards are 17 to 32.
        Cards are in this order: 1-A, 1-B, 2-A, 2-B, ...
        The slot number is computed once at instance creation time,
        and not expected to change (duh).
        """
        return self.slot

    def GetMgmtAddr(self):
        """
        Get the IPv6 link local management address of this chassis card
        """
        if self.slot < 17:
            # fabric card
            return "fe80::%x" % (self.slot,)
        elif self.slot % 2 == 1:
            # line card left
            return "fe80::%x01" % ((self.slot - 15)/2,)
        else:
            # line card right
            return "fe80::%x02" % ((self.slot - 16)/2,)

    def GetBmcMgmtAddr(self):
        """
        Get the IPv6 link local management address of the BMC for a slot
        """
        return None

    def GetMgmtIf(self):
        """
        Return the name of the management interface
        """
        return "eth0.4088"

    def GetNumFabIntfs(self):
        """
        Return the number of fabric interfaces on this card.
        """
        if self.slot < 17:
            return 32
        else:
            return 16

        return 0

    def isFabricCard(self):
        """
        Return true if this is a fabric card. Line cards return False.
        """
        return self.slot < 17

    def GetModuleName(self, slot = None):
        """
        Return the name of a slot (fc[a|b] or lc[a|b]).
        Default slot is this module.
        """
        if slot is None:
            slot = self.slot
        card = "fc" if slot < 17 else "lc"
        ab = "a" if slot % 2 == 1 else "b"
        return card + ab
