#
# Copyright (C) 2017 Cumulus Networks, Inc.
#

#----------------------------------------------------------------------------
#
#   Celestica BigStone-G chassis
#
#   Do not instantiate this class or import this module directly.
#   Use the parent module's probe() function.  See __init__.py.
#
#----------------------------------------------------------------------------

from cumulus.chassis.base import chassisBase

class chassisImpl(chassisBase):
    """
    Functions specific to the Celestica BigStone-G Chassis
    """

    # Constants
    _SYSFS_SLOT         = "/sys/devices/platform/syscpld.0/slot_id"
    _SYSFS_SWE          = "/sys/devices/platform/syscpld.0/swe_id"
    _SYSFS_CMM1_STATUS  = "/sys/devices/platform/syscpld.0/cmm1_status"
    _SYSFS_CMM2_STATUS  = "/sys/devices/platform/syscpld.0/cmm2_status"

    def __init__(self):
        chassisBase.__init__(self)
        try:
            hw_slot = self._readint(self._SYSFS_SLOT)
            if hw_slot in (8, 9, 10, 11):
                sw_slot = hw_slot - 7
            elif hw_slot in (0, 1, 2, 3):
                hw_swe = self._readint(self._SYSFS_SWE)
                if hw_swe not in (0, 1):
                    raise RuntimeError("Invalid sysfs SWE ID (%d)" % hw_swe)
                sw_slot = hw_slot * 2 + 5 + hw_swe
            else:
                raise RuntimeError("Invalid sysfs slot ID (%d)" % hw_slot)
        except (IOError, ValueError, TypeError) as e:
            raise RuntimeError("Could not determine chassis slot number: %s" %
                               str(e))
        self.slot = sw_slot

    def GetSlotNum(self):
        """
        Return the slot number of the module on which this code is running.
        Fabric cards are 1 to 4.  Line cards are 5 to 12.
        Line cards are in this order: 1-left, 1-right, 2-left, 2-right, ...
        The slot number is computed once at instance creation time,
        and not expected to change (duh).
        """
        return self.slot

    def GetMasterCmm(self):
        """
        Retrieve the current master CMM number (1 or 2).
        """
        try:
            s1 = self._readstring(self._SYSFS_CMM1_STATUS)
            if s1 == "master":
                return 1
            s2 = self._readstring(self._SYSFS_CMM2_STATUS)
            if s2 == "master":
                return 2
        except (IOError, ValueError, TypeError) as e:
            raise RuntimeError("Could not determine master CMM: %s" % str(e))
        raise RuntimeError("Neither CMM is master (status \"%s\" \"%s\")" %
                           (s1, s2))

    def GetMgmtAddr(self):
        """
        Get the IPv6 link local management address of this chassis card
        """
        if self.slot < 5:
            # fabric card
            return "fe80::%x" % (self.slot,)
        elif self.slot % 2 == 1:
            # line card left
            return "fe80::%x01" % ((self.slot - 3)/2,)
        else:
            # line card right
            return "fe80::%x02" % ((self.slot - 4)/2,)

    def GetBmcMgmtAddr(self):
        """
        Get the IPv6 link local management address of the BMC for a slot
        """
        return self.GetMgmtAddr() + ":1"

    def GetCmmMgmtAddr(self, number):
        """
        Get the IPv6 link local management address of a CMM BMC
        """
        if number < 1 or number > 2:
            raise RuntimeError("Invalid CMM number (%d)" % number)
        return "fe80::c%d:1" % number

    def GetMgmtIf(self):
        """
        Return the name of the management interface
        """
        return "eth0.4088"

    def GetNumFabIntfs(self):
        """
        Return the number of fabric interfaces on this card.
        """
        if self.slot < 5:
            return 32
        else:
            return 16

        return 0

    def isFabricCard(self):
        """
        Return true if this is a fabric card. Line cards return False.
        """
        return self.slot in (1, 2, 3, 4)

    def GetModuleName(self, slot = None):
        """
        Return the name of a slot (fab[1-4] or lc[1-4][lr]).
        Default slot is this module.
        """
        if slot is None:
            slot = self.slot
        if slot < 5:
            return "fab%d" % slot
        elif slot % 2 == 1:
            return "lc%dl" % ((slot - 3) / 2)
        else:
            return "lc%dr" % ((slot - 3) / 2)

    def GetCmmName(self, number):
        return "cmm%d" % number
