#!/usr/bin/python
# Copyright 2017 Cumulus Networks Inc, all rights reserved
#
# datapath-config_parse --
#
#
#
#
#
#
#

import sys
import os
import subprocess
import re
import math
import time
import logging
import logging.handlers
import traceback

import cumulus.porttab

global porttab
porttab = cumulus.porttab.porttab()

# initialize the global logger
global logger
logger = logging.getLogger('datapath_parser')
fmt = logging.Formatter(fmt='%(name)s %(levelname)s: %(message)s')
handler = logging.handlers.SysLogHandler('/dev/log')
handler.setFormatter(fmt)
logger.setLevel(logging.INFO)
#logger.setLevel(logging.DEBUG)
logger.addHandler(handler)

# ==============================================================================
#
class PortDesc(object):

    def __init__(self, chip, config_manager) :
        self.linux_port_list = porttab.get_linux_ports()

# ==============================================================================
#
class BaseTypeConfig(object):

    """
    Base Type Config class
    """
    # ------------------------------------------------------------------
    #
    def __init__ (self, name, format_string) :
        self.name          = name
        self.format_string = format_string
        self.value         = None

    def init_config(self, config_mgr) :
        pass

    def set_config(self, name, value):
        pass

    def check_config(self) :
        pass

    def process_config(self) :
        pass

    def get_print_string(self):
        if self.value != None:
            return self.format_string % (self.name, self.value)
        else :
            return '%s : None' % self.name

    def print_me(self):
        print self.get_print_string()

    def get_value(self):
        return self.value

# ==============================================================================
#
class IntConfig(BaseTypeConfig):
    """
    Integer Config class
    """
    def __init__(self, name):
        super(IntConfig,self).__init__(name, '%s: %d')

    def set_config(self, name, value):
        super(IntConfig,self).set_config(name, value)
        self.value = int(value)

# ==============================================================================
#
class IntListConfig(BaseTypeConfig):
    """
    Integer List Config class
    """
    # ------------------------------------------------------------------
    #
    def __init__ (self, name) :
        super(IntListConfig,self).__init__(name, None)

    def set_config(self, name, value):
        super(IntListConfig,self).set_config(name, value)
        self.value = []
        if value == '':
            return
        int_list_re = re.compile('\[' +'(?P<list>((\d+),)*(\d+)?)' + '\]')
        m = int_list_re.search(value)
        if m :
            typed_value = m.group('list').split(',')
            temp_value_list = []
            for idx in range(len(typed_value)) :
                if typed_value[idx] != '' :
                    temp_value_list.append(int(typed_value[idx]))
            self.value = temp_value_list
        else :
            self.config_mgr.report_error('Config %s value %s not recognized as an integer list' % (name, value))

    def print_me(self):
        print '%s:', self.value, '\n'

# ==============================================================================
#
class FloatConfig(BaseTypeConfig):
    """
    Float Config class
    """
    # ------------------------------------------------------------------
    #
    def __init__ (self, name) :
        super(FloatConfig,self).__init__(name, '%s: %d')

    def set_config(self, name, value):
        super(FloatConfig,self).set_config(name, value)
        self.value = float(value)

# ==============================================================================
#
class BoolConfig(BaseTypeConfig):
    """
    Bool Config class
    """
    # ------------------------------------------------------------------
    #
    def __init__ (self, name) :
        super(IntConfig,self).__init__(name, '%s: %s')

    def set_config(self, name, value):
        super(FloatConfig,self).set_config(name, value)
        if value.lower() == 'true':
            self.value = True
        else:
            self.value = False

# ==============================================================================
#
class StringConfig(BaseTypeConfig):
    """
    String Config class
    """
    # ------------------------------------------------------------------
    #
    def __init__ (self, name) :
        super(StringConfig,self).__init__(name, '%s: %s')

    def set_config(self, name, value):
        super(StringConfig,self).set_config(name, value)
        self.value = value

# ==============================================================================
#
class StringListConfig(BaseTypeConfig):
    """
    String List Config class
    """

    # ------------------------------------------------------------------
    #
    def __init__ (self, name) :
        super(StringListConfig,self).__init__(name, None)

    def set_config(self, name, value):
        super(StringListConfig,self).set_config(name, value)
        self.value = []
        temp_list = value.split('[')
        temp_list = temp_list[1].split(']')
        temp_list = temp_list[0].split(',')
        for item in temp_list:
            if item == '':
                continue
            # XXX fixme!  does not handle whitespace correctly
            item.strip('0')
            item.strip(' ')
            item.strip('[')
            item.strip(']')
            self.value.append(item)

    def print_me(self):
        print '%s:' % self.name, self.value, '\n'

# ==============================================================================
#
class ConfigNode(object):
    """
    class ConfigNode
    """
    # ------------------------------------------------------------------
    #
    def __init__(self, name, config_dict, child_dict):
        self.name        = name
        self.config_dict = config_dict # configurations which have not been created at init time
        self.child_dict  = child_dict  # configurations that have already been created

    def init_config(self, config_mgr):
        self.config_mgr = config_mgr
        for child_node_name, child_node in self.child_dict.items():
            child_node.init_config(config_mgr)

    def set_config(self, name, value):
        parameter_name = name[1]
        if parameter_name not in self.child_dict.keys():
            if parameter_name in self.config_dict.keys():
                child_node = self.config_dict[parameter_name](parameter_name)
                child_node.init_config(self.config_mgr)
                self.child_dict[parameter_name] = child_node
            else:
                raise RuntimeError('%s is not a valid configuration parameter for node %s' % (parameter_name, self.name))
        self.child_dict[parameter_name].set_config(name[1:], value)

    def check_config(self):
        for child_node_name, child_node in self.child_dict.items():
            child_node.check_config()

    def process_config(self):
        for child_node_name, child_node in self.child_dict.items():
            setattr(self, child_node_name, child_node.get_value())
            child_node.process_config()

    def print_me(self):
        for child_node_name, child_node in self.child_dict.items():
            child_node.print_me()

    def get_value(self):
        return self

# ==============================================================================
#
class PortSetConfig(ConfigNode):
    def __init__ (self, name) :
        config_dict = {}
        child_dict = {}
        self.linux_port_list = []
        super(PortSetConfig,self).__init__('Port Set', config_dict, child_dict)

    def set_config(self, name, value):
        # parse the port set string
        if len(self.linux_port_list) > 0:
            logger.error('%s: linux port list was configured twice' % self.name)
            self.linux_port_list = []
        port_group_list = value.split(',')
        for port_group in port_group_list:
            port_group = port_group.strip()
            range_list = port_group.split('-')
            if len(range_list) > 0:
                start_linux_port = range_list[0]
                if start_linux_port not in self.config_mgr.port_desc.linux_port_list :
                    self.config_mgr.report_error('linux start port %s not found in linux port list' % start_linux_port)
                    return []
            if len(range_list) > 1 :
                end_linux_port = range_list[1]
                if end_linux_port not in self.config_mgr.port_desc.linux_port_list :
                    self.config_mgr.report_error('linux end port %s not found in linux port list' % end_linux_port)
                    return []
            else :
                end_linux_port = range_list[0]
            in_range_flag = False
            for linux_port_label in self.config_mgr.port_desc.linux_port_list :
                if linux_port_label == start_linux_port :
                    in_range_flag = True
                if in_range_flag == True :
                    self.linux_port_list.append(linux_port_label)
                if linux_port_label == end_linux_port :
                    break

# ==============================================================================
#
class MonitorActionNode(ConfigNode):
    def __init__ (self, name, config_dict = {}, child_dict = {}) :
        config_dict['packet_congestion_drops'] = IntConfig
        config_dict['packet_error_drops']      = IntConfig
        config_dict['queue_bytes']             = IntConfig
        super(MonitorActionNode,self).__init__('Monitor %s' % name, config_dict, child_dict)

# ==============================================================================
#
class SnapshotActionNode(MonitorActionNode):
    def __init__ (self, name) :
        config_dict = { 'file'       : StringConfig,
                        'file_count' : IntConfig }
        child_dict  = {}
        super(SnapshotActionNode,self).__init__('SnapshotAction %s' % name, config_dict, child_dict)

# ==============================================================================
#
class LogActionNode(MonitorActionNode):
    def __init__ (self, name) :
        config_dict = {}
        child_dict  = {}
        super(LogActionNode,self).__init__('Log Action', config_dict=config_dict, child_dict=child_dict)

# ==============================================================================
#
class CollectActionNode(MonitorActionNode):
    def __init__ (self, name) :
        config_dict = { 'port_group_list': StringListConfig }
        child_dict  = { }
        super(CollectActionNode,self).__init__('CollectAction %s' % name, config_dict, child_dict)

# ==============================================================================
#
class HistogramNode(ConfigNode):

    default_min_bytes_boundary   = 960
    default_histogram_size_bytes = 12288
    default_sample_time_ns       = 1024
    def __init__ (self, name) :
        config_dict = {}
        child_dict  = {}
        name = 'minimum_bytes_boundary'
        child_dict[name] = IntConfig(name)
        child_dict[name].set_config(name, self.default_min_bytes_boundary)
        name = 'histogram_size_bytes'
        child_dict[name] = IntConfig(name)
        child_dict[name].set_config(name, self.default_histogram_size_bytes)
        name = 'sample_time_ns'
        child_dict[name] = IntConfig(name)
        child_dict[name].set_config(name, self.default_sample_time_ns)
        super(HistogramNode,self).__init__('Histogram %s name', config_dict, child_dict)

# ==============================================================================
#
class MonitorNode(ConfigNode):
    def __init__ (self, name) :
        config_dict = {'stat_type'          : StringConfig,
                       'trigger_type'       : StringConfig,
                       'action_list'        : StringListConfig,
                       'timer'              : StringConfig,
                       'port_set'           : PortSetConfig,
                       'cos_list'           : IntListConfig,
                       'histogram'          : HistogramNode,
                       'snapshot'           : SnapshotActionNode,
                       'collect'            : CollectActionNode,
                       'log'                : LogActionNode }
        child_dict           = {}
        self.linux_port_list = []
        self.cos_list        = []
        self.legal_trigger_type_list = ['timer', 'on-demand']
        super(MonitorNode,self).__init__('Monitor ' + name, config_dict, child_dict)

    def check_config (self) :
        if 'stat_type' not in self.child_dict:
            RuntimeError("'stat_type' configuration parameter not included for node %s" % (self.name))
            return
        self.stat_type = self.child_dict['stat_type'].get_value()
        if 'action_list' not in self.child_dict:
            RuntimeError("'action_list' configuration parameter not included for node %s" % (self.name))
            return
        self.action_list = self.child_dict['action_list'].get_value()
        if 'trigger_type' not in self.child_dict:
            self.trigger_type = 'on-demand'
        else:
            self.trigger_type = self.child_dict['trigger_type'].get_value()
        if self.trigger_type not in self.legal_trigger_type_list:
            RuntimeError("%s is not a valid trigger type for node %s" % (self.trigger_type, self.name))
            return
        if self.trigger_type == 'timer':
            # XXX check the timer string
            pass
        if 'port_set' not in self.child_dict:
            self.linux_port_list = self.config_mgr.port_desc.linux_port_list
        else :
            # move the linux port list to the monitor node
            self.linux_port_list = self.child_dict['port_set'].linux_port_list
        if self.stat_type == 'histogram' and 'histogram' not in self.child_dict:
            RuntimeError("Histogram configuration is not included for node %s" % (self.name))
            return
        for action in self.action_list:
            if action not in self.child_dict:
                RuntimeError("%s is not a valid action for node %s" % (action, self.name))
                return
        # XXX for action with packet drop thresh: check stat type
        # XXX for action with queue leng thresh:  check stat type

        super(MonitorNode,self).check_config()

# ==============================================================================
#
class PortGroupListNode(ConfigNode):
    def __init__ (self, name) :
        config_dict     = { 'port_group_list' : StringListConfig }
        child_dict      = {}
        self.group_label = name
        super(PortGroupListNode,self).__init__('%s Port Group List' % name, config_dict, child_dict)

    def set_config (self, name, value):
        parameter_name = name[1]
        if parameter_name in self.config_dict:
            super(PortGroupListNode,self).set_config(name, value)
        elif 'port_group_list' in self.child_dict and parameter_name in self.child_dict['port_group_list'].value:
            port_group_name = parameter_name
            if port_group_name not in self.child_dict :
                if self.group_label == 'Monitor':
                    self.child_dict[port_group_name] = MonitorNode(port_group_name)
                    self.child_dict[port_group_name].init_config(self.config_mgr)
                else :
                    RuntimeError('%s is not a valid port group list label' % (self.group_label))
                    return
            self.child_dict[port_group_name].set_config(name[1:], value)

    def get_port_group_dict(self):
        port_group_dict = {}
        if 'port_group_list' not in self.child_dict:
            return port_group_dict
        for pg_name in self.child_dict['port_group_list'].value:
            port_group_dict[pg_name] = self.child_dict[pg_name]
        return port_group_dict

# ==============================================================================
#
class IgnoreConfig(object):

    def init_config(self, config_mgr) :
        pass

    def set_config(self, name, value) :
        pass

    def check_config(self) :
        pass

    def process_config(self) :
        pass

# ==============================================================================
#
class ConfigManager(object):

    def __init__(self, chip) :

        super(ConfigManager,self).__init__()

        self.chip            = chip
        self.port_desc       = PortDesc(self.chip, self)
        self.monitor         = PortGroupListNode('Monitor')

        self.manager_list = [ self.monitor ]
        self.error_comment   = ""

    # ------------------------------------------------------------------
    def init_config (self) :
        for manager in self.manager_list:
            manager.init_config(self)

    # ------------------------------------------------------------------
    def read_config_file (self, file_label, file_name) :

        if file_name == None:
            logger.error('read_config_file:  %s file name is not set, skipping' % file_label)
            return 0

        line_buffer = ""
        f = open(file_name)
        for line in f:
            line = line.rstrip()
            line_buffer = line_buffer + line
            line_buffer = line_buffer.rstrip("\\")
            if not re.match(r"(.*)\\", line) :
                line = line_buffer
                line_buffer = ""
            else :
                continue
            config = [x.strip() for x in line.partition('=') if len(x) > 0]
            if len(config) == 0 :
                continue
            if config[0].startswith("#") :
                continue
            if len(config) < 3 :
                self.report_error('configuration line not recognized: %s' % line)
                continue
            name  = config[0]
            value = ' '.join(config[2:])
            # trim any comment
            value = value.split('#')
            value = value[0]
            # remove leading and trailing spaces
            value = value.strip()

            name_level = name.split('.')
            if not name_level :
                self.report_error('configuration parameter %s not recognized' % name)
                continue
            manager_name = name_level[0]
            manager = getattr(self, manager_name, None)
            if manager == None:
                self.report_error('configuration parameter level %s in %s not supported' % (manager_name, name))
                continue
            manager.set_config(name_level, value)
        f.close()

    # ------------------------------------------------------------------
    def report_error(self, error_msg) :
        logger.error(error_msg)
        self.error_comment += '# %s\n' % error_msg

    # ------------------------------------------------------------------
    def check_config (self) :
        for manager in self.manager_list:
            manager.check_config()

    # ------------------------------------------------------------------
    def process_config (self) :
        for manager in self.manager_list:
            manager.process_config()

