#
# Copyright 2012.  Cumulus Networks, Inc.
#
# ledup.py --
#
#     LED microcontroller code generation
#

import os
import tempfile
import subprocess
import shutil
import itertools

class LEDController:
    pass

class BCMLEDController(LEDController):
    '''
    "the SML LED interface contains a very small, very simple processor (1500
     gates) to do all the appropriate data formatting and interface control. It
     is vast overkill"
    '''
    _ledasm = '/usr/lib/cumulus/ledasm'

    # number port data elements accessible from each controller
    ports_per_chain = None

    # leds in output shift register
    num_leds = None

    # platform specific, helper function definitions
    led_functions = ()

    # period of activity blink (in milliseconds)
    blink_period = 90

    # Using pad_n_XXX adds 10 extra bytes of asm code for that routine
    pad_n_code_size = 10

    # pad_XXX routines are 4 bytes long (in rare cases can be eliminated)
    # Don't need this unless we decide to deal with TODO optimization case
    # pad_code_size = 4

    # asm call instructions take 2 bytes (opcode + immediate addr)
    pad_call_size = 2

    # looped call invocations take 4 bytes (two for ld, two for call)
    pad_n_call_size = 4

    # Invocation of loop takes 4 bytes.
    # Each simple call to pad_XXX takes 2 bytes
    # So don't insert a loop until more than 2 calls in a run.
    loop_insert_cost = 2

    # default pad value
    default_pad = 1

    # Include additional code for ethtool phys_port_id only if supported
    has_phys_id = False

    # Base name of LED controller data array, with placeholder for controller num.
    data_array_name = "CMIC_LEDUP{idx}_DATA_RAM"

    data_array_base_addr = 0xC0

    logical_to_led_index = {}

    # Hack to allow one instance of ledup to reference multiple sets of saved state
    call_list = {}

    static_code = { 'constants' : '''
;;;
;;; Constants
;;;

;;;
;;; The link scan base address points to an array of per-port data (32
;;; elements, 2 bytes each) which is continually updated by the main CPU's
;;; linkscan task.
LINKSCAN_P         equ    0x80 ; port data base address

;;; bits in the port data fields
LINKSCAN_LINK      equ    0x0  ; link down/up
LINKSCAN_TX        equ    0x4  ; transmitted packet
LINKSCAN_RX        equ    0x5  ; received packet

;;;
;;; The port status base address points to an array of per-port data (32
;;; elements, 2 bytes each) which is continually updated by the switch chip.
;;; This address is implied by port related op codes
PORTSTATUS_P       equ    0x00 ; port status base address

;;; bits in the linkscan port status fields
PORTSTATUS_RX      equ    0x0  ; received packet
PORTSTATUS_TX      equ    0x1  ; transmitted packet
PORTSTATUS_COLL    equ    0x2  ; collision
PORTSTATUS_SPEED_C equ    0x3  ; 100Mbps
PORTSTATUS_SPEED_M equ    0x4  ; 1Gbps
PORTSTATUS_DUPLEX  equ    0x5  ; half/full duplex
PORTSTATUS_FLOW    equ    0x6  ; flow control capable
PORTSTATUS_LINKUP  equ    0x7  ; link down/up
PORTSTATUS_LINKEN  equ    0x8  ; link disabled/enabled
PORTSTATUS_ZERO    equ    0xe  ; always 0
PORTSTATUS_ONE     equ    0xf  ; always 1

;;;
;;; Blink constants
TICKS_PER_SEC      equ    30   ; LED program runs at 30Hz
'''
    }

    static_no_id_func_code = {
        'variables': '''
;;;
;;; Variables
;;;
;;; Addresses 0xa0 to 0xff are initialized to zero and usable by the program.
;;;

;;;
;;; Keep track of blink state, modified each cycle as a timer
BLINK_TICKS_P      equ    0xf0
BLINK_STATE_P      equ    0xf1
''',
    'functions' : '''
blink_update_state:
        ld      a,(BLINK_TICKS_P)
        cmp     a,0
        jz      blink_toggle
        dec     a
        ld      (BLINK_TICKS_P),a
        ret
blink_toggle:
        ld      a,(BLINK_STATE_P)
        xor     a,0x01
        and     a,0x01
        ld      (BLINK_STATE_P),a
        ld      a,TICKS_PER_BLINK
        ld      (BLINK_TICKS_P),a
        ret
''',
    'blink_updater_calls': '''
        call blink_update_state''',
    }

    static_id_func_code = {
        'variables': '''
;;;
;;; Variables
;;;
;;; Addresses 0xa0 to 0xff are initialized to zero and usable by the program.
;;;

;;; Map and flags for ID/Locator mode
PORT_MAP_ARRAY     equ    0x%(base_addr)x
ID_ENABLE_P        equ    0xe0
ID_ENABLE_BIT      equ    0x00
ID_PORT_P          equ    0xe1

;;;
;;; Keep track of blink state, modified each cycle as a timer
;;;  The "TICKS_PER_XXX_P" locations are initialized by /etc/bcm.d/rc.led
BLINK_TICKS_P      equ    0xf0
BLINK_STATE_P      equ    0xf1
TICKS_PER_BLINK_P  equ    0xf2
WIGGLE_TICKS_P     equ    0xf3
WIGGLE_STATE_P     equ    0xf4
TICKS_PER_WIGGLE_P equ    0xf5
WAG_TICKS_P        equ    0xf6
WAG_STATE_P        equ    0xf7
TICKS_PER_WAG_P    equ    0xf8
''' % {'base_addr': data_array_base_addr},

        'functions': '''
;;;  Routine to update state of one of the three count-down driven blinkers
;;;   Base address of 3-element array sent in via 'b'
blink_update:
        ld      a, (b)
        cmp     a, 0
        jz      blink_toggle
        dec     a
        ld      (b),a
        ret
blink_toggle:
        inc     b        ; Point to the state value
        ld      a, (b)   ; Read in current state
        xor     a, 0x01  ; Invert current state
        and     a, 0x01
        ld      (b), a   ; Write current state back
        inc     b        ; Get max counter value
        ld      a, (b)
        sub     b, 2     ; Point back to counter
        ld      (b), a   ; Reset counter to max value
''',
        'blink_updater_calls': '''
        ld      b, BLINK_TICKS_P
        call    blink_update
        ld      b, WIGGLE_TICKS_P
        call    blink_update
        ld      b, WAG_TICKS_P
        call    blink_update
''',
    }

    # Address and limit value of blink loop constants
    # Used to generate initialization writes to RAM
    in_ram_constants = {'blink':  {'addr': 0xf2, 'val': 3},
                        'wiggle': {'addr': 0xf5, 'val': 19},
                        'wag':    {'addr': 0xf8, 'val': 119}}

    @property
    def defines(self):
        tpb = self.blink_period / 30.0
        if tpb != int(tpb):
            raise RuntimeError('non-integer blink cycles, correct period to multiple of 30')
        return '''
;;;
;;; Platform-specific defines
TICKS_PER_BLINK    equ    %(ticks_per_blink)u    ; 0 to alternate every cycle (15Hz)
NUM_LEDS           equ    %(num_leds)u
''' % { 'ticks_per_blink' : int(tpb),
        'num_leds' : self.num_leds }

    pad_zero = '''
pad_zero:
        push  0
        pack
        ret
'''

    pad_one = '''
pad_one:
        push  1
        pack
        ret
'''

    pad_n_zero = '''
pad_n_zero:
        push  0
        pack
        dec a
        jnz pad_n_zero
        ret
'''

    pad_n_one = '''
pad_n_one:
        push  1
        pack
        dec a
        jnz pad_n_one
        ret
'''

    def __init__(self):
        self.set_functions = set()
        self.set_calls = []
        self._template = '''
%(constants)s
%(defines)s
%(variables)s

main:
        %(set_calls)s
%(blink_updater_calls)s
        send NUM_LEDS

%(functions)s
%(set_functions)s
%(led_functions)s
'''

    def configure(self, controller_idx, ports):
        '''
        Configure LED controller, including saving call_list for later use by led_map()

        Ask each port which LEDs it wants to claim and which code should run.
        We call the port.set_leds() method of each port which returns a
        dictionary mapping led number to a set_led function call.  By default,
        we assume all LEDs require padding.  The ports should put a 'None' in
        the dictionary for LEDs that don't need padding.
        '''
        calls = dict.fromkeys(range(self.num_leds), self.default_pad)

        '''
        Mapping between logical port to led index should be part
        from Port container class. Port container class should have relationships
        between physical ports, logical ports, linux interfaces , led indexes and
        serdes which spawn across various port classes. The fix to include this
        information as part of ledup.py is a short term fix.
        Ex:logical_to_led_index for Ganged port (40G/4) swp13
        '13': [28, 30, 32, 34]
        '''
        logical_to_led_index = {}
        for port in sorted(ports, key=lambda x: x.gang_label):
            ganged_port = port.gang_label
            if logical_to_led_index.get(ganged_port) is None:
                logical_to_led_index[ganged_port] =  [port.leds[1]]
            else:
                logical_to_led_index[ganged_port].append(port.leds[1])

        lport = 1
        for port in sorted(ports, key=lambda x: x.hw_port_num(0)):
            for sub in range(port.num_logical_ports):
                # Arguments for set_leds should be led_index_list and  hw_interface_number only
                # All other arguments need to be cleaned up
                calls.update(port.set_leds(lport, sub, logical_to_led_index[port.gang_label]))
                lport += 1

        sorted_calls = [calls[x] for x in sorted(calls.keys())]
        self.call_list[controller_idx] = [(k, len(list(g))) for k, g in
                                          itertools.groupby(sorted_calls)]
        use_pad_n = self._eval_looping(self.call_list[controller_idx])
        for call, count in self.call_list[controller_idx]:
            if call in (0, 1):
                loop_this_call = use_pad_n[call] and (count > self.loop_insert_cost)
                self._led_pad_call(call, count, loop_this_call)
            elif call is not None:
                self._led_port_call(call, count)

    def _eval_looping(self, calls):
        '''Return boolean list.  True for pad call type(s) that benefit from looping'''
        decision = [False, False]
        looped_size = self._calc_looped_instructions(calls)
        unrolled_size = self._calc_unrolled_instructions(calls)
        for call in (0, 1):
            decision[call] = looped_size[call] < unrolled_size[call]
        return decision

    # TODO: Take account of one corner case: eliminate of "ordinary" pad_XXX routine.
    #  It can be cheaper to always use pad_n_XXX if doing so allows you to drop the
    #  pad_XXX code.  But the max gain from this is just a few bytes and the complexity
    #  of deciding is high - so skipping that for now.  Things to consider if/when taking
    #  that on: what if there are just two singleton calls to pad_XXX?  Dropping that out
    #  may be worth it, iff there is at least one long run of pads, because we can just do
    #  ld a, 1/call pad_n_XXX and reuse the pad_n_XXX routine instead.
    def _calc_looped_instructions(self, calls):
        size = [0, 0]
        loop_code_cost = [0, 0]
        for call, count in calls:
            if call in (0, 1):
                if count <= self.loop_insert_cost:
                    size[call] += count * self.pad_call_size
                else:
                    size[call] += self.pad_n_call_size
                    loop_code_cost[call] = self.pad_n_code_size
        for call in (0, 1):
            size[call] += loop_code_cost[call]
        return size

    def _calc_unrolled_instructions(self, calls):
        size = [0, 0]
        for call, count in calls:
            if call in (0, 1):
                size[call] += count * self.pad_call_size
        return size

    def _led_pad_call(self, value, length, emit_loop):
        if length == 0:
            return

        if value == 0:
            pad = 'zero'
        elif value == 1:
            pad = 'one'
        else:
            raise RuntimeError('bad pad value: %s', value)

        if emit_loop:
            func = 'pad_n_%s' % pad
            self.set_calls.append('''
        ld      a, %u''' % length)
            self.set_calls.append('''
        call    %s''' % func)
        else:
            func = 'pad_%s' % pad
            for x in range(length):
                self.set_calls.append('''
        call    %s''' % func)
        self.set_functions.add(getattr(self, func))

    def _led_port_call(self, call, length):
        if length == 0:
            return
        (call_name, call_code, call_port) = call

        for x in range(0, length):
        # rename the function to be unique to the port class
            fn_name = 'set_led_%s' % call_name
            fn_code = call_code.replace('set_led:', '%s:' % fn_name)
            self.set_functions.add(fn_code)
            if call_port is not None:
                self.set_calls.append('''
        ld      a,%u''' % self._chain_port(call_port))
                self.set_calls.append('''
        call    %s''' % fn_name)

    def _chain_port(self, port_num):
        '''
        convert a hardware port number to a led chain port number
        '''
        return ((port_num - 1) % self.ports_per_chain) + 1

    def led_map(self, controller_idx):
        '''
        Return an ordered list of the led chain port numbers for set of ports.

        Normally, would be called with a list of ports on a given controller
        '''
        map_list = []
        for call, count in self.call_list[controller_idx]:
            if call in (0, 1):
                continue
            elif call is not None:
                (call_name, call_code, call_port) = call
                map_list.append(call_port)
        return map_list

    def led_memory_init_commands(self, controller_idx):
        '''
        Return a list of commands to be included into rc.led

        List is empty for systems with has_phy_id False, since this is
          useful only for those cases.
        These commands do two things:
          1) initialize the LEDuP Data RAM with a list of the PORTSTATUS
              register offsets.  This can allow the main loop to be
              simplified, using those offsets in place of the 'ld a, x' values
          2) initialize the three blink counter 'ticks per blink' values
        '''
        cmd_list = []
        if self.has_phys_id is False:
            return cmd_list
        cmd_params = {'name': self.data_array_name.format(idx=controller_idx),
                      'addr': self.data_array_base_addr,
                      'val' : None}
        for cmd_params['val'] in self.led_map(controller_idx):
            cmd_list.append('setreg {name}[0x{addr:x}] {val}'.format(**cmd_params))
            cmd_params['addr'] += 1
        for const in self.in_ram_constants.values():
            cmd_params.update(const)
            cmd_list.append('setreg {name}[0x{addr:x}] {val}'.format(**cmd_params))
        return cmd_list

    @property
    def asm(self):
        code = self.static_code.copy()
        if self.has_phys_id:
            code.update(self.static_id_func_code)
        else:
            code.update(self.static_no_id_func_code)
        code.update({ 'set_functions' : ''.join(self.set_functions),
                      'set_calls'     : ''.join(self.set_calls),
                      'defines'       : self.defines,
                      'led_functions' : ''.join(self.led_functions) })
        return self._template % code

    @property
    def hex(self):
        return self._compile(self.asm)

    def _compile(self, asm):
        '''
        Compile a LED program.  We have to do this in a temporary directory
        because the assembler only deals in files.

        XXX- For extra credit, someone should append stdout and stderr to any
        raised exceptions.
        '''
        tempdir = tempfile.mkdtemp()
        try:
           asm_filename = os.path.join(tempdir, 'code.asm')
           asm_file = open(asm_filename, 'w')
           asm_file.write(asm)
           asm_file.close()

           null = open('/dev/null')
           subprocess.check_call((self._ledasm, asm_filename[:-4]),
                                 stdin=null, stdout=null, stderr=null)

           hex_filename = os.path.join(tempdir, 'code.hex')
           hex_file = open(hex_filename, 'r')

           return hex_file.read()
        finally:
           shutil.rmtree(tempdir)

class TriumphLEDController(BCMLEDController):
    ports_per_chain = 64

class Helix4LEDController(BCMLEDController):
    ports_per_chain = 64

class Hurricane2LEDController(BCMLEDController):
    ports_per_chain = 64

class TridentLEDController(BCMLEDController):
    ports_per_chain = 36

class TridentTwoLEDController(BCMLEDController):
    ports_per_chain = 64

class TridentTwoPlusLEDController(BCMLEDController):
    ports_per_chain = 64
    
class TomahawkLEDController(BCMLEDController):
    ports_per_chain = 64

    def _chain_port(self, call_port):
        return call_port
