#!/usr/bin/env python
# Copyright 2016 Cumulus Networks, Inc.
#
# mroute-check asic backend helper

import json
import os
import re
import socket
import struct
import subprocess
import sys

import cumulus.porttab

try:
    import bcmshell
except:
    bcmshell = None

class KernMdbEntry:
    def __init__(self, vid, grp):
        # grp here is the DMAC
        self.grp = grp
        self.vid = vid
        self.swp = []

    def add_port(self, dev):
        self.swp.append(dev)

    def __str__(self):
        dispStr = ''
        if self.vid != -1:
            dispStr += '  vid: %d grp: %s\n' % (self.vid, self.grp)
        else:
            dispStr += '  grp: %s\n' % (self.grp)
        for swp in self.swp:
            dispStr += '    port: %s\n' % swp
        return dispStr

class KernMdbCache:
    def __init__(self, dev, vlan_filtering, mdb_entries={}):
        self.dev = dev
        self.vlan_filtering = vlan_filtering
        # key - vlan.grp-mac, value - KernMdbEntry
        self.mdb_entries = mdb_entries

    def __str__(self):
        dispStr = ''
        dispStr += 'dev: %s:\n' % (self.dev)
        for key in self.mdb_entries:
            dispStr += '%s' % self.mdb_entries[key]
        return dispStr

class KernVlanEntry:
    def __init__(self, vid):
        # grp here is the DMAC
        self.vid = vid
        self.tagged_swp = []
        self.untagged_swp = []

    def add_port(self, port, untagged):
        if untagged:
            self.untagged_swp.append(port)
        else:
            self.tagged_swp.append(port)

    def __str__(self):
        dispStr = ''
        dispStr += 'vid: %d\n' % self.vid
        dispStr += '  tagged: %s\n' % ','.join(self.tagged_swp)
        dispStr += '  untagged: %s\n' % ','.join(self.untagged_swp)
        return dispStr

class KernDevEntry:
    def __init__(self, dev, kind, vlan_filtering):
        self.dev = dev
        self.kind = kind
        self.vlan_filtering = vlan_filtering

    def __str__(self):
        dispStr = '%s: kind: %s vlan_filtering: %d\n' % (self.dev, self.kind, self.vlan_filtering)
        return dispStr

class PortmapEntry:
    def __init__(self, dport, logical_port, linux_port):
        self.dport = dport
        self.logical_port = logical_port
        self.linux_port = linux_port
        self.port_vid = -1

    def update_port_vid(self, port_vid):
        self.port_vid = port_vid

    def __str__(self):
        return ('dport: %s, lport: %d, linux_port: %s, port_vid: %d' %\
                    (self.dport, self.logical_port, self.linux_port, self.port_vid))

class VlanXlate:
    def __init__(self, lport, ivid, evid):
        self.lport = lport
        self.ivid = ivid
        self.evid = evid # if -1 packet is shipped untagged

    def __str__(self):
        return ('lport: %s, ivid: %d, evid: %d' %\
                (self.lport, self.ivid, self.evid))

class VlanEntry:
    def __init__(self, vid, untagged_dports):
        self.vid = vid
        self.untagged_dports = untagged_dports

    def __str__(self):
        return ('vid: %d, untagged_ports: %s' %\
                (self.vid, ','.join(self.untagged_dports)))

class Encap:
    def __init__(self, linux_port, bcm_dport, intf, vlan, evid):
        self.linux_port = linux_port
        self.bcm_dport = bcm_dport
        self.intf = intf
        self.vlan = vlan
        self.evid = evid

    def __str__(self):
        _encap_str = 'port: %s' % self.linux_port
        if self.evid != -1:
            _encap_str += ', vlan: %d' % self.evid
        return _encap_str

    def encap2swp(self):
        if self.evid == -1:
            return self.linux_port
        return '%s.%d' % (self.linux_port, self.evid)

class ReplicationGroup:
    def __init__(self, mcast_grp, l2_encaps, l3_encaps):
        # encaps is a list of Encap
        self.mcast_grp = mcast_grp
        self.l2_encaps = l2_encaps
        self.l3_encaps = l3_encaps

    def __str__(self):
        dispStr = ''
        for encap in self.l3_encaps:
            dispStr += '  %s\n' % encap
        if self.l2_encaps:
            dispStr += '  L2 replication ports:\n'
        for encap in self.l2_encaps:
            dispStr += '    %s\n' % encap
        return dispStr

    def get_l3_linux_encaps(self):
        return set([encap.encap2swp() for encap in self.l3_encaps])

    def get_l2_linux_encaps(self):
        return set([encap.encap2swp() for encap in self.l2_encaps])

class Hmroute:
    def __init__(self, sip, dip, iif, mcast_grp):
        self.sip = sip
        self.dip = dip
        self.iif = iif
        self.kernel_iif = None
        self.grp = mcast_grp | 0x2000000
        self.iif_mismatch = []
        self.oil_mismatch = []

    def __str__(self):
        dispStr = '(%s, %s), iif: %s' % (self.sip, self.dip, self.kernel_iif if self.kernel_iif else self.iif)
        if self.iif_mismatch or self.oil_mismatch:
            dispStr += '\n  errors:'
        for mismatch in self.iif_mismatch:
            dispStr += '\n    %s' % mismatch
        for mismatch in self.oil_mismatch:
            dispStr += '\n    %s' % mismatch
        return dispStr

class Asic:
    _devs_supported = ('bridge', 'bond', 'vlan', 'swp')

    def __init__(self):
        self.desc = "asic"
        self.pt = cumulus.porttab.porttab()
        self.bcm = bcmshell.bcmshell()

        self.linux_ports = self.pt.get_linux_ports()
        # key: devName info: KernDevEntry
        self.kern_dev_cache = {}

        # key: devName(br) info: KernMdbCache; on-demand cache that is created
        # anytime a bridge's mdb entries are needed
        self.kern_mdb_cache = {}

        # key: vid info: KernVlanEntry
        self.flood_mdb_misses = True
        self.kern_vlan_cache_created = False
        self.kern_vlan_cache = {}

        # key: devName(br) info: list of members (swpN.vid)
        self.kern_br_cache = {}

        # list of PortmapEntry
        self.portmap = []
        self._parse_portmap()
        # XXX - convert parse_ports to cache if this becomes to expensive
        self._parse_ports()

        # XXX - parsing l3 intf table can be expensive need to convert this
        # into an on-demand-cache
        # key - intf_id, info - vlan
        self.l3_intfs = {}
        self._parse_l3_intf()

        # list of VlanXlate
        self.vlan_xlate = []
        self._parse_vlan_xlate()

        # key: vlan, info: vlan_entry; this is an on-demand cache i.e. not the
        # entire vlan table
        self.vlans = {}

        # key - mcast_grp, info - ReplicationGroup
        self.replication_groups = {}
        self._parse_multicast_repln_groups()

        # key - '(a.b.c.d, a.b.c.d)', info - Hmroute
        self.mroutes = {}
        if self._is_1G_chip():
            self._parse_ipmc_table()
        else:
            self._parse_l3_entry_ipmc_table()

    def __str__(self):
        ipmr_str = 'Asic ip mroutes:\n'
        for key in sorted(self.mroutes):
            ipmr_str += self._dump_one_mroute(self.mroutes[key])
        return ipmr_str

    @staticmethod
    def _get_cmd_output(cmdList, dispStr=None):
        try:
            with open(os.devnull, 'w') as fnull:
                dispStr = subprocess.check_output(cmdList, stderr=fnull)
        except subprocess.CalledProcessError:
            pass
        return dispStr

    def _is_1G_chip(self):
        try:
            unit_str = self.bcm.run('show unit')
        except IOError:
            # bcmshell is not accessible, maybe switchd is not running?
            raise RuntimeError('show unit timed out')

        # currently H4 is the only 1G on which ipmc is supported
        if "56340" in unit_str:
            return True
        return False

    def _key_to_sip_dip(self, key):
        pat = re.compile("\((?P<sip>\S+), (?P<dip>\S+)\)")
        obj = pat.match(key)
        if obj:
            sip = obj.group('sip')
            dip = obj.group('dip')
        else:
            sip = None
            dip = None
        return sip, dip

    def _parse_portmap(self):
        # Sample "show portmap" output (T2) -
        #
        #             logical  physical    mmu   ucast Qbase  mcast Qbase
        #      cpu0      0         0        52         0         2000
        #       xe0      1        25         6        72         1540
        # >>>>>>>>>>>>>>>>>>>>>>>>>> SNIP >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
        try:
            portmap_str = self.bcm.run('show portmap')
        except IOError:
            # bcmshell is not accessible, maybe switchd is not running?
            raise RuntimeError('show portmap timed out')

        portmap_lines = portmap_str.splitlines()
        if len(portmap_lines) < 2 or 'Error' in portmap_lines[0]:
            raise RuntimeError('show portmap is not supported on this chip')

        try:
            logicalIndex = portmap_lines[0].split().index('logical') + 1
        except ValueError:
            raise RuntimeError('show portmap format unexpected')

        for port_str in portmap_lines[1:]:
            port_cols = port_str.split()
            dport = port_cols[0]
            logical_port = int(port_cols[logicalIndex])
            try:
                linux_port = self.pt.bcm2linux(dport)
            except:
                linux_port = None
            portmap_entry = PortmapEntry(dport, logical_port, linux_port)
            self.portmap.append(portmap_entry)

    def _parse_ports(self):
        try:
            dispStr = self.bcm.run('dump chg port')
        except IOError:
            raise RuntimeError('dump chg port')
        entries = dispStr.splitlines()
        for entry in entries:
            key, fields = self._field_dict_build(entry)
            if not key:
                continue
            port_vid = fields.get('PORT_VID', '0x0')
            if '0x' in port_vid:
                port_vid = int(port_vid, 16)
            else:
                port_vid = int(port_vid)
            self._update_portmap_port_vid(int(key), port_vid)

    def _parse_vlan(self, vid):
        cmd = 'vlan show %d' % vid
        try:
            dispStr = self.bcm.run(cmd)
        except IOError:
            raise RuntimeError(cmd)
        pat = re.compile('vlan (?P<vlan>\d+).*untagged (?P<ports>\S+)')
        obj = pat.search(dispStr)
        if obj:
            vlan = int(obj.group('vlan'))
            port_str = obj.group('ports')
            cmd = 'ports ' + port_str
            try:
                dispStr = self.bcm.run(cmd)
            except IOError:
                raise RuntimeError(cmd)
            portList = []
            ports = dispStr.splitlines()
            if len(ports) < 3:
                return

            for port in ports[2:]:
                portList.append(port.split()[0].split('(')[0])
            self.vlans[vlan] = VlanEntry(vlan, portList)

    def _parse_l3_intf(self):
        # Sample output:
        # Free L3INTF entries: 8153
        # Unit  Intf  VRF Group VLAN    Source Mac     MTU TTL Tunnel InnerVlan  NATRealm
        # ------------------------------------------------------------------
        # 0     1     0     0     3001 44:38:39:00:69:84  1500 0    0     0     0
        try:
            dispStr = self.bcm.run('l3 intf show')
        except IOError:
            raise RuntimeError('l3 intf show')

        lines = dispStr.splitlines()
        title = lines[1].split()
        intf_index = title.index('Intf')
        vlan_index = title.index('VLAN')

        for line in lines[3:]:
            l3_intf = line.split()
            if len(l3_intf) < vlan_index:
                continue
            intf = int(l3_intf[intf_index])
            vlan = int(l3_intf[vlan_index])
            self.l3_intfs[intf] = vlan

    def _parse_ipmc_table(self):
        try:
            dispStr = self.bcm.run('ipmc table show')
        except IOError:
            raise RuntimeError('ipmc table show timed out')

        # sample format -
        # SRC IP ADDRESS  MC IP ADDRESS     VLAN MOD PORT V COS TS   GROUP     VRF
        # CLASS HIT
        # 0.0.0.0         239.1.1.3          0   0    4   1  -1  0   0x2000002   0
        # 0     no
        # 33.1.1.1        239.1.1.1          0   0    5   1  -1  0   0x2000003   0
        # 0     n
        addr_str_len = 2
        lines = dispStr.splitlines()
        title = lines[0].split()
        src_index = title.index('SRC')
        mc_index = title.index('MC') - addr_str_len
        addr_offset = 2 * addr_str_len
        grp_index = title.index('GROUP') - addr_offset
        vlan_index = title.index('VLAN') - addr_offset

        for line in lines[2:]:
            ipmc = line.split()
            if len(ipmc) < grp_index:
                continue
            sip = ipmc[src_index]
            dip = ipmc[mc_index]
            iif = int(ipmc[vlan_index])
            mcast_grp = int(ipmc[grp_index], 16)
            mroute = Hmroute(sip, dip, iif, mcast_grp)
            self.mroutes['(%s, %s)' % (sip, dip)] = mroute

    def _parse_vlan_xlate(self):
        try:
            dispStr = self.bcm.run('vlan translate egress show')
        except IOError:
            raise RuntimeError('vlan translate egress show')
        entries = dispStr.splitlines()
        for entry in entries:
            fields = entry.split(',')
            tvs = {}
            for field in fields:
                if 'modid' in field:
                    pat = re.compile('modid = (?P<modid>\d+) port = (?P<port>\d+)')
                    obj = pat.search(field)
                    if obj:
                        tvs['modid'] = obj.group('modid')
                        tvs['port'] = obj.group('port')
                else:
                    tv = field.strip().split('=')
                    tvs[tv[0].strip()] = tv[1].strip()

            # XXX - temporarily assume that modPort is the same as logical
            # port
            port = int(tvs.get('port', '-1'))
            old_vid = int(tvs.get('old_vid', '-1'))
            new_vid = int(tvs.get('new_vid', '-1'))
            self.vlan_xlate.append(VlanXlate(port, old_vid, new_vid))

    def _parse_l3_entry_ipmc_table(self):
        try:
            dispStr = self.bcm.run('dump chg l3_entry_ipv4_multicast')
        except IOError:
            raise RuntimeError('dump chg l3_entry_ipv4_multicast')
        entries = dispStr.splitlines()
        for entry in entries:
            key, fields = self._field_dict_build(entry)
            if not key:
                continue
            addr = int(fields.get('SOURCE_IP_ADDR', '0x0'), 16)
            sip = socket.inet_ntoa(struct.pack('!L', addr))
            addr = int(fields['GROUP_IP_ADDR'], 16)
            dip = socket.inet_ntoa(struct.pack('!L', addr))
            iif = fields.get('EXPECTED_L3_IIF', '0')
            iif = int(iif, 16) if '0x' in iif else int(iif)
            mcast_grp = int(fields.get('L3MC_INDEX', '0x'), 16)
            mroute = Hmroute(sip, dip, iif, mcast_grp)
            self.mroutes['(%s, %s)' % (sip, dip)] = mroute

    def _parse_one_multicast_repln_group(self, dispStr):
        group = dispStr.splitlines()
        if len(group) < 1 or 'L2' in group[0]:
            return

        group_id = int(group[0].split()[0], 16)
        l2_encaps = []
        l3_encaps = []
        for encap in group[1:]:
            pat = re.compile('\s+port (?P<port>\S+), encap id (?P<intf>[+-]?\d+)')
            obj = pat.match(encap)
            if obj:
                dport = obj.group('port')
                intf = int(obj.group('intf'))
                if intf == -1:
                    l2_encaps.append(Encap(self.pt.bcm2linux(dport), dport, -1, -1, -1))
                else:
                    vlan = self.l3_intfs.get(intf, -1)
                    evid = self._ivid2evid(dport, vlan)
                    l3_encaps.append(Encap(self.pt.bcm2linux(dport), dport, intf, vlan, evid))
        repln_group = ReplicationGroup(group_id, l2_encaps, l3_encaps)
        self.replication_groups[group_id] = repln_group

    def _parse_multicast_repln_groups(self):
        try:
            dispStr = self.bcm.run('multicast show')
        except IOError:
            raise RuntimeError('multicast show')
        entries = dispStr.split('Group')[1:]
        for entry in entries:
            self._parse_one_multicast_repln_group(entry)

    def _field_dict_build(self, entry):
        key = None
        fields = {}

        # format table[index]: <field1=val1,field2=val2>
        pat=re.compile('\[(?P<key>\d+)\]: \<(?P<data>\S+)\>')
        obj = pat.search(entry)
        if not obj:
            return key, fields

        data = obj.group('data')
        key = obj.group('key')

        for entry in data.split(','):
            entry=entry.split('=')
            if len(entry) > 1:
                fields[entry[0]]=entry[1]
        return key, fields

    def _update_portmap_port_vid(self, logical_port, port_vid):
        for pe in self.portmap:
            if pe.logical_port == logical_port:
                pe.update_port_vid(port_vid)

    def _dport2logical_port(self, dport):
        for pe in self.portmap:
            if pe.dport == dport:
                return pe.logical_port
        return -1

    def _get_logical_port_and_pvid(self, linux_port):
        for pe in self.portmap:
            if pe.linux_port == linux_port:
                return pe.logical_port, pe.port_vid
        return -1, -1

    def _is_port_untagged_for_vid(self, dport, vid):
        vlan_entry = self.vlans.get(vid)

        if not vlan_entry:
            self._parse_vlan(vid)
            vlan_entry = self.vlans.get(vid)

        if not vlan_entry:
            return False

        if dport in vlan_entry.untagged_dports:
            return True

        return False

    def _ivid2evid(self, dport, ivid):
        lport = self._dport2logical_port(dport)
        if lport == -1:
            return ivid

        for entry in self.vlan_xlate:
            if entry.lport == lport and entry.ivid == ivid:
                return entry.evid

        # no vlan xlate - check if port is an untagged member of ivid
        if self._is_port_untagged_for_vid(dport, ivid):
            return -1

        return ivid

    def _evid2ivid(self, logical_port, evid):
        for entry in self.vlan_xlate:
            if entry.lport == logical_port and entry.evid == evid:
                return entry.ivid
        return -1

    def _grp_ip2mac(self, grp):
        ip_arr=[int(elem) for elem in grp.split('.')]
        return '01:00:5e:%02x:%02x:%02x' % (ip_arr[1]&0x7f, ip_arr[2], ip_arr[3])

    def _kern_create_vlan_cache(self):
        self.kern_vlan_cache_created = True
        cmd = '/sbin/bridge -j vlan show'
        dispStr = self._get_cmd_output(cmd.split(), '{}')

        bridge_vlan = json.loads(dispStr)
        for dev in bridge_vlan:
            devInfo = bridge_vlan[dev]
            for vlanInfo in devInfo:
                kind = self._kern_dev_kind_get(dev)
                if kind == 'bridge':
                    continue
                flags = vlanInfo.get('flags', [])
                untagged = True if 'Egress Untagged' in flags else False
                vid = int(vlanInfo.get('vlan', 0))
                vlan_entry = self.kern_vlan_cache.get(vid)
                if not vlan_entry:
                    vlan_entry = KernVlanEntry(vid)
                    self.kern_vlan_cache[vid] = vlan_entry
                if kind == 'bond':
                    members = self._expand_kern_bond(dev)
                    for member in members:
                        vlan_entry.add_port(member, untagged)
                else:
                    vlan_entry.add_port(dev, untagged)

        '''
        print 'vlan-cache'
        for vlan in self.kern_vlan_cache:
            vlanInfo = self.kern_vlan_cache[vlan]
            print vlanInfo
        '''

    def _kern_get_vlan_members(self, vid):
        if not self.kern_vlan_cache_created:
            self._kern_create_vlan_cache()

        vlan_entry = self.kern_vlan_cache.get(vid)
        if not vlan_entry:
            return []
        return vlan_entry.tagged_swp + vlan_entry.untagged_swp

    def _kern_setup_br_cache(self, dev):
        file_path = '/sys/class/net/' + dev + '/brif'
        try:
            members = os.listdir(file_path)
        except:
            members = []

        new_members = []
        for member in members:
            kind, new_member, new_vlan = self._kern_base_dev_kind_get(member)
            if kind == 'bond':
                bond_members = self._expand_kern_bond(new_member)
                if new_vlan != -1:
                    bond_members = ['%s.%d'%(bond_member,new_vlan) for bond_member in bond_members]
                new_members += bond_members
            else:
                new_members.append(member)

        self.kern_br_cache[dev] = new_members
        return self.kern_br_cache[dev]

    def _kern_get_br_members(self, dev):
        if dev in self.kern_br_cache:
            return self.kern_br_cache[dev]
        return self._kern_setup_br_cache(dev)

    def _kern_dev_get_mdb_entry(self, orig_dev, orig_vlan, orig_grp):
        # here orig_grp is mac not ip
        orig_key = '%d.%s' % (orig_vlan, orig_grp)
        cache = self.kern_mdb_cache.get(orig_dev)
        if cache:
            return cache.mdb_entries.get(orig_key)

        # if mdb cache doesn't exist build it
        cmd = '/sbin/bridge mdb show dev %s' % orig_dev
        dispStr = self._get_cmd_output(cmd.split(), '')

        lines = dispStr.splitlines()
        vlan_filtering = self._kern_dev_vlan_filtering_get(orig_dev)
        cache = KernMdbCache(orig_dev, vlan_filtering)
        self.kern_mdb_cache[orig_dev] = cache

        if vlan_filtering:
            pat = re.compile("dev (?P<dev>\S+) port (?P<port>\S+) grp (?P<grp_ip>\S+) \S+ vid (?P<vid>\d+)")
        else:
            vid = -1
            pat = re.compile("dev (?P<dev>\S+) port (?P<port>\S+) grp (?P<grp_ip>\S+)")

        for line in lines:
            obj = pat.match(line)
            if obj:
                dev = obj.group('dev')
                if dev != orig_dev:
                    continue
                port = obj.group('port')
                grp_ip = obj.group('grp_ip')
                if vlan_filtering:
                    vid = int(obj.group('vid'))
                grp_mac = self._grp_ip2mac(grp_ip)
                key = '%d.%s' % (vid, grp_mac)
                entry = cache.mdb_entries.get(key)
                if not entry:
                    entry = KernMdbEntry(vid, grp_mac)
                    cache.mdb_entries[key] = entry
                kind, new_port, new_vlan = self._kern_base_dev_kind_get(port, -1)
                if kind == 'bond':
                    members = self._expand_kern_bond(new_port)
                    for member in members:
                        if new_vlan == -1:
                            entry.add_port(member)
                        else:
                            entry.add_port('%s.%d' % (member, new_vlan))
                else:
                    entry.add_port(port)
        '''
        print 'mdb-cache'
        print cache
        '''
        return cache.mdb_entries.get(orig_key)

    def _kern_dev_kind_parse(self, dev):
        if dev in self.linux_ports:
            kind = 'swp'
            return kind

        cmd = '/bin/ip -d link show %s' % dev
        dispStr = self._get_cmd_output(cmd.split(), '')

        lines = dispStr.splitlines()
        if not lines or len(lines) < 3 or 'does not exist' in lines[0]:
            return 'unk'

        kind = lines[2].split()[0]
        return kind

    def _kern_dev_vlan_filtering_parse(self, dev):
        # return the admin state as 1 (IFF_UP) or 0
        vlan_filtering = 0
        file_path = '/sys/class/net/' + dev + '/bridge/vlan_filtering'
        if os.path.isfile(file_path):
            try:
                for line in open(file_path):
                    vlan_filtering = int(line.strip(), 0)
            except IOError:
                pass
        return vlan_filtering

    def _kern_dev_cache_get(self, dev):
        if dev not in self.kern_dev_cache:
            kind = self._kern_dev_kind_parse(dev)
            vlan_filtering = self._kern_dev_vlan_filtering_parse(dev)
            kernDev = KernDevEntry(dev, kind, vlan_filtering)
            self.kern_dev_cache[dev] = kernDev
        return self.kern_dev_cache[dev]

    def _kern_dev_vlan_filtering_get(self, dev):
        kernDev = self._kern_dev_cache_get(dev)
        return kernDev.vlan_filtering

    def _kern_dev_kind_get(self, dev):
        kernDev = self._kern_dev_cache_get(dev)
        return kernDev.kind

    def _kern_get_dev_vlan(self, dev, kind, parent_vlan=-1):
        if kind == 'vlan':
            tmp = dev.split('.')
            vlan = int(tmp[len(tmp)-1])
            dev = '.'.join(tmp[:len(tmp) - 1])
        else:
            vlan = parent_vlan
        return dev, vlan

    def _kern_base_dev_kind_get(self, orig_dev, parent_vlan=-1):
        kind = self._kern_dev_kind_get(orig_dev)
        dev, vlan = self._kern_get_dev_vlan(orig_dev, kind, parent_vlan)
        if dev != orig_dev:
            #re-evaluate kind
            kind = self._kern_dev_kind_get(dev)
        return kind, dev, vlan

    def _expand_kern_vlan_aware_bridge(self, grp_mac, dev, vlan):
        entry = self._kern_dev_get_mdb_entry(dev, vlan, grp_mac)
        if entry:
            return entry.swp

        if self.flood_mdb_misses:
            return self._kern_get_vlan_members(vlan)

        return []

    def _expand_kern_vlan_unaware_bridge(self, grp_mac, dev):
        entry = self._kern_dev_get_mdb_entry(dev, -1, grp_mac)
        if entry:
            return entry.swp

        if self.flood_mdb_misses:
            return self._kern_get_br_members(dev)
        return []

    def _expand_kern_bridge(self, vlan_filtering, grp, dev, vlan):
        grp_mac = self._grp_ip2mac(grp)

        if vlan_filtering:
            return self._expand_kern_vlan_aware_bridge(grp_mac, dev, vlan)
        return self._expand_kern_vlan_unaware_bridge(grp_mac, dev)

    def _expand_kern_bond(self, dev):
        # returns a list of bond member names
        bondMembers = []
        bondPath = '/sys/class/net/%s' % dev + '/bonding/'
        if os.path.isdir(bondPath):
            try:
                for line in open(bondPath + 'slaves'):
                    bondMembers.extend(line.strip().split())
            except IOError:
                pass
        return bondMembers

    def _expand_kern_dev(self, grp, orig_dev, parent_vlan=-1):
        # expand the kernel OIL to a list of swpX.vid
        kind, dev, vlan = self._kern_base_dev_kind_get(orig_dev, parent_vlan)
        if not kind in self._devs_supported:
            return

        if kind == 'bridge':
            vlan_filtering = self._kern_dev_vlan_filtering_get(dev)
            members = self._expand_kern_bridge(vlan_filtering, grp, dev, vlan)
            for member in members:
                if vlan_filtering:
                    self.tmp_encaps.add('%s.%d' % (member, vlan))
                else:
                    self.tmp_encaps.add('%s' % member)
        elif kind == 'bond':
            members = self._expand_kern_bond(dev)
            for member in members:
                if vlan == -1:
                    self.tmp_encaps.add('%s' % member)
                else:
                    self.tmp_encaps.add('%s.%d' % (member, vlan))
        elif kind == 'swp':
            if vlan == -1:
                self.tmp_encaps.add('%s' % dev)
            else:
                self.tmp_encaps.add('%s.%d' % (dev, vlan))

    def _kern_iif_to_hw_iif(self, base_kind, base_iif, iif_vlan, member):
        if base_kind == 'bridge':
            if iif_vlan != -1:
                return iif_vlan

        if not member:
            return -1

        member_info = member.split('.')
        linux_port = member_info[0]
        iif_vlan = int(member_info[1]) if len(member_info) > 1 else -1
        logical_port, pvid = self._get_logical_port_and_pvid(linux_port)
        if iif_vlan == -1:
            return pvid

        return self._evid2ivid(logical_port, iif_vlan)

    def _dump_one_mroute(self, mroute):
        mroute_str = '%s\n' % mroute
        replication_group = self.replication_groups.get(mroute.grp)
        if replication_group:
            mroute_str += '%s' % replication_group
        return mroute_str

    def mroute_eq(self, key, iif, oil):
        src, grp = self._key_to_sip_dip(key)
        # return True if entries match
        hmroute = self.mroutes.get(key)
        if not hmroute:
            return True

        # get kernel l2 encaps from IIF
        kern_l2_encaps = set()
        kind, new_iif, iif_vlan = self._kern_base_dev_kind_get(iif)
        self.tmp_encaps = set()
        self._expand_kern_dev(grp, iif)
        if kind == 'bridge':
            kern_l2_encaps = set([encap.split('.')[0] for encap in self.tmp_encaps])

        # translate the kern IIF to hw L3_IIF
        first_member = self.tmp_encaps.pop() if len(self.tmp_encaps) > 0 else ''
        kern_iif_vlan = self._kern_iif_to_hw_iif(kind, new_iif, iif_vlan, first_member)
        #print 'kern mroute %s iif %s/%s iif_vlan %d' % (key, iif, first_member, kern_iif_vlan)
        #print 'hw mroute %s iif_vlan %d' % (key, hmroute.iif)
        if kern_iif_vlan != hmroute.iif:
            hmroute.iif_mismatch.append('kernel IIF does not match hw IIF')

        # get kernel l3 encaps from OIL
        kern_l3_encaps = set()
        for oif in oil:
            if oif == iif:
                continue
            self.tmp_encaps = set()
            self._expand_kern_dev(grp, oif)
            kern_l3_encaps |= self.tmp_encaps

        replication_group = self.replication_groups.get(hmroute.grp)
        hw_l3_encaps = replication_group.get_l3_linux_encaps() if replication_group else set()
        hw_l2_encaps = replication_group.get_l2_linux_encaps() if replication_group else set()
        #print 'kern mroute %s l3-encaps %s l2-encaps %s' % (key, kern_l3_encaps, kern_l2_encaps)
        #print 'hw mroute %s l3-encaps %s l2-encaps %s' % (key, hw_l3_encaps, hw_l2_encaps)
        diff = hw_l3_encaps - kern_l3_encaps
        if diff:
            hmroute.oil_mismatch.append('hw has OIFs not in the kernel')
        diff = kern_l3_encaps - hw_l3_encaps
        if diff:
            hmroute.oil_mismatch.append('kernel has OIFs not in the hw')

        diff = hw_l2_encaps - kern_l2_encaps
        if diff:
            hmroute.oil_mismatch.append('hw has L2-OIFs not in the kernel')
        diff = kern_l2_encaps - hw_l2_encaps
        if diff:
            hmroute.oil_mismatch.append('kernel has L2-OIFs not in the hw')

        if hmroute.iif_mismatch or hmroute.oil_mismatch:
            return False
        return True

    def get_mroutes(self):
        # return mroute keys as a set
        return set(self.mroutes)

    def dump_mroutes(self, filter_mroutes=None):
        if not filter_mroutes:
            for key in sorted(self.mroutes):
                sys.stdout.write('%s' % self._dump_one_mroute(self.mroutes[key]))
            return

        for key in filter_mroutes:
            mroute = self.mroutes.get(key)
            if mroute:
                sys.stdout.write('%s' % self._dump_one_mroute(mroute))
