#!/usr/bin/env python
# Copyright 2016 Cumulus Networks, Inc.
#
# mroute-check kernel helper

import re
import os
import subprocess
import sys

import cumulus.porttab

class Kmroute:
    def __init__(self, sip, dip, iif, oil, orig_str):
        self.sip = sip
        self.dip = dip
        self.iif = iif
        self.oil = oil
        self.orig_str = orig_str

    def __str__(self):
        return self.orig_str

class Kernel:
    _devs_supported = ('bridge', 'bond', 'vlan')

    def __init__(self, asic_present):
        self.desc = "kernel"
        self.asic_present = asic_present
        if asic_present:
            self.pt = cumulus.porttab.porttab()
            self.linux_ports = self.pt.get_linux_ports()
        # key: devName info: link_kind
        self.dev_kind = {}

        # pull the kernel mroute table
        # dict key: '(1.1.1.1, 239.1.1.1)', info: Kmroute
        self.mroutes = {}
        # entries that cannot be added to the hw are maintained in
        # a black list. this is needed to match up with quagga.
        self.blist_mroutes = {}
        self.unresolved_mroutes = {}
        self._mroute_parse_iproute2_output()

    def __str__(self):
        ipmr_str = 'Kernel ip mroutes:\n'
        for key in sorted(self.mroutes):
            ipmr_str += '%s\n' % self.mroutes[key]
        if self.blist_mroutes:
            ipmr_str += 'Kernel ip mroutes (skip-hw):\n'
            for key in sorted(self.blist_mroutes):
                ipmr_str += '%s\n' % self.blist_mroutes[key]
        if self.unresolved_mroutes:
            ipmr_str += 'Kernel ip mroutes (unresolved):\n'
            for key in sorted(self.unresolved_mroutes):
                ipmr_str += '%s\n' % self.unresolved_mroutes[key]
        return ipmr_str

    @staticmethod
    def _get_cmd_output(cmdList, dispStr=None):
        try:
            with open(os.devnull, 'w') as fnull:
                dispStr = subprocess.check_output(cmdList, stderr=fnull)
        except subprocess.CalledProcessError:
            pass
        return dispStr

    def _mroute_install_hw_dev_ok(self, dev):
        if dev in self.linux_ports:
            return True

        # check if the device is in the cache
        kind = self.dev_kind.get(dev)
        if not kind:
            # if not in cache get and cache it
            cmd = '/bin/ip -d link show %s' % dev
            dispStr = self._get_cmd_output(cmd.split(), '')

            lines = dispStr.splitlines()
            if not lines or len(lines) < 3 or 'does not exist' in lines[0]:
                return False

            kind = lines[2].split()[0]
            self.dev_kind[dev] = kind

        return True if kind in self._devs_supported else False

    def _mroute_install_hw_ok(self, mroute):
        if not self.asic_present:
            # no chip inside; nothing to blacklist
            return True

        if not self._mroute_install_hw_dev_ok(mroute.iif):
            return False

        for oif in mroute.oil:
            if not self._mroute_install_hw_dev_ok(oif):
                return False

        return True

    def _mroute_key2sip_and_dip(self, key):
        pat=re.compile('\((?P<sip>\S+), (?P<dip>\S+)\)')
        obj = pat.match(key)
        if obj:
            return obj.group('sip'), obj.group('dip')
        return None, None

    def _mroute_parse_iproute2_output(self):
        cmd = '/bin/ip mroute show'
        dispStr = self._get_cmd_output(cmd.split(), '')

        # sample format (two possible patterns) -
        # (33.1.1.1, 239.1.1.1)            Iif: swp1       Oifs: swp2 swp4  State: resolved
        # (33.1.1.1, 239.1.1.1)            Iif: swp1       State: resolved
        pat1 = re.compile('(?P<key>.*)Iif: (?P<iif>\S+)\s+Oifs: (?P<oil>.*)\s+State: (?P<state>\S+)')
        pat2 = re.compile('(?P<key>.*)Iif: (?P<iif>\S+)\s+State: (?P<state>\S+)')
        for line in dispStr.splitlines():
            if 'Oifs' in line:
                pat = pat1
                oilPresent = True
            else:
                pat = pat2
                oilPresent = False
            obj = pat.match(line)
            if not obj:
                continue

            key = obj.group('key').strip()
            sip, dip = self._mroute_key2sip_and_dip(key)
            iif = obj.group('iif')
            oil = obj.group('oil').split() if oilPresent else []
            state = obj.group('state')

            mroute = Kmroute(sip, dip, iif, oil, line)
            # unresolved cache entries are local to the kernel
            if state == 'unresolved':
                self.unresolved_mroutes[key] = mroute
                continue
            # mroutes that cannot be added to the hw are black listed
            if self._mroute_install_hw_ok(mroute):
                self.mroutes[key] = mroute
            else:
                self.blist_mroutes[key] = mroute

    def get_mroutes(self):
        # return a set of mroute keys
        return set(self.mroutes)

    def get_blist_mroutes(self):
        # return a set of mroute keys
        return set(self.blist_mroutes)

    def find_mroute(self, key):
        mroute = self.mroutes.get(key)
        if not mroute:
            # look in the black list
            mroute = self.blist_mroutes.get(key)
        return mroute

    def get_mroute_iif_and_oil(self, key):
        mroute = self.find_mroute(key)
        if mroute:
            return mroute.iif, mroute.oil
        return None, None

    def dump_mroutes(self, filter_mroutes=None):
        if not filter_mroutes:
            for key in sorted(self.mroutes):
                sys.stdout.write('%s\n' % self.mroutes[key])
            for key in sorted(self.blist_mroutes):
                sys.stdout.write('%s\n' % self.blist_mroutes[key])
            return

        for key in filter_mroutes:
            mroute = self.find_mroute(key)
            if mroute:
                sys.stdout.write('%s\n' % mroute)
