#!/usr/bin/env python
# Copyright 2016 Cumulus Networks, Inc.
#
# mroute-check quagga helper

import json
import os
import subprocess
import sys

class Qmroute:
    def __init__(self, sip, dip, iif, oil):
        self.sip = sip
        self.dip = dip
        self.iif = iif
        self.oil = oil
        self.mismatch = []

    def __str__(self):
        dispStr = '(%s, %s) iif: %s' % (self.sip, self.dip, self.iif)
        if self.oil:
            dispStr += ' oil:%s' % ' '.join(self.oil)
        if self.mismatch:
            dispStr += '\n  errors:'
        for mismatch in self.mismatch:
            dispStr += '\n    %s' % mismatch
        return dispStr

class Quagga:
    def __init__(self):
        # pull the quagga mroute table
        # dict key: '(1.1.1.1, 239.1.1.1)', info: Qmroute
        self.desc = "quagga"
        self.mroutes = {}
        self._mroute_parse()

    def __str__(self):
        ipmr_str = 'Quagga ip mroutes:\n'
        for key in sorted(self.mroutes):
            ipmr_str += '%s\n' % self.mroutes[key]
        return ipmr_str

    @staticmethod
    def _get_cmd_output(cmdList, dispStr=None):
        try:
            with open(os.devnull, 'w') as fnull:
                dispStr = subprocess.check_output(cmdList, stderr=fnull)
        except subprocess.CalledProcessError:
            dispStr = ""
        return dispStr

    def _mroute_parse(self):
        dispStr = self._get_cmd_output(['/usr/bin/vtysh', '-c', 'show ip mroute json'], '')

        # sample format -
        #{
        #  "239.1.1.1":{
        #    "33.1.1.1":{
        #      "installed":1,
        #      "refCount":2,
        #      "oilSize":1,
        #      "OilInheritedRescan":0,
        #      "iif":"swp2s0",
        #      "oil":{
        #        "swp2s1":{
        #          >>> SNIP >>>
        #          "ttl":1,
        #        }
        #      }
        #    }
        #  }
        #}
        ipmr = json.loads(dispStr)
        for dip in ipmr:
            dip_info = ipmr[dip]
            for sip in dip_info:
                sip_info = dip_info[sip]
                if not sip_info.get('installed', 0):
                    continue
                iif = sip_info.get('iif', 'unk')
                oil_dict = sip_info.get('oil', {})
                # XXX: ideally we should use iVifI and oVifI that would also
                # validate iface info falling out of sync
                oil = []
                for oif in oil_dict:
                    oil.append(oif)
                if sip == '*':
                    sip = '0.0.0.0'
                mroute = Qmroute(sip, dip, iif, oil)
                key = '(%s, %s)' % (sip, dip)
                self.mroutes[key] = mroute

    def get_mroutes(self):
        # return a set of mroute keys
        return set(self.mroutes)

    def get_mroute_iif_and_oil(self, key):
        mroute = self.mroutes.get(key)
        if mroute:
            return mroute.iif, mroute.oil
        return None, None

    def mroute_eq(self, key, iif, oil):
        mroute = self.mroutes.get(key)
        if not mroute:
            return True

        if mroute.iif != iif:
            mroute.mismatch.append('kernel IIF does not match quagga IIF')
        qoil = set(mroute.oil)
        koil = set(oil)
        # the kernel requires that iif be partof oil for (*,G)
        if iif in koil:
            koil.remove(iif)
        # quagga can also set iif to be in oil for (*,G)
        if mroute.iif in qoil:
            qoil.remove(mroute.iif)
        diff = qoil - koil
        if diff:
            mroute.mismatch.append('quagga has OIFs not in the kernel')
        diff = koil - qoil
        if diff:
            mroute.mismatch.append('kernel has OIFs not in quagga')

        if mroute.mismatch:
            return False
        return True

    def dump_mroutes(self, filter_mroutes=None):
        if not filter_mroutes:
            for key in sorted(self.mroutes):
                sys.stdout.write('%s\n' % self.mroutes[key])
            return

        for key in filter_mroutes:
            mroute = self.mroutes.get(key)
            if mroute:
                sys.stdout.write('%s\n' % mroute)
