#
# Copyright 2012.  Cumulus Networks, Inc.
#
# platform.py --
#     Shared platform-specific objects.  Typically used by specific switch
#     platforms defined in cumulus.platforms.
#

import os

class NoSuchPlatform(RuntimeError):
    pass
class NoSuchChip(RuntimeError):
    pass

class Platform:
    cpld = None
    switch = None

class SDKSwitch:
    '''
    SDK platform common attributes

    has_phy_ucode_spi_rom - true if platform supports SPI-ROM for phy microcode
    has_phy_ucode_mdio    - true if platform supports MDIO for phy microcode
    led_controllers       - Specify the set of led controllers to populate.
    '''

    has_phy_ucode_spi_rom = None
    has_phy_ucode_mdio = None
    led_controllers = None

    def __init__(self, chip) :
        self.chip = chip

class BCMSwitch(SDKSwitch):
    '''
    Broadcom switch platform
    '''
    def __init__(self, chip) :
        SDKSwitch.__init__(self, chip)

class NoSuchMode(RuntimeError):
    pass

class Port(object):
    '''
    A physical switch port.  Typically on the front panel with one physical
    text label per port.

    label - the text written next to the physical port
    '''
    is_fabric = False
    create_fabric = False
    def __init__(self, label):
        self.label = label
        self.sfp = None

class SDKPort(Port):
    '''
    SDK port data.

    modes - tuple of valid modes (e.g. 4x10G, 40G/4)
    default_mode - the default mode
    gang_modes - a tuple of supported ganged port modes (e.g. 40G/4 or 100G/4)
    max_gang_size - maximum number of ports in a ganged group
    pbmp_map - a map from hardware interface name to SDK pbmp position (e.g. "hg0 -> 1")
    phy_init - a map of register to values used to initalize PHYs
    '''
    modes = ()
    default_mode = None
    gang_modes = ()
    max_gang_size = 0
    pbmp_map = {}
    phy_init = {}
    _speed_map = {}
    # phys which are automatically detectable by some SDK
    _autodetect_phys = ('8727',
                        '8747',
                        '8754',
                        '54282',
                        '54680',
                        '54280',
                        '84328',
                        '84758',
                        '84848',
                        '84858')
    disable_prefix_xlate = None

    # Serdes Interface map is dictionary of (Port speed, Interface Mode)
    # which is platform specific
    # Ex: 10:'SFI', 40: 'XLAUI'
    _serdes_intf_map = {}

    def __init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer, prim_offset,
                 leds, unit, rx_polarity_flip, tx_polarity_flip,
                 rx_lane_map, tx_lane_map, mdi_pair_map, pre_emphasis, drv_current,
                 pre_drv_current, oversub, phy_id1, sys_interface):
        '''
        hw_ports - An ordered list of hardware interfaces associated with the port.
                   In this context, ports are identified by their unmapped
                   interface name as in the SDK chip datasheets (For example,
                   the "Ports Summary" on page 143 of 56840-PR101-R).  This usually
                   means 10G interfaces are specified using their "hg" prefix.

        phy_types - An ordered list of PHY device types associated with the port.
        phy_addrs - An ordered list of PHY addresses associated with the port.
        phy_layer - A description of the PHY layer, one of: '10GBaseT',
                      'SFP+', 'QSSFP+', '10BaseT'
        leds - An ordered list of led chain addresses associated with the port.
               (controller, chain position)
        unit - In multi-chip systems, the chip number, "unit" in SDK terms.
        rx_polarity_flip - flip the rx polarity in hardware
                               (True/False for single serdes ports, SDK-style
                                4-digit hex number for multi-serdes ports)
        tx_polarity_flip - flip the tx polarity in hardware
        rx_lane_map - controls lane swapping
        tx_lane_map - controls lane swapping
        mdi_pair_map - mapping of physical pairs in an MDI interface
        pre_emphasis - per-lane pre-emphasis (Ref: Table7 56840-AN603-R)
        drv_current - per-lane driver current (Ref: Table7 56840-AN603-R)
        pre_drv_current - per-lane pre-driver current (Ref: Table7 56840-AN603-R)
        oversub  - a bool whether the port bandwidth is over-subscribed
        phy_id1 - Second part of a uniqe PHY identifier
        sys_interface - Specify the sytem side interface to be configured for the PHY
        '''
        Port.__init__(self, label)
        self.hw_ports = hw_ports
        self.prim_offset = prim_offset
        self.phy_types = phy_types
        self.phy_addrs = phy_addrs
        self.phy_layer = phy_layer
        self.leds = leds
        self.unit = unit
        self.rx_polarity_flip = rx_polarity_flip
        self.tx_polarity_flip = tx_polarity_flip
        self.rx_lane_map = rx_lane_map
        self.tx_lane_map = tx_lane_map
        self.mdi_pair_map = mdi_pair_map
        self.pre_emphasis = pre_emphasis
        self.drv_current = drv_current
        self.pre_drv_current = pre_drv_current
        self.oversub = oversub
        self.phy_id1 = phy_id1
        self.sys_interface = sys_interface

    def configurable(self, config_map):
        '''
        A ports is SDK configurable if it's not marked as 'disable'.
        Some platforms may add further qualifications by overriding this method.
        '''
        mode = config_map.get(self.label, self.default_mode)
        return not mode.lower().startswith('disable')

    def phy_autodetect(self, mode, sub):
        return self.phy_types[sub] in self._autodetect_phys

    @classmethod
    def _check_mode(self, mode):
        '''
        Verify the mode specified in the config file is valid.
        '''
        if mode not in self.modes:
            shorthand_modes = set()
            for m in self.modes:
                if 'x' in m:
                    shorthand_modes.add('1x')
                    shorthand_modes.add('/1')
                    shorthand_modes.add(m[:2])

                if '/' in m:
                    shorthand_modes.add('1x')
                    shorthand_modes.add('/1')
                    shorthand_modes.add(m[-2:])
            raise NoSuchMode('error: mode not found: %s, must be one of: %s' %
                             (mode, str(list(shorthand_modes) + list(self.modes))))

    @classmethod
    def speed(self, mode):
        '''
        Return the port speed in gigabits per second for a given mode.
        '''
        self._check_mode(mode)
        return self._speed_map[mode]

    def num_physical_ports(self, mode):
        '''
        Return the number of physical (front panel holes) for a given port.
        '''
        return self.num_logical_ports(mode)

    def num_logical_ports(self, mode):
        '''
        Return the number of logical (Linux interfaces) for a given port.
        '''
        raise NotImplementedError

    @property
    def max_logical_ports(self):
        '''
        Return the maximum number of logical (Linux interfaces) for a given port.
        '''
        raise NotImplementedError

    def mac_addrs_contrib_total(self, config_map):
        '''
        Return the number of MAC addresses this port would contribute to the
        system total.
        '''
        if self.configurable(config_map) and (not self.is_fabric or self.create_fabric):
            return self.max_logical_ports
        return 0

    @property
    def num_phys(self):
        '''
        Return the number of phys for a given port.
        '''
        if self.phy_addrs is not None:
            return len(filter(lambda x: x is not None, self.phy_addrs))
        else:
            return 0

    def hw_intf_prefix(self, mode, sub):
        '''
        Return the physical port interface prefix (always unmapped by portmap)
        '''
        return self.hw_ports[sub][:2]

    def hw_intf_num(self, mode, sub):
        '''
        Return the physical port interface number (always unmapped by portmap)
        '''
        if self.num_logical_ports(mode) == 2:
            return int(self.hw_ports[sub * 2][2:])
        else:
            return int(self.hw_ports[sub][2:])

    def base_hw_intf_num(self, mode, sub):
        '''
        Return the lowest numbered hw_intf_num that makes up a logical port.
        '''
        lports = self.num_logical_ports(mode)
        hports = len(self.hw_ports)
        if hports > lports:
            # the interface uses of multiple serdes
            return min([ self.hw_intf_num(mode, x) for x in range(hports) ])
        else:
            return self.hw_intf_num(mode, sub)

    def hw_intf(self, mode, sub):
        '''
        Return the physical hardware port for the sub-interface (prefix
        concatenated with number).
        '''
        return '%s%u' % (self.hw_intf_prefix(mode, sub), self.hw_intf_num(mode, sub))

    def hw_pbmp(self, mode, sub):
        '''
        Return the hardware port bitmap mask for the sub-interface.
        '''
        return 1 << self.pbmp_map[self.hw_ports[sub]]

    def hw_port_num(self, mode, sub):
        '''
        Return the hardware port number (pbmp bit position)
        '''
        return self.pbmp_map[self.hw_ports[sub]]

    def port_mac_index(self, mode, mac_index, sub):
        '''
        Return the MAC address index for the sub-interface
        '''
        return mac_index + sub

    def led_port_num(self, mode, sub):
        '''
        Return the led port number (identical to the hw_port_num on most platforms)
        '''
        return self.pbmp_map[self.hw_ports[sub]]

    def set_leds(self, mode, led_port, sub, led_index_list):
        return {}

    def polarity_flip_val(self, mode, sub, polarity_flip):
        '''
        Return the generic TX/RX polarity_flip parameter for the
        interface as an integer.
        '''
        raise NotImplementedError

    def rx_logical_polarity_flip(self, mode, sub):
        '''
        Return the rx_polarity_flip parameter for the interface
        as an integer.
        '''
        return self.polarity_flip_val(mode, sub, self.rx_polarity_flip)

    def tx_logical_polarity_flip(self, mode, sub):
        '''
        Return the rx_polarity_flip parameter for the interface
        as an integer.
        '''
        return self.polarity_flip_val(mode, sub, self.tx_polarity_flip)

    def serdes_if_type(self, speed):
        '''
        Return the serdes interface type based on the port speed
        '''
        return self._serdes_intf_map.get(speed, None)

    def an_c37(self, speed):
        return None

    def an_c73(self, speed):
        return None

    def logical_port_num(self, mode, sub):
        '''
        Return the logical port number for the sub-interface
        '''
        raise NotImplementedError

class BCMPort(SDKPort):
    '''
    Broadcom-specific port data.
    '''
    def __init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer, prim_offset,
                 leds, unit, rx_polarity_flip, tx_polarity_flip,
                 rx_lane_map, tx_lane_map, mdi_pair_map, pre_emphasis, drv_current,
                 pre_drv_current, oversub, phy_id1, sys_interface):
        SDKPort.__init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer, prim_offset,
                 leds, unit, rx_polarity_flip, tx_polarity_flip,
                 rx_lane_map, tx_lane_map, mdi_pair_map, pre_emphasis, drv_current,
                 pre_drv_current, oversub, phy_id1, sys_interface)

class TomahawkPort(BCMPort):
    pbmp_map = dict([ (v,k) for k,v in enumerate(
        [ 'ce%u' % x for x in range(0, 132) ], 1) ])

    def __init__(self, label, serdes, phy_types, phy_addrs, phy_layer,
                 prim_offset, leds, unit=0,
                 rx_polarity_flip=None, tx_polarity_flip=None,
                 rx_lane_map=None, tx_lane_map=None, pre_emphasis=None,
                 drv_current=None, pre_drv_current=None, oversub=False,
                 phy_id1=None, sys_interface=None, mdi_pair_map = None):

        hw_port = tuple([ 'ce%u' % x for x in serdes ])

        BCMPort.__init__(self, label, hw_port, phy_types, phy_addrs,
                         phy_layer, prim_offset, leds, unit,
                         rx_polarity_flip, tx_polarity_flip,
                         rx_lane_map, tx_lane_map, mdi_pair_map, pre_emphasis,
                         drv_current, pre_drv_current, oversub, phy_id1,
                         sys_interface)

    def led_port_num(self, mode, sub):
        hw_port = self.hw_intf_num(mode, sub)
        # 0 - 31, 96 - 127 are one chain0
        # 32 - 95 are chain1
        # 128, 130 (mgmt) are chain2
        if hw_port == 128:
            hw_port = 0
        elif hw_port == 130:
            hw_port = 1
        elif hw_port > 31:
            if hw_port < 96:
                hw_port = hw_port - 32
            else:
                hw_port -= 64
        return hw_port

    def hw_port_num(self, mode, sub):
        return int(self.hw_ports[sub][2:])

    def hw_intf_prefix(self, mode, sub):
        if mode in ('100G', '100G/4'):
            return 'ce'
        return 'xe'

    def logical_port_num(self, mode, sub, port_base):
        lport = self.hw_port_num(mode, sub) + 1
        if lport == 129:
            # mgmt0
            lport = 66
        elif lport == 131:
            # mgmt1
            lport = 100
        else:
            # 33 is lb0
            if lport > 32:
                lport += 1
            # 66 is mgmt0
            # 67 is lb1
            if lport > 65:
                lport += 2
            # 100 is mgmt1
            # 101 is lb2
            if lport > 99:
                lport += 2
        return lport

class TridentTwoPort(BCMPort):
    pbmp_map = dict([ (v,k) for k,v in enumerate(
                        [ 'hg%u' % x for x in range(0, 128) ], 1) ])

    def led_port_num(self, mode, sub):
        raise RuntimeError('Trident 2 LED port unknown')

    def __init__(self, label, serdes, phy_types, phy_addrs, phy_layer,
                 prim_offset, leds, unit=0,
                 rx_polarity_flip=None, tx_polarity_flip=None,
                 rx_lane_map=None, tx_lane_map=None, pre_emphasis=None,
                 drv_current=None, pre_drv_current=None, oversub=False,
                 phy_id1=None, sys_interface=None, mdi_pair_map = None):
        hw_port = tuple([ 'hg%u' % x for x in serdes ])
        BCMPort.__init__(self, label, hw_port, phy_types, phy_addrs,
                         phy_layer, prim_offset, leds, unit,
                         rx_polarity_flip, tx_polarity_flip,
                         rx_lane_map, tx_lane_map, mdi_pair_map, pre_emphasis,
                         drv_current, pre_drv_current, oversub, phy_id1,
                         sys_interface)

    # T2 has a hardware limitation where it can't use a single setting for
    # clause 37 and clause 73 autoneg.
    #
    # It also requires some extra phy programming to allow 10G and 1G
    # interfaces with autoneg to coexist on the same core.
    #
    # Therefore, for 10G modes advertise clause 37 autoneg, turn off clause 73
    # autoneg, and use 'auto' to enable the special 1G/10G phy programming.
    #
    # https://support.broadcom.com/KMS/Main.aspx?id=HT2267
    def an_c37(self, mode):
        if self.phy_layer in ('10GBaseT', '1GBaseT'):
            return None

        if mode in ('10G', '4x10G'):
            return 'auto'

        return None

    def an_c73(self, mode):
        if self.phy_layer in ('10GBaseT', '1GBaseT'):
            return None

        if mode in ('10G', '4x10G'):
            return 'off'

        return None

class TridentTwoPlusPort(TridentTwoPort):
    def an_c37(self, mode):
        if self.phy_layer in ('10GBaseT', '1GBaseT'):
            return None

        if mode in ('10G', '4x10G'):
            return 'on'

        return None

    def logical_port_num(self, mode, sub, port_base):
        return port_base + sub

class TridentPort(BCMPort):
    pbmp_map = dict([ (v,k) for k,v in enumerate(
                        [ 'hg%u' % x for x in range(0, 72) ], 1) ])

    def __init__(self, label, serdes, phy_types, phy_addrs,
                 phy_layer, prim_offset, leds, unit=0,
                 rx_polarity_flip=None, tx_polarity_flip=None,
                 rx_lane_map=None, tx_lane_map=None, mdi_pair_map=None):
        hw_port = tuple([ 'hg%u' % x for x in serdes ])
        BCMPort.__init__(self, label, hw_port, phy_types, phy_addrs,
                         phy_layer, prim_offset, leds, unit,
                         rx_polarity_flip, tx_polarity_flip,
                         rx_lane_map, tx_lane_map, mdi_pair_map, pre_emphasis=None,
                         drv_current=None, pre_drv_current=None,
                         oversub=False, phy_id1=None, sys_interface=None)

class TriumphPort(BCMPort):
    pbmp_map = dict([ (v,k) for k,v in enumerate(
        ('ge48', 'ge8',  'ge9',  'ge10', 'ge11', 'ge12',
         'ge13', 'ge14', 'ge15', 'ge20', 'ge21', 'ge22',
         'ge23', 'ge24', 'ge25', 'ge26', 'ge27', 'ge32',
         'ge33', 'ge34', 'ge35', 'ge36', 'ge37', 'ge38',
         'ge39', 'hg0',  'hg1',  'hg2',  'hg3',  'ge0',
         'ge1',  'ge2',  'ge3',  'ge4',  'ge5',  'ge6',
         'ge7',  'ge16', 'ge17', 'ge18', 'ge19', 'ge28',
         'ge29', 'ge30', 'ge31', 'ge40', 'ge41', 'ge42',
         'ge43', 'ge44', 'ge45', 'ge46', 'ge47'), 1) ])

    def __init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer, prim_offset, leds, unit=0,
                    rx_polarity_flip=None, tx_polarity_flip=None,
                    rx_lane_map=None, tx_lane_map=None, mdi_pair_map=None):
        BCMPort.__init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer,
                         prim_offset, leds, unit, rx_polarity_flip=None, tx_polarity_flip=None,
                         rx_lane_map=None, tx_lane_map=None, mdi_pair_map=None,
                         pre_emphasis=None, drv_current=None,
                         pre_drv_current=None, oversub=False,
                         phy_id1=None, sys_interface=None)


class ApolloPort(BCMPort):
    def hw_port_num(self, mode, sub):
        num = self.hw_intf_num(mode, sub)
        prefix = self.hw_intf_prefix(mode, sub)

        # this platform has a hole at 36 and hg interfaces start at index 50
        if prefix == 'ge':
            if num > 35:
                return num + 2
            else:
                return num + 1
        elif prefix == 'hg':
            return num + 50
        else:
            raise RuntimeError('bad interface prefix')

    def hw_pbmp(self, mode, sub):
        return 1 << self.hw_port_num(mode, sub)

    def led_port_num(self, mode, sub):
        num = self.hw_intf_num(mode, sub)
        prefix = self.hw_intf_prefix(mode, sub)

        # The led data array has no holes, hg starts at element 73 (on the 2nd
        # led controller)
        if prefix == 'ge':
            return num + 1
        elif prefix == 'hg':
            return num + 73
        else:
            raise RuntimeError('bad interface prefix')

    def __init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer, prim_offset, leds, unit):
        BCMPort.__init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer, prim_offset,
                         leds, unit, rx_polarity_flip=None, tx_polarity_flip=None,
                         rx_lane_map=None, tx_lane_map=None, mdi_pair_map=None,
                         pre_emphasis=None, drv_current=None,
                         pre_drv_current=None, oversub=False,
                         phy_id1=None, sys_interface=None)

class Helix4Port(BCMPort):
    pbmp_map = dict([ (v,k) for k,v in enumerate(
        ('ge0',  'ge1',  'ge2',  'ge3',  'ge4',  'ge5',
         'ge6',  'ge7',  'ge8',  'ge9',  'ge10', 'ge11',
         'ge12', 'ge13', 'ge14', 'ge15', 'ge16', 'ge17',
         'ge18', 'ge19', 'ge20', 'ge21', 'ge22', 'ge23',
         'ge24', 'ge25', 'ge26', 'ge27', 'ge28', 'ge29',
         'ge30', 'ge31', 'ge32', 'ge33', 'ge34', 'ge35',
         'ge36', 'ge37', 'ge38', 'ge39', 'ge40', 'ge41',
         'ge42', 'ge43', 'ge44', 'ge45', 'ge46', 'ge47',
         'ge48', 'hg0',  'hg1',  'hg2',  'hg3',  'hg4',
         'hg5'), 1) ])
    def led_port_num(self, mode, sub):
        num = self.hw_intf_num(mode, sub)
        prefix = self.hw_intf_prefix(mode, sub)

        # The led data array has no holes, hg starts at element 49
        if prefix == 'ge':
            return num + 1
        elif prefix == 'hg':
            return num + 49
        else:
            raise RuntimeError('bad interface prefix')

    def __init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer, prim_offset, leds, unit=0,
                    rx_polarity_flip=None, tx_polarity_flip=None,
                    rx_lane_map=None, tx_lane_map=None, mdi_pair_map=None):
        BCMPort.__init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer,
                         prim_offset, leds, unit, rx_polarity_flip, tx_polarity_flip,
                         rx_lane_map, tx_lane_map, mdi_pair_map,
                         pre_emphasis=None, drv_current=None,
                         pre_drv_current=None, oversub=False,
                         phy_id1=None, sys_interface=None)

class Hurricane2Port(BCMPort):

    def hw_port_num(self, mode, sub):
        num = self.hw_intf_num(mode, sub)
        prefix = self.hw_intf_prefix(mode, sub)

        # this platform has a ge ports  at 2 and hg interfaces start at index 27
        if prefix == 'ge':
            return num + 2
        elif prefix == 'xe':
            return num + 26
        elif prefix == 'hg':
            return num + 28
        else:
            raise RuntimeError('bad interface prefix')

    def led_port_num(self, mode, sub):
        num = self.hw_intf_num(mode, sub)
        prefix = self.hw_intf_prefix(mode, sub)
        # this platform has a ge ports  at 2 and hg interfaces start at index 27
        if prefix == 'ge':
            return num + 2
        elif prefix == 'xe':
            return (num*2) + 26
        else:
            raise RuntimeError('bad interface prefix')

    def hw_pbmp(self, mode, sub):
        return 1 << self.hw_port_num(mode, sub)

    #pbmp_map = dict([ (v,k) for k,v in enumerate(
    #    ('ge0',  'ge1',  'ge2',  'ge3',  'ge4',  'ge5',
    #     'ge6',  'ge7',  'ge8',  'ge9',  'ge10', 'ge11',
    #     'ge12', 'ge13', 'ge14', 'ge15', 'ge16', 'ge17',
    #     'ge18', 'ge19', 'ge20', 'ge21', 'ge22', 'ge23',
    #     'xe0', 'xe1', 'hg0', 'hg1'), 1) ])

    def __init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer, prim_offset, leds, unit=0,
                    rx_polarity_flip=None, tx_polarity_flip=None,
                    rx_lane_map=None, tx_lane_map=None, mdi_pair_map=None):
        BCMPort.__init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer,
                         prim_offset, leds, unit, rx_polarity_flip, tx_polarity_flip,
                         rx_lane_map, tx_lane_map, mdi_pair_map,
                         pre_emphasis=None, drv_current=None,
                         pre_drv_current=None, oversub=False,
                         phy_id1=None, sys_interface=None)


class Apollo1GPort(ApolloPort):
    modes = ('1G')
    default_mode = modes[0]

    def __init__(self, label, hw_port, phy_type, phy_addr, leds, unit, prim_offset=None):
        ApolloPort.__init__(self,
                            label=label,
                            hw_ports=(hw_port,),
                            phy_types=(phy_type,),
                            phy_addrs=(phy_addr,),
                            phy_layer='1GBaseT',
                            prim_offset=prim_offset,
                            leds=leds,
                            unit=unit)

    def speed(self, mode):
        return 1

    def num_logical_ports(self, mode):
        return 1

    @property
    def max_logical_ports(self):
        return 1

class Triumph1GPort(TriumphPort):
    modes = ('1G')
    default_mode = modes[0]

    def __init__(self, label, hw_port, phy_type, phy_addr, phy_layer, leds, unit, prim_offset=None):
        TriumphPort.__init__(self,
                             label=label,
                             hw_ports=(hw_port,),
                             phy_types=(phy_type,),
                             phy_addrs=(phy_addr,),
                             phy_layer='1GBaseT',
                             prim_offset=prim_offset,
                             leds=leds,
                             unit=unit)

    def speed(self, mode):
        return 1

    def num_logical_ports(self, mode):
        return 1

    @property
    def max_logical_ports(self):
        return 1

class Helix41GPort(Helix4Port):
    modes = ('1G')
    default_mode = modes[0]

    def __init__(self, label, hw_port, phy_type, phy_addr, phy_layer, leds, unit, prim_offset=None):
        Helix4Port.__init__(self,
                             label=label,
                             hw_ports=(hw_port,),
                             phy_types=(phy_type,),
                             phy_addrs=(phy_addr,),
                             phy_layer='1GBaseT',
                             prim_offset=prim_offset,
                             leds=leds,
                             unit=unit)

    def speed(self, mode):
        return 1

    def num_logical_ports(self, mode):
        return 1

    @property
    def max_logical_ports(self):
        return 1

class Hurricane21GPort(Hurricane2Port):
    modes = ('1G')
    default_mode = modes[0]

    def __init__(self, label, hw_port, phy_type, phy_addr, phy_layer, leds, unit, prim_offset=None):
        Hurricane2Port.__init__(self,
                             label=label,
                             hw_ports=(hw_port,),
                             phy_types=(phy_type,),
                             phy_addrs=(phy_addr,),
                             phy_layer='1GBaseT',
                             prim_offset=prim_offset,
                             leds=leds,
                             unit=unit)

    def speed(self, mode):
        return 1

    def num_logical_ports(self, mode):
        return 1

    @property
    def max_logical_ports(self):
        return 1

class Apollo10GPort(ApolloPort):
    modes = ('10G',)
    default_mode = modes[0]
    _speed_map = { '10G' : 10 }

    def __init__(self, label, hw_port, phy_type, phy_addr, phy_layer, leds, unit):
        ApolloPort.__init__(self,
                            label=label,
                            hw_ports=(hw_port,),
                            phy_types=(phy_type,),
                            phy_addrs=(phy_addr,),
                            phy_layer=phy_layer,
                            prim_offset=None,
                            leds=leds,
                            unit=unit)

    # return how many logical ports when configured in the given mode
    def num_logical_ports(self, mode):
        return 1

    @property
    def max_logical_ports(self):
        return 1

class Triumph10GPort(TriumphPort):
    modes = ('10G',)
    default_mode = modes[0]
    _speed_map = { '10G' : 10 }

    def __init__(self, label, hw_port, phy_type, phy_addr, phy_layer, leds, unit,
                 rx_lane_map=None, rx_polarity_flip=None,
                 tx_lane_map=None, tx_polarity_flip=None, mdi_pair_map=None):
        TriumphPort.__init__(self,
                             label=label,
                             hw_ports=(hw_port,),
                             phy_types=(phy_type,),
                             phy_addrs=(phy_addr,),
                             phy_layer=phy_layer,
                             prim_offset=None,
                             leds=leds,
                             unit=unit,
                             rx_lane_map=rx_lane_map, tx_lane_map=tx_lane_map,
                             mdi_pair_map=mdi_pair_map,
                             rx_polarity_flip=rx_polarity_flip, tx_polarity_flip=tx_polarity_flip)


    # return how many logical ports when configured in the given mode
    def num_logical_ports(self, mode):
        return 1

    @property
    def max_logical_ports(self):
        return 1

    def polarity_flip_val(self, mode, sub, polarity_flip):
        self._check_mode(mode)

        if polarity_flip is True:
            return 1
        else:
            return 0

class Helix410GPort(Helix4Port):
    modes = ('10G',)
    default_mode = modes[0]
    _speed_map = { '10G' : 10 }

    def __init__(self, label, hw_port, phy_type, phy_addr, phy_layer, leds, unit,
                 rx_lane_map=None, rx_polarity_flip=None,
                 tx_lane_map=None, tx_polarity_flip=None, mdi_pair_map=None):
        Helix4Port.__init__(self,
                             label=label,
                             hw_ports=(hw_port,),
                             phy_types=(phy_type,),
                             phy_addrs=(phy_addr,),
                             phy_layer=phy_layer,
                             prim_offset=None,
                             leds=leds,
                             unit=unit,
                             rx_lane_map=rx_lane_map, tx_lane_map=tx_lane_map,
                             mdi_pair_map=mdi_pair_map,
                             rx_polarity_flip=rx_polarity_flip, tx_polarity_flip=tx_polarity_flip)

    # return how many logical ports when configured in the given mode
    def num_logical_ports(self, mode):
        return 1

    @property
    def max_logical_ports(self):
        return 1

    def polarity_flip_val(self, mode, sub, polarity_flip):
        self._check_mode(mode)

        if polarity_flip is True:
            return 1
        else:
            return 0

class Hurricane210GPort(Hurricane2Port):
    modes = ('10G',)
    default_mode = modes[0]
    _speed_map = { '10G' : 10 }
    disable_prefix_xlate = True

    def __init__(self, label, hw_port, phy_type, phy_addr, phy_layer, leds, unit,
                 rx_lane_map=None, rx_polarity_flip=None,
                 tx_lane_map=None, tx_polarity_flip=None, mdi_pair_map=None):
        Hurricane2Port.__init__(self,
                             label=label,
                             hw_ports=(hw_port,),
                             phy_types=(phy_type,),
                             phy_addrs=(phy_addr,),
                             phy_layer=phy_layer,
                             prim_offset=None,
                             leds=leds,
                             unit=unit,
                             rx_lane_map=rx_lane_map, tx_lane_map=tx_lane_map,
                             mdi_pair_map=mdi_pair_map,
                             rx_polarity_flip=rx_polarity_flip, tx_polarity_flip=tx_polarity_flip)

    # return how many logical ports when configured in the given mode
    def num_logical_ports(self, mode):
        return 1

    @property
    def max_logical_ports(self):
        return 1

    def polarity_flip_val(self, mode, sub, polarity_flip):
        self._check_mode(mode)

        if polarity_flip is True:
            return 1
        else:
            return 0

class Hurricane213GPort(Hurricane2Port):
    modes = ('13G',)
    default_mode = modes[0]
    _speed_map = { '13G' : 13 }
    disable_prefix_xlate = True

    def __init__(self, label, hw_port, unit, phy_type=None, phy_addr=None, phy_layer=None, leds=None,
                 rx_lane_map=None, rx_polarity_flip=None,
                 tx_lane_map=None, tx_polarity_flip=None, mdi_pair_map=None):
        Hurricane2Port.__init__(self,
                             label=label,
                             hw_ports=(hw_port,),
                             phy_types=(phy_type,),
                             phy_addrs=(phy_addr,),
                             phy_layer=phy_layer,
                             prim_offset=None,
                             leds=leds,
                             unit=unit,
                             rx_lane_map=rx_lane_map, tx_lane_map=tx_lane_map,
                             mdi_pair_map=mdi_pair_map,
                             rx_polarity_flip=rx_polarity_flip, tx_polarity_flip=tx_polarity_flip)


    # return how many logical ports when configured in the given mode
    def num_logical_ports(self, mode):
        return 1

    @property
    def max_logical_ports(self):
        return 1

class Trident10GPort(TridentPort):
    gang_modes = ('40G/4',)
    modes = ('10G',) + gang_modes
    default_mode = modes[0]
    max_gang_size = 4
    _speed_map = { '10G'   : 10,
                   '40G/4' : 40 }

    def __init__(self, label, serdes, phy_type, phy_addr, phy_layer, leds, unit,
                 rx_lane_map=None, rx_polarity_flip=None,
                 tx_lane_map=None, tx_polarity_flip=None,
                 mdi_pair_map=None):
        TridentPort.__init__(self,
                             label=label,
                             serdes=(serdes,),
                             phy_types=(phy_type,),
                             phy_addrs=(phy_addr,),
                             phy_layer=phy_layer,
                             prim_offset=None,
                             leds=leds,
                             unit=unit,
                             rx_polarity_flip=rx_polarity_flip,
                             tx_polarity_flip=tx_polarity_flip,
                             rx_lane_map=rx_lane_map,
                             tx_lane_map=tx_lane_map,
                             mdi_pair_map=mdi_pair_map)

    def num_logical_ports(self, mode):
        self._check_mode(mode)
        if mode == '10G':
            return 1
        elif mode == '40G/4':
            if int(self.hw_ports[0][2:]) % 4 == 0:
                return 1
            else:
                return 0

    @property
    def max_logical_ports(self):
        return 1

    def num_physical_ports(self, mode):
        self._check_mode(mode)
        if mode == '10G':
            return 1
        elif mode == '40G/4':
            if int(self.hw_ports[0][2:]) % 4 == 0:
                return 4
            else:
                return 0

    def polarity_flip_val(self, mode, sub, polarity_flip):
        self._check_mode(mode)

        if polarity_flip is True:
            return 1
        else:
            return 0

class TridentTwo10GPort(TridentTwoPort):
    gang_modes = ('40G/4',)
    modes = ('10G',) + gang_modes
    default_mode = modes[0]
    max_gang_size = 4
    _speed_map = { '10G'   : 10,
                   '40G/4' : 40 }

    def __init__(self, label, serdes, phy_types, phy_addrs, phy_layer, leds, unit,
                 rx_lane_map=None, rx_polarity_flip=None,
                 tx_lane_map=None, tx_polarity_flip=None, mdi_pair_map=None,
                 pre_emphasis=None, drv_current=None, pre_drv_current=None,
                 oversub=False, phy_id1=None, sys_interface=None, prim_offset=None):
        TridentTwoPort.__init__(self,
                                label=label,
                                serdes=serdes,
                                phy_types=phy_types,
                                phy_addrs=phy_addrs,
                                phy_layer=phy_layer,
                                prim_offset=prim_offset,
                                leds=leds,
                                unit=unit,
                                rx_polarity_flip=rx_polarity_flip,
                                tx_polarity_flip=tx_polarity_flip,
                                rx_lane_map=rx_lane_map,
                                tx_lane_map=tx_lane_map,
                                mdi_pair_map=mdi_pair_map,
                                pre_emphasis=pre_emphasis,
                                drv_current=drv_current,
                                pre_drv_current=pre_drv_current,
                                oversub=oversub,
                                phy_id1=phy_id1, sys_interface=sys_interface)

    def num_logical_ports(self, mode):
        self._check_mode(mode)
        if mode == '10G':
            return 1
        elif mode == '40G/4':
            if int(self.hw_ports[0][2:]) % 4 == 0:
                return 1
            else:
                return 0

    @property
    def max_logical_ports(self):
        return 1

    def num_physical_ports(self, mode):
        self._check_mode(mode)
        if mode == '10G':
            return 1
        elif mode == '40G/4':
            if int(self.hw_ports[0][2:]) % 4 == 0:
                return 4
            else:
                return 0

    def polarity_flip_val(self, mode, sub, polarity_flip):
        self._check_mode(mode)

        if polarity_flip is True:
            return 1
        else:
            return 0

class TridentTwoPlus10GPort(TridentTwoPlusPort):
    gang_modes = ('40G/4',)
    modes = ('10G',) + gang_modes + ('1G',)
    default_mode = modes[0]
    max_gang_size = 4
    '''
     Todo: Trident2 Plus supports 100G speed
     Need to include 100G speed when required
    '''
    _speed_map = { '1G'    : 1,
                   '10G'   : 10,
                   '40G/4' : 40 }

    def __init__(self, label, serdes, phy_types, phy_addrs, phy_layer, leds, unit,
                 rx_lane_map=None, rx_polarity_flip=None,
                 tx_lane_map=None, tx_polarity_flip=None, mdi_pair_map=None,
                 pre_emphasis=None, drv_current=None, pre_drv_current=None,
                 oversub=False, phy_id1=None, sys_interface=None, prim_offset=None):
        TridentTwoPlusPort.__init__(self,
                                label=label,
                                serdes=serdes,
                                phy_types=phy_types,
                                phy_addrs=phy_addrs,
                                phy_layer=phy_layer,
                                prim_offset=prim_offset,
                                leds=leds,
                                unit=unit,
                                rx_polarity_flip=rx_polarity_flip,
                                tx_polarity_flip=tx_polarity_flip,
                                rx_lane_map=rx_lane_map,
                                tx_lane_map=tx_lane_map,
                                mdi_pair_map=mdi_pair_map,
                                pre_emphasis=pre_emphasis,
                                drv_current=drv_current,
                                pre_drv_current=pre_drv_current,
                                oversub=oversub,
                                phy_id1=phy_id1, sys_interface=sys_interface)

    def num_logical_ports(self, mode):
        self._check_mode(mode)
        if mode in ('1G', '10G'):
            return 1
        elif mode == '40G/4':
            if int(self.hw_ports[0][2:]) % 4 == 0:
                return 1
            else:
                return 0

    @property
    def max_logical_ports(self):
        return 1

    def num_physical_ports(self, mode):
        self._check_mode(mode)
        if mode == ('1G', '10G'):
            return 1
        elif mode == '40G/4':
            if int(self.hw_ports[0][2:]) % 4 == 0:
                return 4
            else:
                return 0

    def polarity_flip_val(self, mode, sub, polarity_flip):
        self._check_mode(mode)

        if polarity_flip is True:
            return 1
        else:
            return 0

class Trident40GPort(TridentPort):
    modes = ('40G', '4x10G')
    default_mode = modes[0]
    _speed_map = { '4x10G' : 10,
                   '40G'   : 40 }

    def __init__(self, label, serdes, phy_types, phy_addrs, leds, unit,
                 rx_lane_map=None, rx_polarity_flip=None,
                 tx_lane_map=None, tx_polarity_flip=None,
                 mdi_pair_map=None,
                 prim_offset=None):
        TridentPort.__init__(self,
                             label=label,
                             serdes=serdes,
                             phy_types=phy_types,
                             phy_addrs=phy_addrs,
                             phy_layer='QSFP+',
                             prim_offset=prim_offset,
                             leds=leds,
                             unit=unit,
                             rx_polarity_flip=rx_polarity_flip,
                             tx_polarity_flip=tx_polarity_flip,
                             rx_lane_map=rx_lane_map,
                             tx_lane_map=tx_lane_map,
                             mdi_pair_map=mdi_pair_map)

    def num_logical_ports(self, mode):
        self._check_mode(mode)

        if mode == '4x10G':
            return 4
        elif mode == '40G':
            return 1

    @property
    def max_logical_ports(self):
        return 4

    def polarity_flip_val(self, mode, sub, polarity_flip_list):
        self._check_mode(mode)

        if mode == '4x10G':
            polarity_flip = polarity_flip_list[sub]
            if polarity_flip is True:
                return 1
            else:
                return 0
        elif mode == '40G':
            flip_intf = 0x0
            for polarity_flip in reversed(polarity_flip_list):
                if polarity_flip is True:
                    flip_intf = (flip_intf << 4) | 0xf
                else:
                    flip_intf = (flip_intf << 4) | 0x0

            return flip_intf

class TridentTwo40GPort(TridentTwoPort):
    modes = ('40G', '4x10G')
    default_mode = modes[0]
    _speed_map = { '4x10G' : 10,
                   '40G'   : 40 }

    def __init__(self, label, serdes, phy_types, phy_addrs, phy_layer, leds, unit,
                 rx_lane_map=None, rx_polarity_flip=None,
                 tx_lane_map=None, tx_polarity_flip=None, mdi_pair_map=None,
                 pre_emphasis=None, drv_current=None, pre_drv_current=None,
                 oversub=False, phy_id1=None, sys_interface=None, prim_offset=None):
        TridentTwoPort.__init__(self,
                                label=label,
                                serdes=serdes,
                                phy_types=phy_types,
                                phy_addrs=phy_addrs,
                                phy_layer=phy_layer,
                                prim_offset=prim_offset,
                                leds=leds,
                                unit=unit,
                                rx_polarity_flip=rx_polarity_flip,
                                tx_polarity_flip=tx_polarity_flip,
                                rx_lane_map=rx_lane_map,
                                tx_lane_map=tx_lane_map,
                                mdi_pair_map=mdi_pair_map,
                                pre_emphasis=pre_emphasis,
                                drv_current=drv_current,
                                pre_drv_current=pre_drv_current,
                                oversub=oversub,
                                phy_id1=phy_id1,
                                sys_interface=sys_interface)

    def num_logical_ports(self, mode):
        self._check_mode(mode)

        if mode == '4x10G':
            return 4
        elif mode == '40G':
            return 1

    @property
    def max_logical_ports(self):
        return 4

    def polarity_flip_val(self, mode, sub, polarity_flip_list):
        self._check_mode(mode)

        if mode == '4x10G':
            if polarity_flip_list & (1 << sub):
                return 1
            else:
                return 0
        elif mode == '40G':
            return polarity_flip_list

class TridentTwoPlus40GPort(TridentTwoPlusPort):
    modes = ('40G', '4x10G', '4x1G')
    default_mode = modes[0]
    _speed_map = { '4x1G'  : 1,
                   '4x10G' : 10,
                   '40G'   : 40 }

    def __init__(self, label, serdes, phy_types, phy_addrs, leds, unit,
                 rx_lane_map=None, rx_polarity_flip=None,
                 tx_lane_map=None, tx_polarity_flip=None, mdi_pair_map=None,
                 pre_emphasis=None, drv_current=None, pre_drv_current=None,
                 oversub=False, phy_id1=None, sys_interface=None, prim_offset=None):
        TridentTwoPlusPort.__init__(self,
                                label=label,
                                serdes=serdes,
                                phy_types=phy_types,
                                phy_addrs=phy_addrs,
                                phy_layer='QSFP+',
                                prim_offset=prim_offset,
                                leds=leds,
                                unit=unit,
                                rx_polarity_flip=rx_polarity_flip,
                                tx_polarity_flip=tx_polarity_flip,
                                rx_lane_map=rx_lane_map,
                                tx_lane_map=tx_lane_map,
                                mdi_pair_map=mdi_pair_map,
                                pre_emphasis=pre_emphasis,
                                drv_current=drv_current,
                                pre_drv_current=pre_drv_current,
                                oversub=oversub,
                                phy_id1=phy_id1,
                                sys_interface=sys_interface)

    def num_logical_ports(self, mode):
        self._check_mode(mode)

        if mode in ('4x10G', '4x1G'):
            return 4
        elif mode == '40G':
            return 1

    @property
    def max_logical_ports(self):
        return 4

    def polarity_flip_val(self, mode, sub, polarity_flip_list):
        self._check_mode(mode)

        if mode in ('4x1G', '4x10G'):
            if polarity_flip_list & (1 << sub):
                return 1
            else:
                return 0
        elif mode == '40G':
            return polarity_flip_list


class Tomahawk100GPort(TomahawkPort):
    # Re-visit ganging for this chip if/when a box with 25G ports is
    # available.
    _speed_map = {
        '100G'  : 100,
        '40G'   : 40,
        '50G'   : 50,
        '2x50G' : 50,
        '2x40G' : 40,
        '2x20G' : 20,
        '4x25G' : 25,
        '4x10G' : 10,
        }
    modes = list(_speed_map)
    default_mode = '100G'

    def __init__(self, label, serdes, phy_types, phy_addrs, phy_layer, leds, unit,
                 rx_lane_map=None, rx_polarity_flip=None,
                 tx_lane_map=None, tx_polarity_flip=None, mdi_pair_map=None,
                 pre_emphasis=None, drv_current=None, pre_drv_current=None,
                 oversub=False, phy_id1=None, sys_interface=None, prim_offset=None):
        self.default_mode = '100G'
        TomahawkPort.__init__(self,
                              label=label,
                              serdes=serdes,
                              phy_types=phy_types,
                              phy_addrs=phy_addrs,
                              phy_layer=phy_layer,
                              prim_offset=prim_offset,
                              leds=leds,
                              unit=unit,
                              rx_polarity_flip=rx_polarity_flip,
                              tx_polarity_flip=tx_polarity_flip,
                              rx_lane_map=rx_lane_map,
                              tx_lane_map=tx_lane_map,
                              mdi_pair_map=mdi_pair_map,
                              pre_emphasis=pre_emphasis,
                              drv_current=drv_current,
                              pre_drv_current=pre_drv_current,
                              oversub=oversub,
                              phy_id1=phy_id1,
                              sys_interface=sys_interface)

    def num_logical_ports(self, mode):
        self._check_mode(mode)
        if mode[1] == 'x':
            return int(mode[0])
        else:
            return 1

    @property
    def max_logical_ports(self):
        return 4

    def polarity_flip_val(self, mode, sub, polarity_flip_list):
        self._check_mode(mode)

        if self.num_logical_ports(mode) == 4:
            if polarity_flip_list & (1 << sub):
                return 1
            else:
                return 0
        elif self.num_logical_ports(mode) == 2:
            return (polarity_flip_list >> (sub * 2)) & 0x3
        else:
            return polarity_flip_list

class Tomahawk25GPort(TomahawkPort):

    _speed_map = {
        '10G'    : 10,
        '25G'    : 25,
        '40G/4'  : 40,
        '100G/4' : 100,
        }
    gang_modes = ('100G/4', '40G/4')
    modes = ('25G','10G',) + gang_modes
    default_mode = modes[0]
    max_gang_size = 4

    def __init__(self, label, serdes, phy_types, phy_addrs, phy_layer, leds, unit,
                 rx_lane_map=None, rx_polarity_flip=None,
                 tx_lane_map=None, tx_polarity_flip=None, mdi_pair_map=None,
                 pre_emphasis=None, drv_current=None, pre_drv_current=None,
                 oversub=False, phy_id1=None, sys_interface=None, prim_offset=None):
       self.default_mode = self.modes[0]
       TomahawkPort.__init__(self,
                              label=label,
                              serdes=serdes,
                              phy_types=phy_types,
                              phy_addrs=phy_addrs,
                              phy_layer=phy_layer,
                              prim_offset=prim_offset,
                              leds=leds,
                              unit=unit,
                              rx_polarity_flip=rx_polarity_flip,
                              tx_polarity_flip=tx_polarity_flip,
                              rx_lane_map=rx_lane_map,
                              tx_lane_map=tx_lane_map,
                              mdi_pair_map=mdi_pair_map,
                              pre_emphasis=pre_emphasis,
                              drv_current=drv_current,
                              pre_drv_current=pre_drv_current,
                              oversub=oversub,
                              phy_id1=phy_id1,
                              sys_interface=sys_interface)

    def num_logical_ports(self, mode):
        self._check_mode(mode)
        if mode == '25G' or mode == '10G':
            return 1
        elif mode == '100G/4' or mode == '40G/4':
            if int(self.hw_ports[0][2:]) % 4 == 0:
                return 1
            else:
                return 0

    @property
    def max_logical_ports(self):
        return 1

    def num_physical_ports(self, mode):
        self._check_mode(mode)
        if mode == '25G' or mode == '10G':
            return 1
        elif mode == '100G/4' or mode == '40G/4':
            if int(self.hw_ports[0][2:]) % 4 == 0:
                return 4
            else:
                return 0

    def polarity_flip_val(self, mode, sub, polarity_flip):
        self._check_mode(mode)

        if polarity_flip is True:
            return 1
        else:
            return 0

class TomahawkMgmt10GPort(TomahawkPort):
    _speed_map = {
        '10G'  : 10,
        }
    modes = list(_speed_map)

    def __init__(self, label, serdes, phy_types, phy_addrs, phy_layer, leds, unit,
                 rx_lane_map=None, rx_polarity_flip=None,
                 tx_lane_map=None, tx_polarity_flip=None, mdi_pair_map=None,
                 pre_emphasis=None, drv_current=None, pre_drv_current=None,
                 oversub=False, phy_id1=None, sys_interface=None, prim_offset=None):
        self.default_mode = self.modes[0]
        TomahawkPort.__init__(self,
                              label=label,
                              serdes=serdes,
                              phy_types=phy_types,
                              phy_addrs=phy_addrs,
                              phy_layer=phy_layer,
                              prim_offset=prim_offset,
                              leds=leds,
                              unit=unit,
                              rx_polarity_flip=rx_polarity_flip,
                              tx_polarity_flip=tx_polarity_flip,
                              rx_lane_map=rx_lane_map,
                              tx_lane_map=tx_lane_map,
                              mdi_pair_map=mdi_pair_map,
                              pre_emphasis=pre_emphasis,
                              drv_current=drv_current,
                              pre_drv_current=pre_drv_current,
                              oversub=oversub,
                              phy_id1=phy_id1,
                              sys_interface=sys_interface)

    def num_logical_ports(self, mode):
        self._check_mode(mode)
        return 1

    @property
    def max_logical_ports(self):
        return 1

class MLXSpeedModeInfo:
    '''
    Speed mode information

    speed       - speed in Gbps
    speed_bmap  - SDK speed bitmap value
    width       - number of serdes lanes
    num_ports   - number of logical ports

    '''
    def __init__(self, speed=None, speed_bmap=None,
                 width=None, num_ports=None):
        self.speed = speed
        self.speed_bmap = speed_bmap
        self.width = width
        self.num_ports = num_ports

class MLXSwitch(SDKSwitch):
    '''
    Mellanox switch platform
    '''
    def __init__(self, chip) :
        SDKSwitch.__init__(self, chip)

class SwitchXTwoSwitch(MLXSwitch):
    '''
    Mellanox switch with Switch-X2 ASIC
    '''
    def __init__(self, chip) :
        MLXSwitch.__init__(self, chip)

class SpectrumSwitch(MLXSwitch):
    '''
    Mellanox switch with Spectrum ASIC
    '''
    def __init__(self, chip) :
        MLXSwitch.__init__(self, chip)

class MLXPort(SDKPort):
    '''
    Mellanox specific port data.
    '''
    _autodetect_phys = ()
    _mode_info_map = {}

    def __init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer, prim_offset,
             leds, unit, rx_polarity_flip, tx_polarity_flip,
             rx_lane_map, tx_lane_map, mdi_pair_map, pre_emphasis, drv_current,
             pre_drv_current, oversub, phy_id1, sys_interface):
        SDKPort.__init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer, prim_offset,
             leds, unit, rx_polarity_flip, tx_polarity_flip,
             rx_lane_map, tx_lane_map, mdi_pair_map, pre_emphasis, drv_current,
             pre_drv_current, oversub, phy_id1, sys_interface)

    def phy_autodetect(self, mode, sub):
        raise NotImplementedError

    def speed(self, mode):
        self._check_mode(mode)
        return self._mode_info_map[mode].speed

    def speed_bmap(self, mode):
        self._check_mode(mode)
        return self._mode_info_map[mode].speed_bmap

    def num_logical_ports(self, mode):
        self._check_mode(mode)
        return self._mode_info_map[mode].num_ports

    def serdes_width(self, mode):
        self._check_mode(mode)
        return self._mode_info_map[mode].width

    # Convert (mode, p) to Mellanox XML (serdes) lanes string
    #
    def xml_lanes(self, mode, p):
        self._check_mode(mode)
        if p >= self.num_logical_ports(mode):
            raise IndexError
        return str(p)

    # Convert (mode, p, phy_addrs) to Mellanox XML lane-to-module string
    #
    def xml_modules(self, mode, p):
        self._check_mode(mode)
        if p >= self.num_logical_ports(mode):
            raise IndexError
        return str(self.phy_addrs[p])

class SwitchXTwoPort(MLXPort):
    def __init__(self, label, adj_label, hw_ports, phy_addrs, phy_layer,
                 blocked_adj_split=None):
        '''
        Addition port attributes:

        adj_label is front panel port label of the adjacent port (because we
            don't want to automatically assume the adjacent label is label+1),
        blocked_adj_split Port is blocked when its adjacent port is split into
            this number of ports (or more)
        '''
        self.adj_label = adj_label
        self.blocked_adj_split = blocked_adj_split

        MLXPort.__init__(self, label, hw_ports=hw_ports, phy_types=None,
                         phy_addrs=phy_addrs, phy_layer=phy_layer,
                         prim_offset=None, leds=None, unit=0,
                         rx_polarity_flip=None, tx_polarity_flip=None,
                         rx_lane_map=None, tx_lane_map=None, mdi_pair_map=None,
                         pre_emphasis=None, drv_current=None,
                         pre_drv_current=None, oversub=False,
                         phy_id1=None, sys_interface=None)

    def blocked_adj_split_num(self):
        if not self.blocked_adj_split is None:
            return self.blocked_adj_split

        return 0

    def configurable(self, config_map):
        '''
        On Mellanox platforms, a port may be blocked (i.e., left unused)
        when an adjacent port is split. As such, it would not appear
        in any config file (such as ports.conf).

        Alternatively, the blocked port might have been marked as 'disable'.
        '''
        if config_map.get(self.label, None) is not None:
            mode = config_map.get(self.label, self.default_mode)
            return not mode.lower().startswith('disable')
        return False


class SwitchXTwo10GPort(SwitchXTwoPort):
    modes = ('10G')
    default_mode = modes[0]
    _mode_info_map = { '10G' : MLXSpeedModeInfo(speed=10, speed_bmap=1<<4,  width=1, num_ports=1)}

    def __init__(self, label, adj_label, hw_port, phy_addr):
        SwitchXTwoPort.__init__(self, label, adj_label=adj_label, hw_ports=(hw_port,),
                                phy_addrs=(phy_addr,), phy_layer='SFP+')

    @property
    def max_logical_ports(self):
        return 1

class SwitchXTwo40GPort(SwitchXTwoPort):
    modes = ('40G', '4x10G', '2x10G')
    default_mode = '40G'
    _mode_info_map = { '40G'   : MLXSpeedModeInfo(speed=40, speed_bmap=1<<6, width=4, num_ports=1),
                       '4x10G' : MLXSpeedModeInfo(speed=10, speed_bmap=1<<4, width=1, num_ports=4),
                       '2x10G' : MLXSpeedModeInfo(speed=10, speed_bmap=1<<4, width=1, num_ports=2)}

    def __init__(self, label, adj_label, hw_ports, phy_addrs, blocked_adj_split=None):
        SwitchXTwoPort.__init__(self, label, adj_label=adj_label, hw_ports=hw_ports,
                                phy_addrs=phy_addrs, phy_layer='QSFP+',
                                blocked_adj_split=blocked_adj_split)

    @property
    def max_logical_ports(self):
        return len(self.hw_ports)

    def mac_addrs_contrib_total(self, config_map):
        '''
        Return the number of MAC addresses this port would contribute to the
        system total. It contributes zero if it could be blocked by a split
        adjacent port, in which case, the contribution would come from the
        adjacent port when everything is eventually summed up.
        '''
        if self.blocked_adj_split_num() > 0:
            return 0

        return self.max_logical_ports

    # Convert (mode, p) to Mellanox XML (serdes) lanes string
    #
    def xml_lanes(self, mode, p):
        _xml_lanes_map = { '40G'  : '0,1,2,3',
                           '4x10G' : str(p),
                           '2x10G' : str(p)}

        self._check_mode(mode)
        if p >= self.num_logical_ports(mode):
            raise IndexError

        return _xml_lanes_map[mode]

    # Convert (mode, p, phy_addrs) to Mellanox XML lane-to-module string
    #
    def xml_modules(self, mode, p):
        self._check_mode(mode)
        if p >= self.num_logical_ports(mode):
            raise IndexError

        # return phy_addrs tuple as a string without any parens or spaces
        #
        if mode == '40G':
            return str(self.phy_addrs).strip('()').replace(' ', '')

        return str(self.phy_addrs[p])

class SpectrumPort(MLXPort):
    def __init__(self, label, adj_label, hw_ports, phy_addrs, phy_layer,
                 blocked_adj_split=None):
        '''
        Addition port attributes:

        adj_label is front panel port label of the adjacent port (because we
            don't want to automatically assume the adjacent label is label+1),
        blocked_adj_split Port is blocked when its adjacent port is split into
            this number of ports (or more)
        '''
        self.adj_label = adj_label
        self.blocked_adj_split = blocked_adj_split

        MLXPort.__init__(self, label, hw_ports=hw_ports, phy_types=None,
                         phy_addrs=phy_addrs, phy_layer=phy_layer,
                         prim_offset=None, leds=None, unit=0,
                         rx_polarity_flip=None, tx_polarity_flip=None,
                         rx_lane_map=None, tx_lane_map=None, mdi_pair_map=None,
                         pre_emphasis=None, drv_current=None,
                         pre_drv_current=None, oversub=False,
                         phy_id1=None, sys_interface=None)

    def blocked_adj_split_num(self):
        if not self.blocked_adj_split is None:
            return self.blocked_adj_split

        return 0

    def configurable(self, config_map):
        '''
        On Mellanox platforms, a port may be blocked (i.e., left unused)
        when an adjacent port is split. As such, it would not appear
        in any config file (such as ports.conf).

        Alternatively, the blocked port might have been marked as 'disable'.
        '''
        if config_map.get(self.label, None) is not None:
            mode = config_map.get(self.label, self.default_mode)
            return not mode.lower().startswith('disable')
        return False

    def port_mac_index(self, mode, mac_index, sub):
        '''
        Return the MAC address index for the sub-interface
        '''
        return self.hw_intf_num(mode, sub) - 1

class Spectrum25GPort(SpectrumPort):
    modes = ( '10G',
              '25G')

    default_mode = '25G'
    _mode_info_map = { '10G'   : MLXSpeedModeInfo(speed=10, speed_bmap=0x00007001, width=1, num_ports=1),
                       '25G'   : MLXSpeedModeInfo(speed=25, speed_bmap=0x28007001, width=1, num_ports=1)}

    def __init__(self, label, adj_label, hw_port, phy_addr):
        SpectrumPort.__init__(self, label, adj_label=adj_label, hw_ports=(hw_port,),
                              phy_addrs=(phy_addr,), phy_layer='SFP28')

    @property
    def max_logical_ports(self):
        return 1

class Spectrum10GPort(SpectrumPort):
    modes = ( '10G')

    default_mode = '10G'
    _mode_info_map = { '10G'   : MLXSpeedModeInfo(speed=10, speed_bmap=0x00007001, width=1, num_ports=1)}

    def __init__(self, label, adj_label, hw_port, phy_addr):
        SpectrumPort.__init__(self, label, adj_label=adj_label, hw_ports=(hw_port,),
                              phy_addrs=(phy_addr,), phy_layer='SFP+')

    @property
    def max_logical_ports(self):
        return 1

class Spectrum100GPort(SpectrumPort):
    modes = ( '40G',
              '50G',
              '100G',
              '2x50G',
              '4x10G',
              '4x25G')

    default_mode = '100G'

    # 40G always uses 4 lanes per CR4/SR4 standard, hence
    # splitting into 2x40G is not possible. Split types other than
    # listed below are supported in hardware but for now we exclude
    # them to simplify testing.
    #
    _mode_info_map = { '40G'   : MLXSpeedModeInfo(speed=40, speed_bmap=0x2801F041, width=4, num_ports=1),
                       '50G'   : MLXSpeedModeInfo(speed=50, speed_bmap=0x68007001, width=2, num_ports=1),
                       '100G'  : MLXSpeedModeInfo(speed=100,speed_bmap=0x68B1F041, width=4, num_ports=1),
                       '2x10G' : MLXSpeedModeInfo(speed=10, speed_bmap=0x00007001, width=1, num_ports=2),
                       '2x25G' : MLXSpeedModeInfo(speed=25, speed_bmap=0x28007001, width=1, num_ports=2),
                       '2x50G' : MLXSpeedModeInfo(speed=50, speed_bmap=0x68007001, width=2, num_ports=2),
                       '4x10G' : MLXSpeedModeInfo(speed=10, speed_bmap=0x00007001, width=1, num_ports=4),
                       '4x25G' : MLXSpeedModeInfo(speed=25, speed_bmap=0x28007001, width=1, num_ports=4)}

    def __init__(self, label, adj_label, hw_ports, phy_addrs, blocked_adj_split=None):
        SpectrumPort.__init__(self, label, adj_label=adj_label, hw_ports=hw_ports,
                              phy_addrs=phy_addrs, phy_layer='QSFP28',
                              blocked_adj_split=blocked_adj_split)

    @property
    def max_logical_ports(self):
        return len(self.hw_ports)

    def mac_addrs_contrib_total(self, config_map):
        '''
        Return the number of MAC addresses this port contributes to the system
        total. If there is an adjacent port, then it contributes 4 minus the
        number of macs consumed by the adjacent port, up to the number of ports.
        If there is no adjacent port the it contributes the number of ports, and
        at least 2.
        '''
        adj_mode = config_map.get(self.adj_label)
        if adj_mode is not None:
            adj_modeInfo = self._mode_info_map.get(adj_mode)
            if adj_modeInfo is not None:
                return min(4-adj_modeInfo.num_ports, self.max_logical_ports)
        return max(2, self.num_logical_ports(config_map.get(self.label)))

    # Convert (mode, p) to Mellanox XML (serdes) lanes string
    #
    def xml_lanes(self, mode, p):
        _xml_2x50G_lanes_map = { 0 : '0,1',
                                 1 : '2,3'}

        _xml_lanes_map = { '1G'    : str(p),
                           '10G'   : str(p),
                           '25G'   : str(p),
                           '40G'   : '0,1,2,3',
                           '50G'   : '0,1',
                           '100G'  : '0,1,2,3',
                           '2x1G'  : str(p),
                           '2x10G' : str(p),
                           '2x25G' : str(p),
                           '2x50G' : None,
                           '4x1G'  : str(p),
                           '4x10G' : str(p),
                           '4x25G' : str(p)}
        self._check_mode(mode)
        if p >= self.num_logical_ports(mode):
            raise IndexError

        if mode == '2x50G':
            return _xml_2x50G_lanes_map[p]
        return _xml_lanes_map[mode]

    # Convert (mode, p, phy_addrs) to Mellanox XML lane-to-module string
    #
    def xml_modules(self, mode, p):
        self._check_mode(mode)
        if p >= self.num_logical_ports(mode):
            raise IndexError

        width = self.serdes_width(mode)
        addr_idx = p * width
        mod_list = []
        for w in range(width):
            mod_list.append(self.phy_addrs[addr_idx])
            addr_idx = addr_idx + 1

        # Return list as a string without any brackets or spaces,
        # e.g., [1, 2] would be returned as '1,2'
        #
        return str(mod_list).strip('[]').replace(' ', '')

    def hw_intf_num(self, mode, sub):
        '''
        Return the physical port interface number (always unmapped by portmap)
        '''
        return int(self.hw_ports[sub][2:])

class Spectrum40GPort(Spectrum100GPort):
    modes = ( '40G',
              '4x10G')

    default_mode = '40G'

    def __init__(self, label, adj_label, hw_ports, phy_addrs, blocked_adj_split=None):
        Spectrum100GPort.__init__(self, label, adj_label=adj_label, hw_ports=hw_ports,
                              phy_addrs=phy_addrs, blocked_adj_split=blocked_adj_split)

class VXSwitch():
    '''
    Virtual Switch platform for Cumulus VX
    '''
    has_phy_ucode_spi_rom = False

    def __init__(self, chip):
        self.chip = chip
        self.ports = ()
        self.led_controllers = ()

class SDKChip:
    '''
    SDK chip common attributes
    '''
    sw_base         = None # SDK
    pci_vendor      = None # PCI vendor ID
    pci_device      = None # PCI device ID
    file_list       = None # datapath configuration file list
    port_channel_type_map = {} # maps number of channels to a port class

    # XXX - These belong in BCMChip
    portmap_capable = None # supports Broadcom portmap remapping
    dport_capable   = None # supports Broadcom dport mapping
    bcm_pn          = None # Broadcom part number
    hsp_bw          = None # chip bandwidth
    base_pbmp       = None # defaut/base port bitmap, typically enables ports
                           # to the CPU and "load balancer"

    def __init__(self, initial_path) :
        self.chip_dir_list = [initial_path]

    @classmethod
    def is_acpi(self):
        return False

    @property
    def chip_config_path(self):
            return os.path.dirname(__file__) + '/__chip_config/'

    @property
    def bcm_config_path(self):
            return os.path.dirname(__file__) + '/__bcm_config/'

    def get_config(self, file_name) :
        if self.file_list == None :
            return None
        if file_name in self.file_list :
            for prefix in self.chip_dir_list :
                file_path = self.chip_config_path + prefix + '/' + file_name
                if os.path.exists(file_path) :
                    with open(file_path, 'r') as config_file :
                        config = config_file.read()
                        return config
        return None

class BCMChip(SDKChip):
    '''
    BCM chip common attributes
    '''
    sw_base = 'bcm'
    ipv4_host_table_size = 8192

    def __init__(self) :
        SDKChip.__init__(self, '/bcm')

class TridentPlusChip(BCMChip):
    '''
    Trident+ chip common attributes
    '''
    portmap_capable = True
    base_pbmp = 0x20000000000000001 # (LBPORT0 | CPU)
    file_list = 'datapath.conf', 'rc.forwarding', 'hw_desc', 'sdk.bcm'
    port_channel_type_map = { 1 : Trident10GPort,
                              4 : Trident40GPort }

    def __init__(self) :
        BCMChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/Trident')

class TridentChip(TridentPlusChip):
    '''
    Trident chip common attributes

    Basically the same as Trident+, but with a smaller host table size.
    '''
    ipv4_host_table_size = 4096

class TridentTwoChip(BCMChip):
    '''
    Trident2 chip common attributes
    '''
    portmap_capable = True
    base_pbmp = (0x1 << 105) | 0x1 # (LBPORT0 | CPU)
    file_list = 'datapath.conf', 'rc.forwarding', 'hw_desc', 'sdk.bcm', 'uft.json'
    port_channel_type_map = { 1 : TridentTwo10GPort,
                              4 : TridentTwo40GPort }

    def __init__(self) :
        BCMChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/TridentTwo')

class TridentTwo_56850_Chip(TridentTwoChip):
    pci_vendor = 0x14e4
    pci_device = 0xb850
    bcm_pn = "56850"
    hsp_bw = 100

    def __init__(self) :
        TridentTwoChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/56850')

class TridentTwo_56854_Chip(TridentTwoChip):
    pci_vendor = 0x14e4
    pci_device = 0xb854
    bcm_pn = "56854"
    hsp_bw = 40

    def __init__(self) :
        TridentTwoChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/56854')

class TridentTwoPlusChip(TridentTwoChip):
    '''
    Trident2 Plus chip common attributes

    '''
    port_channel_type_map = { 1 : TridentTwoPlus10GPort,
                              4 : TridentTwoPlus40GPort }
    dport_capable = True

    def __init__(self) :
        TridentTwoChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + 'Plus')

class TridentTwoPlus_56860_Chip(TridentTwoPlusChip):
    def __init__(self) :
        TridentTwoPlusChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/56860')

class TridentTwoPlus_56864_Chip(TridentTwoPlusChip):
    hsp_bw = 40
    def __init__(self) :
        TridentTwoPlusChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/56864')

class TomahawkChip(BCMChip):
    '''
    Tomahawk chip common attributes
    '''
    pci_vendor = 0x14e4
    pci_device = 0xb960
    bcm_pn = "56960"
    portmap_capable = True
    dport_capable = True
    #XXX - Need to confirm if this is okay
    #XXX   This was derived by running the pbmp command in bcm.user and OR'ing
    #XXX   the CMIC and LB bitmaps.
    base_pbmp = 0x8000000020000000080000000200000001
    file_list = 'datapath.conf', 'rc.forwarding', 'hw_desc', 'sdk.bcm', 'uft.json'
    port_channel_type_map = { 1 : Tomahawk25GPort,
                              4 : Tomahawk100GPort }

    def __init__(self) :
        BCMChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/Tomahawk')

class TriumphChip(BCMChip) :
    portmap_capable = False
    base_pbmp = 0x1 # (CPU)
    file_list = 'rc.forwarding', 'sdk.bcm'
    ipv4_host_table_size = 4096

    def __init__(self) :
        BCMChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/Triumph')

class ApolloChip(BCMChip):
    portmap_capable = False
    base_pbmp = 0x1 # (CPU)
    file_list = 'rc.forwarding', 'sdk.bcm'

    ipv4_host_table_size = 4096

    def __init__(self) :
        BCMChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/Apollo')

class Helix4Chip(BCMChip) :
    portmap_capable = False
    base_pbmp = 0x3c02000000000001 # (AXP_PORTS | LBPORT0| CPU)
    file_list = 'datapath.conf', 'rc.forwarding', 'hw_desc', 'sdk.bcm'

    ipv4_host_table_size = 4096

    def __init__(self) :
        BCMChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/Helix4')

class Hurricane2Chip(BCMChip) :
    portmap_capable = False
    base_pbmp = 0x01 # (CPU)
    file_list = 'rc.forwarding', 'sdk.bcm'

    ipv4_host_table_size = 4096

    def __init__(self) :
        BCMChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/Hurricane2')

class MLXChip(SDKChip):
    '''
    Mellanox chip common attributes
    '''
    sw_base = 'mlx'
    ipv4_host_table_size = 8192
    file_list = 'datapath.conf'

    def __init__(self) :
        SDKChip.__init__(self, '/mlx')

class SwitchXTwoChip(MLXChip):
    '''
    Mellanox Switch-X2 ASIC
    '''
    def __init__(self) :
        MLXChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/Switch-X2')

class SpectrumChip(MLXChip):
    '''
    Mellanox Spectrum ASIC
    '''
    def __init__(self) :
        MLXChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/Spectrum')

class VXChip(SDKChip):
    '''
    Virtual Chip for Cumulus VX
    '''
    sw_base = 'vx'
    file_list = None
    ipv4_host_table_size = 8192

    def __init__(self) :
        SDKChip.__init__(self, '/vx')

chips = (TriumphChip,
         ApolloChip,
         TridentChip,
         Helix4Chip,
         Hurricane2Chip,
         TridentTwo_56850_Chip,
         TridentTwo_56854_Chip,
         TomahawkChip)
