#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class AcctonAS5712_54XSFPp(cumulus.platform.TridentTwo10GPort):
    _set_led = '''
;;; AcctonAS5712_54XSFPp
;;; Two bits per port: red (active low), green (active low)
set_led:
set_bicolor_led:
        pushst  PORTSTATUS_ONE   ; turn off red LED
        pack
set_green_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        ret
'''
    def __init__(self, label, serdes, leds):
        cumulus.platform.TridentTwoPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=None,
                                                 phy_addrs=None,
                                                 phy_layer='SFP+',
                                                 prim_offset=None,
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, led_port)

        for led in led_index_list:
            set_calls[led] = call
            set_calls[led + 1] = None

        return set_calls

class AcctonAS5712_54XQSFPp(cumulus.platform.TridentTwo40GPort):
    '''
    ;;;
    ;;; There are 5 leds per qsfp port on this platform
    ;;;   1 led to use when the port is in 40G mode
    ;;;   The other 4 are used when the port is in 4x10G mode
    ;;;   all are 1 bit per led, green - active low
    ;;;
    '''
    _set_led_40G = '''
;;; AcctonAS5712_54XQSFPp
set_led:
        call    set_green_led ; set the first led green/off
        ret
'''
    _set_led_4x10 = '''
;;; AcctonAS5712_54XQSFPp_4x10
set_led:
        call    set_green_led
        inc     a
        call    set_green_led
        inc     a
        call    set_green_led
        inc     a
        call    set_green_led

        ret
'''
    _set_led_4x10_off = '''
;;; AcctonAS5712_54XQSFPp_4x10_off
;;;   when the port is in 40G mode, we turn these 4 leds off
set_led:
        ld      a,4
        call    pad_n_one
        ret
'''
    def __init__(self, label, serdes, leds):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.TridentTwo40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=None,
                                                 phy_addrs=None,
                                                 phy_layer=sfp.type.upper(),
                                                 leds=leds,
                                                 unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if mode == '4x10G' and sub != 0:
            return set_calls

        if mode == "4x10G":
            # turn the single 40G led off
            call_40G = 1
            # use the group of 4 leds for link/activity
            call_4x10G = ("Act5712_QSFP_4x10",     self._set_led_4x10,     led_port)
        elif mode == "40G":
            # use the single led for link/activity
            call_40G = ("Act5712_QSFP_40G",      self._set_led_40G,      led_port)
            # turn the group of 4 leds off
            call_4x10G = ("Act5712_QSFP_4x10_off", self._set_led_4x10_off, led_port)

        set_calls[self.leds[1]] = call_40G
        set_calls[self.leds[2]] = call_4x10G
        set_calls[self.leds[3]] = None
        set_calls[self.leds[4]] = None
        set_calls[self.leds[5]] = None

        return set_calls


class AcctonAS5712_54XLED(cumulus.ledup.TridentTwoLEDController):
    num_leds = 72

class AcctonAS5712_54XSwitch(cumulus.platform.BCMSwitch):
    ledup = AcctonAS5712_54XLED
    _sfp = AcctonAS5712_54XSFPp
    _qsfp = AcctonAS5712_54XQSFPp

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _sfp(label="1",  serdes=(12,), leds=(0, 70)),
        _sfp(label="2",  serdes=(13,), leds=(0, 68)),
        _sfp(label="3",  serdes=(14,), leds=(0, 66)),
        _sfp(label="4",  serdes=(15,), leds=(0, 64)),
        _sfp(label="5",  serdes=(20,), leds=(0, 62)),
        _sfp(label="6",  serdes=(21,), leds=(0, 60)),
        _sfp(label="7",  serdes=(22,), leds=(0, 58)),
        _sfp(label="8",  serdes=(23,), leds=(0, 56)),
        _sfp(label="9",  serdes=(24,), leds=(0, 54)),
        _sfp(label="10", serdes=(25,), leds=(0, 52)),
        _sfp(label="11", serdes=(26,), leds=(0, 50)),
        _sfp(label="12", serdes=(27,), leds=(0, 48)),
        _sfp(label="13", serdes=(28,), leds=(0, 46)),
        _sfp(label="14", serdes=(29,), leds=(0, 44)),
        _sfp(label="15", serdes=(30,), leds=(0, 42)),
        _sfp(label="16", serdes=(31,), leds=(0, 40)),
        _sfp(label="17", serdes=(44,), leds=(0, 38)),
        _sfp(label="18", serdes=(45,), leds=(0, 36)),
        _sfp(label="19", serdes=(46,), leds=(0, 34)),
        _sfp(label="20", serdes=(47,), leds=(0, 32)),
        _sfp(label="21", serdes=(48,), leds=(0, 30)),
        _sfp(label="22", serdes=(49,), leds=(0, 28)),
        _sfp(label="23", serdes=(50,), leds=(0, 26)),
        _sfp(label="24", serdes=(51,), leds=(0, 24)),
        _sfp(label="25", serdes=(52,), leds=(0, 22)),
        _sfp(label="26", serdes=(53,), leds=(0, 20)),
        _sfp(label="27", serdes=(54,), leds=(0, 18)),
        _sfp(label="28", serdes=(55,), leds=(0, 16)),
        _sfp(label="29", serdes=(56,), leds=(0, 14)),
        _sfp(label="30", serdes=(57,), leds=(0, 12)),
        _sfp(label="31", serdes=(58,), leds=(0, 10)),
        _sfp(label="32", serdes=(59,), leds=(0,  8)),
        _sfp(label="33", serdes=(60,), leds=(0,  6)),
        _sfp(label="34", serdes=(61,), leds=(0,  4)),
        _sfp(label="35", serdes=(62,), leds=(0,  2)),
        _sfp(label="36", serdes=(63,), leds=(0,  0)),
        _sfp(label="37", serdes=(64,), leds=(1, 70)),
        _sfp(label="38", serdes=(65,), leds=(1, 68)),
        _sfp(label="39", serdes=(66,), leds=(1, 66)),
        _sfp(label="40", serdes=(67,), leds=(1, 64)),
        _sfp(label="41", serdes=(68,), leds=(1, 62)),
        _sfp(label="42", serdes=(69,), leds=(1, 60)),
        _sfp(label="43", serdes=(70,), leds=(1, 58)),
        _sfp(label="44", serdes=(71,), leds=(1, 56)),
        _sfp(label="45", serdes=(72,), leds=(1, 54)),
        _sfp(label="46", serdes=(73,), leds=(1, 52)),
        _sfp(label="47", serdes=(74,), leds=(1, 50)),
        _sfp(label="48", serdes=(75,), leds=(1, 48)),

        _qsfp(label="49", serdes=(96, 97, 98, 99),     leds=(1, 45, 18, 19, 20, 21)),
        _qsfp(label="50", serdes=(100, 101, 102, 103), leds=(1, 44, 22, 23, 24, 25)),
        _qsfp(label="51", serdes=(80, 81, 82, 83),     leds=(1, 46, 26, 27, 28, 29)),
        _qsfp(label="52", serdes=(104, 105, 106, 107), leds=(1, 43, 30, 31, 32, 33)),
        _qsfp(label="53", serdes=(108, 109, 110, 111), leds=(1, 42, 34, 35, 36, 37)),
        _qsfp(label="54", serdes=(76, 77, 78, 79),     leds=(1, 47, 38, 39, 40, 41)),
    )

class AcctonAS5712_54XCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/accton_as5712_54x_cpld.0'

'''
   Accton AS5712_54X LEDs
       Front Panel PSU1:
         leave under hardware control, not managed by software.
       Front Panel PSU2:
         leave under hardware control, not managed by software.
       Front Panel Diag:
         GREEN, RED, YELLOW, OFF
       Front Panel FAN Status:
         leave under hardware control, not managed by software.
       Front Panel Locator:
         YELLOW, Blinking YELLOW, OFF
       Back Panel - 5 individual FAN tray LEDs:
         leave under hardware control, not managed by software.

'''

class AcctonAS5712_54XSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'Diag'
    led_location = 'led_diag'

class AcctonAS5712_54XPSU1LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU1'
    sources = ['PSU1']
    led_location = 'led_psu1'

class AcctonAS5712_54XPSU2LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU2'
    sources = ['PSU2']
    led_location = 'led_psu2'

class AcctonAS5712_54XFanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan'
    # Depends on system fans 01 through 10
    sources = [ "Fan%02d" % (x) for x in range(1,11) ]
    led_location = 'led_fan'

class AcctonAS5712_54XLEDs():
    units = (AcctonAS5712_54XSystemLED(),
             AcctonAS5712_54XPSU1LED(),
             AcctonAS5712_54XPSU2LED(),
             AcctonAS5712_54XFanLED(),
             )

class AcctonAS5712_54XTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = '/sys/bus/i2c/devices/15-0048'
    # Accton has specified that fan speed should be at 40% till 49.5 C
    # and then increase to 100% at 57.7
    temp_crit = 85
    temp_max = 58
    temp_fan_min = 50

class AcctonAS5712_54XTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Rear Air sensor'
    driver_path = '/sys/bus/i2c/devices/16-0049'
    # Accton has specified that fan speed should be at 40% till 49.5 C
    # and then increase to 100% at 57.7
    temp_crit = 85
    temp_max = 58
    temp_fan_min = 50

class AcctonAS5712_54XTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Front Air sensor'
    driver_path = '/sys/bus/i2c/devices/17-004a'
    temp_crit = 85
    temp_max = 58
    temp_fan_min = 50

AcctonAS5712_54X_core_path = '/sys/devices/platform/coretemp.0'
class AcctonAS5712_54XTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = 'Intel CPU die sensor'
    driver_path = AcctonAS5712_54X_core_path
    driver_hwmon = ['temp2']
    temp_crit = 98

class AcctonAS5712_54XTemp5(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    description = 'Intel CPU die sensor'
    driver_path = AcctonAS5712_54X_core_path
    driver_hwmon = ['temp3']
    temp_fan_min = 50
    temp_crit = 98

class AcctonAS5712_54XTemp6(cumulus.sensors.Temp_Unit):
    name = 'Temp6'
    description = 'Intel CPU die sensor'
    driver_path = AcctonAS5712_54X_core_path
    driver_hwmon = ['temp4']
    temp_fan_min = 50
    temp_crit = 98

class AcctonAS5712_54XTemp7(cumulus.sensors.Temp_Unit):
    name = 'Temp7'
    description = 'Intel CPU die sensor'
    driver_path = AcctonAS5712_54X_core_path
    driver_hwmon = ['temp5']
    temp_fan_min = 50
    temp_crit = 98

class AcctonAS5712_54XDIMMTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.dimm_sensor_name
    driver_hwmon = ['temp1']
    temp_crit = 95
    temp_fan_min = 50
    temp_max = 90

    def generate_driver_path(self, i2c_addr):
        pci_0_1f_3_path = '/sys/devices/pci0000:00/0000:00:1f.3'

        if os.path.exists(pci_0_1f_3_path):
            for x in os.listdir(pci_0_1f_3_path):
                if x.startswith('i2c-'):
                    a,b = x.split('-')
                    return os.path.join(pci_0_1f_3_path, x) + '/' + b + '-00' + i2c_addr
        return None

class AcctonAS5712_54XTemp8(AcctonAS5712_54XDIMMTemp):
    name = 'Temp8'

    def __init__(self):
        self.driver_path = self.generate_driver_path('1a')
        AcctonAS5712_54XDIMMTemp.__init__(self)

class AcctonAS5712_54XTemp9(AcctonAS5712_54XDIMMTemp):
    name = 'Temp9'

    def __init__(self):
        self.driver_path = self.generate_driver_path('1b')
        AcctonAS5712_54XDIMMTemp.__init__(self)

class AcctonAS5712_54X_PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = 'rmmod cpr4011 && modprobe cpr4011'
            subprocess.call(cmd, shell=True)
            self.init_done = True

class AcctonAS5712_54XPSU1(AcctonAS5712_54X_PSU):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class AcctonAS5712_54XPSU2(AcctonAS5712_54X_PSU):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

# The fan modules on the AS5712_54X contain two fans each, one called
# "FAN" (21500 RPM) and one called "FANR" (18000 RPM).  All the odd
# fans are of type FAN and the evens are of type FANR.  Create a base
# class for each type.
class AcctonAS5712_54XFan(cumulus.sensors.Fan_Unit):
    driver_path = AcctonAS5712_54XCPLD.path
    fan_min = 9600
    fan_max = 23500
    hwmon_search = False
    # Accton has specified that the default fan speed must be 40% of max.
    minpwm = 102

class AcctonAS5712_54XFanR(cumulus.sensors.Fan_Unit):
    driver_path = AcctonAS5712_54XCPLD.path
    fan_min = 8100
    fan_max = 20000
    hwmon_search = False
    # Accton has specified that the default fan speed must be 40% of max.
    minpwm = 102

class AcctonAS5712_54XFan1(AcctonAS5712_54XFan):
    name = 'Fan01'
    description = 'Fan Tray 1, Fan A'
    # driver is fan speed ...
    driver_hwmon = ['fan01']
    # status is "cpld_hwmon"
    cpld_hwmon = ['fan01']
    # The CPLD driver only exports "pwm1", which controls all the
    # fans.
    pwm_hwmon = ['pwm1']

class AcctonAS5712_54XFan2(AcctonAS5712_54XFanR):
    name = 'Fan02'
    description = 'Fan Tray 1, Fan B'
    driver_hwmon = ['fan02']
    cpld_hwmon = ['fan02']

class AcctonAS5712_54XFan3(AcctonAS5712_54XFan):
    name = 'Fan03'
    description = 'Fan Tray 2, Fan A'
    driver_hwmon = ['fan03']
    cpld_hwmon = ['fan03']

class AcctonAS5712_54XFan4(AcctonAS5712_54XFanR):
    name = 'Fan04'
    description = 'Fan Tray 2, Fan B'
    driver_hwmon = ['fan04']
    cpld_hwmon = ['fan04']

class AcctonAS5712_54XFan5(AcctonAS5712_54XFan):
    name = 'Fan05'
    description = 'Fan Tray 3, Fan A'
    driver_hwmon = ['fan05']
    cpld_hwmon = ['fan05']

class AcctonAS5712_54XFan6(AcctonAS5712_54XFanR):
    name = 'Fan06'
    description = 'Fan Tray 3, Fan B'
    driver_hwmon = ['fan06']
    cpld_hwmon = ['fan06']

class AcctonAS5712_54XFan7(AcctonAS5712_54XFan):
    name = 'Fan07'
    description = 'Fan Tray 4, Fan A'
    driver_hwmon = ['fan07']
    cpld_hwmon = ['fan07']

class AcctonAS5712_54XFan8(AcctonAS5712_54XFanR):
    name = 'Fan08'
    description = 'Fan Tray 4, Fan B'
    driver_hwmon = ['fan08']
    cpld_hwmon = ['fan08']

class AcctonAS5712_54XFan9(AcctonAS5712_54XFan):
    name = 'Fan09'
    description = 'Fan Tray 5, Fan A'
    driver_hwmon = ['fan09']
    cpld_hwmon = ['fan09']

class AcctonAS5712_54XFan10(AcctonAS5712_54XFanR):
    name = 'Fan10'
    description = 'Fan Tray 5, Fan B'
    driver_hwmon = ['fan10']
    cpld_hwmon = ['fan10']

# PSU I2C addresses
AcctonAS5712_54X_PSU1_i2c_path = '/sys/bus/i2c/devices/11-003c'
AcctonAS5712_54X_PSU2_i2c_path = '/sys/bus/i2c/devices/12-003f'

# PSU Temp Sensors
class AcctonAS5712_54XPSUTemp(cumulus.sensors.Temp_Unit):
    temp_crit = 65
    temp_max = 50
    temp_fan_min = 25
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class AcctonAS5712_54XTemp10(AcctonAS5712_54XPSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS5712_54X_PSU1_i2c_path
    driver_hwmon = ['temp1']

class AcctonAS5712_54XTemp11(AcctonAS5712_54XPSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS5712_54X_PSU1_i2c_path
    driver_hwmon = ['temp2']

class AcctonAS5712_54XTemp12(AcctonAS5712_54XPSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS5712_54X_PSU2_i2c_path
    driver_hwmon = ['temp1']

class AcctonAS5712_54XTemp13(AcctonAS5712_54XPSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS5712_54X_PSU2_i2c_path
    driver_hwmon = ['temp2']

# PSU Fan Sensors
class AcctonAS5712_54XPSUFan(cumulus.sensors.Fan_Unit):
    minpwm = 64
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_max = 22000
    fan_min = 8000
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class AcctonAS5712_54XFan11(AcctonAS5712_54XPSUFan):
    psu = 'PSU1'
    name = psu + 'Fan1'
    description = psu + ' Fan'
    driver_path = AcctonAS5712_54X_PSU1_i2c_path

class AcctonAS5712_54XFan12(AcctonAS5712_54XPSUFan):
    psu = 'PSU2'
    name = psu + 'Fan1'
    description = psu + ' Fan'
    driver_path = AcctonAS5712_54X_PSU2_i2c_path

class AcctonAS5712_54XSensors():
    units = ( AcctonAS5712_54XPSU1(),
              AcctonAS5712_54XPSU2(),
              AcctonAS5712_54XTemp1(),
              AcctonAS5712_54XTemp2(),
              AcctonAS5712_54XTemp3(),
              AcctonAS5712_54XTemp4(),
              AcctonAS5712_54XTemp5(),
              AcctonAS5712_54XTemp6(),
              AcctonAS5712_54XTemp7(),
              AcctonAS5712_54XTemp8(),
              AcctonAS5712_54XTemp9(),
              AcctonAS5712_54XTemp10(),
              AcctonAS5712_54XTemp11(),
              AcctonAS5712_54XTemp12(),
              AcctonAS5712_54XTemp13(),
              AcctonAS5712_54XFan1(),
              AcctonAS5712_54XFan2(),
              AcctonAS5712_54XFan3(),
              AcctonAS5712_54XFan4(),
              AcctonAS5712_54XFan5(),
              AcctonAS5712_54XFan6(),
              AcctonAS5712_54XFan7(),
              AcctonAS5712_54XFan8(),
              AcctonAS5712_54XFan9(),
              AcctonAS5712_54XFan10(),
              AcctonAS5712_54XFan11(),
              AcctonAS5712_54XFan12(),
              )

class AcctonAS5712_54XPWM1(cumulus.sensors.PWM_Unit):
    # This is only one PWM control knob for all 10 system fans, so
    # only use Fan1 here.
    name = 'System PWM'
    fans = (AcctonAS5712_54XFan1(),
            AcctonAS5712_54XFan2(),
            AcctonAS5712_54XFan3(),
            AcctonAS5712_54XFan4(),
            AcctonAS5712_54XFan5(),
            AcctonAS5712_54XFan6(),
            AcctonAS5712_54XFan7(),
            AcctonAS5712_54XFan8(),
            AcctonAS5712_54XFan9(),
            AcctonAS5712_54XFan10(),
            )
    temps = ( AcctonAS5712_54XTemp1(),
              AcctonAS5712_54XTemp2(),
              AcctonAS5712_54XTemp3(),
              AcctonAS5712_54XTemp4(),
              AcctonAS5712_54XTemp5(),
              AcctonAS5712_54XTemp6(),
              AcctonAS5712_54XTemp7(),
              AcctonAS5712_54XTemp8(),
              AcctonAS5712_54XTemp9(),
              )

# PSU1 Fan Controller
class AcctonAS5712_54XPWM2(cumulus.sensors.PWM_Unit):
    name = 'PSU1 PWM'
    fans = (AcctonAS5712_54XFan11(),
            )
    temps = (AcctonAS5712_54XTemp10(),
             AcctonAS5712_54XTemp11(),
             )

# PSU2 Fan Controller
class AcctonAS5712_54XPWM3(cumulus.sensors.PWM_Unit):
    name = 'PSU2 PWM'
    fans = (AcctonAS5712_54XFan12(),
            )
    temps = (AcctonAS5712_54XTemp12(),
             AcctonAS5712_54XTemp13(),
             )

class AcctonAS5712_54XPWMs():
    units = (AcctonAS5712_54XPWM1(),
             AcctonAS5712_54XPWM2(),
             AcctonAS5712_54XPWM3(),
             )

class AcctonAS5712_54X(cumulus.platform.Platform):
    name = 'accton,as5712_54x'
    chip = cumulus.platform.TridentTwo_56854_Chip()
    switch = AcctonAS5712_54XSwitch(chip)
    cpld = AcctonAS5712_54XCPLD()
    statusleds = AcctonAS5712_54XLEDs()
    pwms = AcctonAS5712_54XPWMs()
    sensors = AcctonAS5712_54XSensors()

Platform = AcctonAS5712_54X
