#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
from   cumulus.platforms.accton_as5712_54x import *

class AcctonAS5812_54XSFPp(cumulus.platform.TridentTwoPlus10GPort):
    _set_led = '''
;;; AcctonAS5812_54XSFPp
;;; Two bits per port: red (active low), green (active low)
set_led:
set_bicolor_led:
        pushst  PORTSTATUS_ONE   ; turn off red LED
        pack
set_green_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        ret
'''
    def __init__(self, label, serdes, leds):
        cumulus.platform.TridentTwoPlusPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=None,
                                                 phy_addrs=None,
                                                 phy_layer='SFP+',
                                                 prim_offset=None,
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')
        self._serdes_intf_map = {10:'SFI'}

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, led_port)

        for led in led_index_list:
            set_calls[led] = call
            set_calls[led + 1] = None

        return set_calls

class AcctonAS5812_54XQSFPp(cumulus.platform.TridentTwoPlus40GPort):
    '''
    ;;;
    ;;; There are 5 leds per qsfp port on this platform
    ;;;   1 led to use when the port is in 40G mode
    ;;;   The other 4 are used when the port is in 4x10G mode
    ;;;   all are 1 bit per led, green - active low
    ;;;
    '''
    _set_led_40G = '''
;;; AcctonAS5812_54XQSFPp
set_led:
        call    set_green_led ; set the first led green/off
        ret
'''
    _set_led_4x10 = '''
;;; AcctonAS5812_54XQSFPp_4x10
set_led:
        call    set_green_led
        inc     a
        call    set_green_led
        inc     a
        call    set_green_led
        inc     a
        call    set_green_led

        ret
'''
    _set_led_4x10_off = '''
;;; AcctonAS5812_54XQSFPp_4x10_off
;;;   when the port is in 40G mode, we turn these 4 leds off
set_led:
        ld      a,4
        call    pad_n_one
        ret
'''
    def __init__(self, label, serdes, leds):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.TridentTwoPlus40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=None,
                                                 phy_addrs=None,
                                                 leds=leds,
                                                 unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode in ('4x1G', '4x10G')) and (sub != 0):
            return set_calls

        if mode in ('4x1G', '4x10G'):
            # turn the single 40G led off
            call_40G = 1
            # use the group of 4 leds for link/activity
            call_4x10G = ("Act5812_QSFP_4x10",     self._set_led_4x10,     led_port)
        elif mode == "40G":
            # use the single led for link/activity
            call_40G = ("Act5812_QSFP_40G",      self._set_led_40G,      led_port)
            # turn the group of 4 leds off
            call_4x10G = ("Act5812_QSFP_4x10_off", self._set_led_4x10_off, led_port)

        set_calls[self.leds[1]] = call_40G
        set_calls[self.leds[2]] = call_4x10G
        set_calls[self.leds[3]] = None
        set_calls[self.leds[4]] = None
        set_calls[self.leds[5]] = None

        return set_calls


class AcctonAS5812_54XLED(cumulus.ledup.TridentTwoPlusLEDController):
    num_leds = 72

class AcctonAS5812_54XSwitch(cumulus.platform.BCMSwitch):
    ledup = AcctonAS5812_54XLED
    _sfp = AcctonAS5812_54XSFPp
    _qsfp = AcctonAS5812_54XQSFPp

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _sfp(label="1",  serdes=(12,), leds=(0, 70)),
        _sfp(label="2",  serdes=(13,), leds=(0, 68)),
        _sfp(label="3",  serdes=(14,), leds=(0, 66)),
        _sfp(label="4",  serdes=(15,), leds=(0, 64)),
        _sfp(label="5",  serdes=(20,), leds=(0, 62)),
        _sfp(label="6",  serdes=(21,), leds=(0, 60)),
        _sfp(label="7",  serdes=(22,), leds=(0, 58)),
        _sfp(label="8",  serdes=(23,), leds=(0, 56)),
        _sfp(label="9",  serdes=(24,), leds=(0, 54)),
        _sfp(label="10", serdes=(25,), leds=(0, 52)),
        _sfp(label="11", serdes=(26,), leds=(0, 50)),
        _sfp(label="12", serdes=(27,), leds=(0, 48)),
        _sfp(label="13", serdes=(28,), leds=(0, 46)),
        _sfp(label="14", serdes=(29,), leds=(0, 44)),
        _sfp(label="15", serdes=(30,), leds=(0, 42)),
        _sfp(label="16", serdes=(31,), leds=(0, 40)),
        _sfp(label="17", serdes=(44,), leds=(0, 38)),
        _sfp(label="18", serdes=(45,), leds=(0, 36)),
        _sfp(label="19", serdes=(46,), leds=(0, 34)),
        _sfp(label="20", serdes=(47,), leds=(0, 32)),
        _sfp(label="21", serdes=(48,), leds=(0, 30)),
        _sfp(label="22", serdes=(49,), leds=(0, 28)),
        _sfp(label="23", serdes=(50,), leds=(0, 26)),
        _sfp(label="24", serdes=(51,), leds=(0, 24)),
        _sfp(label="25", serdes=(52,), leds=(0, 22)),
        _sfp(label="26", serdes=(53,), leds=(0, 20)),
        _sfp(label="27", serdes=(54,), leds=(0, 18)),
        _sfp(label="28", serdes=(55,), leds=(0, 16)),
        _sfp(label="29", serdes=(56,), leds=(0, 14)),
        _sfp(label="30", serdes=(57,), leds=(0, 12)),
        _sfp(label="31", serdes=(58,), leds=(0, 10)),
        _sfp(label="32", serdes=(59,), leds=(0,  8)),
        _sfp(label="33", serdes=(60,), leds=(0,  6)),
        _sfp(label="34", serdes=(61,), leds=(0,  4)),
        _sfp(label="35", serdes=(62,), leds=(0,  2)),
        _sfp(label="36", serdes=(63,), leds=(0,  0)),
        _sfp(label="37", serdes=(64,), leds=(1, 70)),
        _sfp(label="38", serdes=(65,), leds=(1, 68)),
        _sfp(label="39", serdes=(66,), leds=(1, 66)),
        _sfp(label="40", serdes=(67,), leds=(1, 64)),
        _sfp(label="41", serdes=(68,), leds=(1, 62)),
        _sfp(label="42", serdes=(69,), leds=(1, 60)),
        _sfp(label="43", serdes=(70,), leds=(1, 58)),
        _sfp(label="44", serdes=(71,), leds=(1, 56)),
        _sfp(label="45", serdes=(72,), leds=(1, 54)),
        _sfp(label="46", serdes=(73,), leds=(1, 52)),
        _sfp(label="47", serdes=(74,), leds=(1, 50)),
        _sfp(label="48", serdes=(75,), leds=(1, 48)),

        _qsfp(label="49", serdes=(96, 97, 98, 99),     leds=(1, 45, 18, 19, 20, 21)),
        _qsfp(label="50", serdes=(100, 101, 102, 103), leds=(1, 44, 22, 23, 24, 25)),
        _qsfp(label="51", serdes=(80, 81, 82, 83),     leds=(1, 46, 26, 27, 28, 29)),
        _qsfp(label="52", serdes=(104, 105, 106, 107), leds=(1, 43, 30, 31, 32, 33)),
        _qsfp(label="53", serdes=(108, 109, 110, 111), leds=(1, 42, 34, 35, 36, 37)),
        _qsfp(label="54", serdes=(76, 77, 78, 79),     leds=(1, 47, 38, 39, 40, 41)),
    )

class AcctonAS5812_54X(cumulus.platform.Platform):
    name = 'accton,as5812_54x'
    chip = cumulus.platform.TridentTwoPlus_56864_Chip()
    switch = AcctonAS5812_54XSwitch(chip)
    cpld = AcctonAS5712_54XCPLD()
    statusleds = AcctonAS5712_54XLEDs()
    pwms = AcctonAS5712_54XPWMs()
    sensors = AcctonAS5712_54XSensors()

Platform = AcctonAS5812_54X
