#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
from   cumulus.platforms.accton_as6712_32x import *

class AcctonAS6812_32XQSFPp(cumulus.platform.TridentTwoPlus40GPort):
    _set_led = '''
;;; AcctonAS6812_32XQSFPp
;;; This box uses 9 bits per port to set 40G and 4x10G modes.
;;; Bit0 sets 40G mode/non 40G mode.
;;; If bit0=1, we are in 40G mode. Except bit2, we ignore the other bits.
;;; Setting bit2 to 0/1 toggles the led from green to off.
;;; If bit0=0, we are in 4x10G mode. Each port is then
;;; represented by 2 bits.
;;; bit_1, bit_3, bit_5, bit_7, when 1, indicate 10G mode
;;; bit_2, bit_4, bit_6, bit_8, when 1=orange, 0= off

set_off_led:
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ZERO
        pack
        ret

set_led:
        ; bit0=1 for 40G mode
        port a
        pushst  PORTSTATUS_ONE
        pack

        pushst  PORTSTATUS_ONE
        pack
        ; set color
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ; setting other bits to zero
        call set_off_led
        call set_off_led
        call set_off_led
        ret
'''
    _set_led_4x10 = '''
;;; AcctonAS6812_32XQSFPp_4x10
;;; This box uses 9 bits per port to set 40G and 4x10G modes.
;;; Bit0 sets 40G mode/non 40G mode.
;;; If bit0=1, we are in 40G mode. Except bit2, we ignore the other bits.
;;; Setting bit2 to 0/1 toggles the led from green to off.
;;; If bit0=0, we are in 4x10G mode. Each port is then
;;; represented by 2 bits.
;;; bit_1, bit_3, bit_5, bit_7, when 1, indicate 10G mode
;;; bit_2, bit_4, bit_6, bit_8, when 1=orange, 0= off

set_color_led:
        port a

        ; LED 10G Mode
        pushst  PORTSTATUS_ONE
        pack

        ; set color
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret

set_led:
        port a

        ; Set to Non-40G Mode
        pushst  PORTSTATUS_ZERO
        pack

        call set_color_led
        inc a
        call set_color_led
        inc a
        call set_color_led
        inc a
        call set_color_led
        inc a
        ret
'''

    def __init__(self, label, serdes, leds):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.TridentTwoPlus40GPort.__init__(self,
                                                    label=label,
                                                    serdes=serdes,
                                                    phy_types=None,
                                                    phy_addrs=None,
                                                    leds=leds,
                                                    oversub=True,
                                                    unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode in ('4x1G', '4x10G')) and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        if mode in ('4x1G', '4x10G'):
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None
        set_calls[led_index_list[0] + 2] = None
        set_calls[led_index_list[0] + 3] = None
        set_calls[led_index_list[0] + 4] = None
        set_calls[led_index_list[0] + 5] = None
        set_calls[led_index_list[0] + 6] = None
        set_calls[led_index_list[0] + 7] = None
        set_calls[led_index_list[0] + 8] = None

        return set_calls

class AcctonAS6812_32XLED(cumulus.ledup.TridentTwoPlusLEDController):
    num_leds = 255
    default_pad = 0

class AcctonAS6812_32XSwitch(cumulus.platform.BCMSwitch):
    ledup = AcctonAS6812_32XLED
    _qsfp = AcctonAS6812_32XQSFPp

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp(label="1", serdes=(  0,   1,   2,   3), leds=(0, 111)),
        _qsfp(label="2", serdes=(  4,   5,   6,   7), leds=(0, 120)),
        _qsfp(label="3", serdes=(  8,   9,  10,  11), leds=(0, 129)),
        _qsfp(label="4", serdes=( 12,  13,  14,  15), leds=(0, 138)),
        _qsfp(label="5", serdes=( 16,  17,  18,  19), leds=(0, 147)),
        _qsfp(label="6", serdes=( 20,  21,  22,  23), leds=(0, 156)),
        _qsfp(label="7", serdes=( 24,  25,  26,  27), leds=(0, 165)),
        _qsfp(label="8", serdes=( 28,  29,  30,  31), leds=(0, 174)),
        _qsfp(label="9", serdes=( 32,  33,  34,  35), leds=(0, 183)),
        _qsfp(label="10", serdes=( 36,  37,  38,  39), leds=(0, 192)),
        _qsfp(label="11", serdes=( 40,  41,  42,  43), leds=(0, 201)),
        _qsfp(label="12", serdes=( 44,  45,  46,  47), leds=(0, 210)),
        _qsfp(label="13", serdes=( 48,  49,  50,  51), leds=(0, 219)),
        _qsfp(label="14", serdes=( 52,  53,  54,  55), leds=(0, 228)),
        _qsfp(label="15", serdes=( 56,  57,  58,  59), leds=(0, 237)),
        _qsfp(label="16", serdes=( 60,  61,  62,  63), leds=(0, 246)),
        _qsfp(label="17", serdes=( 64,  65,  66,  67), leds=(1, 111)),
        _qsfp(label="18", serdes=( 68,  69,  70,  71), leds=(1, 120)),
        _qsfp(label="19", serdes=( 72,  73,  74,  75), leds=(1, 129)),
        _qsfp(label="20", serdes=( 76,  77,  78,  79), leds=(1, 138)),
        _qsfp(label="21", serdes=( 80,  81,  82,  83), leds=(1, 147)),
        _qsfp(label="22", serdes=( 84,  85,  86,  87), leds=(1, 156)),
        _qsfp(label="23", serdes=( 88,  89,  90,  91), leds=(1, 165)),
        _qsfp(label="24", serdes=( 92,  93,  94,  95), leds=(1, 174)),
        _qsfp(label="25", serdes=( 96,  97,  98,  99), leds=(1, 183)),
        _qsfp(label="26", serdes=(100, 101, 102, 103), leds=(1, 192)),
        _qsfp(label="27", serdes=(104, 105, 106, 107), leds=(1, 201)),
        _qsfp(label="28", serdes=(108, 109, 110, 111), leds=(1, 210)),
        _qsfp(label="29", serdes=(112, 113, 114, 115), leds=(1, 219)),
        _qsfp(label="30", serdes=(116, 117, 118, 119), leds=(1, 228)),
        _qsfp(label="31", serdes=(120, 121, 122, 123), leds=(1, 237)),
        _qsfp(label="32", serdes=(124, 125, 126, 127), leds=(1, 246)),
    )

class AcctonAS6812_32X(cumulus.platform.Platform):
    name = 'accton,as6812_32x'
    chip = cumulus.platform.TridentTwoPlus_56860_Chip()
    switch = AcctonAS6812_32XSwitch(chip)
    cpld = AcctonAS6712_32XCPLD()
    statusleds = AcctonAS6712_32XLEDs()
    pwms = AcctonAS6712_32XPWMs()
    sensors = AcctonAS6712_32XSensors()

Platform = AcctonAS6812_32X
