#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class Accton5652QSFPp(cumulus.platform.Trident40GPort):
    def __init__(self, label, serdes, phy_addrs, leds):
        cumulus.platform.Trident40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=("84740",) * 4,
                                                 phy_addrs=phy_addrs,
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
    _set_led = '''
;;; Accton5652SFPp
;;;
;;; XXX - this is wrong for subdivided ports, need to OR all four ports together
;;;
;;; Two bits per port:
;;;
;;; 1st bit pushed on stack controls amber LED:
;;; 0 - off
;;; 1 - on
;;;
;;; 2n bit pushed on stack controls green LED:
;;; 0 - off
;;; 1 - on
;;;
;;; We will only use green LED.  1st bit pushed is always zero to
;;; disable amber LED.
set_led:
        port 1
        pushst  PORTSTATUS_ZERO ; disable amber LED
        pack

        port    a
        ld      b,LINKSCAN_P
        add     b,a
        ld      b,(b)
        tst     b,LINKSCAN_TX
        push    cy
        tst     b,LINKSCAN_RX
        push    cy
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret
'''

    _set_led_4x10 = '''
;;;
;;; Input : register A, logical port
;;; Input / Output : register B, accumulated port status

;;; For each port accumulate 2 bits of information in register B:
;;;   register B[0] - activity (TX _OR_ RX)
;;;   register B[1] - link enabled

SUB_INTF_ACTIVITY	equ	0x0  ; activity
SUB_INTF_LINKEN		equ	0x1  ; link enabled

sub_intf_info:

        ;;; push OR of current TX/RX activity onto stack
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor

        ;;; push accumlated link activity onto stack
        tst B, SUB_INTF_ACTIVITY
        push cy

        ;;; OR accumlated activity with current activity and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_ACTIVITY

        ;;; push current LINKEN onto stack
        pushst  PORTSTATUS_LINKEN

        ;;; push accumlated link enable onto stack
        tst B, SUB_INTF_LINKEN
        push cy

        ;;; OR accumlated LINKEN with current LINKEN and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_LINKEN

        ret

;;; AcctonAS5652QSFPp_4x10
;;; OR together status of 4 ports for 4x10G QSFP port
;;; Input: register A - first logical port of 4-port group
;;; Modified: register B - used as scratch space
set_led:

        port 1
        pushst  PORTSTATUS_ZERO ; disable yellow LED
        pack

        ;;; initialize B
        ld	b,0

        ;;; load sub-0 info
        call	sub_intf_info

        ;;; load sub-1 info
        inc	a
        call	sub_intf_info

        ;;; load sub-2 info
        inc	a
        call	sub_intf_info

        ;;; load sub-3 info
        inc	a
        call	sub_intf_info

        ;;; push accumlated link activity onto stack
        tst	B, SUB_INTF_ACTIVITY
        push	cy

        ;;; combine with BLINK state
        push	(BLINK_STATE_P)
        tand
        tinv	; flash off with activity

        ;;; push accumlated link enable onto stack
        tst	B, SUB_INTF_LINKEN
        push	cy

        ;;; AND activity-blink state with accumlated LINKEN
        tand

        pack

        ret
'''
    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, 0)
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None


        return set_calls


class Accton5652SFPp(cumulus.platform.Trident10GPort):
    _set_led = '''
;;; Accton5652SFPp
;;;
;;; Two bits per port:
;;;
;;; 1st bit pushed on stack controls amber LED:
;;; 0 - off
;;; 1 - on
;;;
;;; 2n bit pushed on stack controls green LED:
;;; 0 - off
;;; 1 - on
;;;
;;; We will only use green LED.  1st bit pushed is always zero to
;;; disable amber LED.
set_led:
        port 1
        pushst  PORTSTATUS_ZERO ; disable amber LED
        pack

        port    a
        ld      b,LINKSCAN_P
        add     b,a
        ld      b,(b)
        tst     b,LINKSCAN_TX
        push    cy
        tst     b,LINKSCAN_RX
        push    cy
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret
'''

    def __init__(self, label, serdes, phy_addr, leds):
        cumulus.platform.Trident10GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_type="84754",
                                                 phy_addr=phy_addr,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.hw_port_num(mode, 0))
        for led in led_index_list:
            set_calls[led] = call
            set_calls[led + 1] = None

        return set_calls

class Accton5652CPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/ff705000.localbus/ea000000.cpld'

class Accton5652LED(cumulus.ledup.TridentLEDController):
    num_leds = 52

class Accton5652Switch(cumulus.platform.BCMSwitch):
    ledup = Accton5652LED
    _sfp = Accton5652SFPp
    _qsfp = Accton5652QSFPp
    has_phy_ucode_mdio = True
    # This board does not have SPI-ROMs.
    has_phy_ucode_spi_rom = False
    ports = (
        _sfp(label="1",  serdes=64, phy_addr=0x00, leds=(1, 14)),
        _sfp(label="2",  serdes=65, phy_addr=0x01, leds=(1, 12)),
        _sfp(label="3",  serdes=66, phy_addr=0x02, leds=(1, 10)),
        _sfp(label="4",  serdes=67, phy_addr=0x03, leds=(1, 8)),
        _sfp(label="5",  serdes=8,  phy_addr=0x04, leds=(0, 42)),
        _sfp(label="6",  serdes=9,  phy_addr=0x05, leds=(0, 40)),
        _sfp(label="7",  serdes=10, phy_addr=0x06, leds=(0, 38)),
        _sfp(label="8",  serdes=11, phy_addr=0x07, leds=(0, 36)),
        _sfp(label="9",  serdes=60, phy_addr=0x08, leds=(1, 22)),
        _sfp(label="10", serdes=61, phy_addr=0x09, leds=(1, 20)),
        _sfp(label="11", serdes=62, phy_addr=0x0a, leds=(1, 18)),
        _sfp(label="12", serdes=63, phy_addr=0x0b, leds=(1, 16)),
        _sfp(label="13", serdes=68, phy_addr=0x0c, leds=(1, 6)),
        _sfp(label="14", serdes=69, phy_addr=0x0d, leds=(1, 4)),
        _sfp(label="15", serdes=70, phy_addr=0x0e, leds=(1, 2)),
        _sfp(label="16", serdes=71, phy_addr=0x0f, leds=(1, 0)),
        _sfp(label="17", serdes=12, phy_addr=0x10, leds=(0, 34)),
        _sfp(label="18", serdes=13, phy_addr=0x11, leds=(0, 32)),
        _sfp(label="19", serdes=14, phy_addr=0x12, leds=(0, 30)),
        _sfp(label="20", serdes=15, phy_addr=0x13, leds=(0, 28)),
        _sfp(label="21", serdes=4,  phy_addr=0x34, leds=(0, 50)),
        _sfp(label="22", serdes=5,  phy_addr=0x35, leds=(0, 48)),
        _sfp(label="23", serdes=6,  phy_addr=0x36, leds=(0, 46)),
        _sfp(label="24", serdes=7,  phy_addr=0x37, leds=(0, 44)),
        _sfp(label="25", serdes=16, phy_addr=0x38, leds=(0, 26)),
        _sfp(label="26", serdes=17, phy_addr=0x39, leds=(0, 24)),
        _sfp(label="27", serdes=18, phy_addr=0x3a, leds=(0, 22)),
        _sfp(label="28", serdes=19, phy_addr=0x3b, leds=(0, 20)),
        _sfp(label="29", serdes=20, phy_addr=0x3c, leds=(0, 18)),
        _sfp(label="30", serdes=21, phy_addr=0x3d, leds=(0, 16)),
        _sfp(label="31", serdes=22, phy_addr=0x3e, leds=(0, 14)),
        _sfp(label="32", serdes=23, phy_addr=0x3f, leds=(0, 12)),
        _sfp(label="33", serdes=40, phy_addr=0x20, leds=(1, 48)),
        _sfp(label="34", serdes=41, phy_addr=0x21, leds=(1, 46)),
        _sfp(label="35", serdes=42, phy_addr=0x22, leds=(1, 44)),
        _sfp(label="36", serdes=43, phy_addr=0x23, leds=(1, 42)),
        _sfp(label="37", serdes=24, phy_addr=0x24, leds=(0, 10)),
        _sfp(label="38", serdes=25, phy_addr=0x25, leds=(0, 8)),
        _sfp(label="39", serdes=26, phy_addr=0x26, leds=(0, 6)),
        _sfp(label="40", serdes=27, phy_addr=0x27, leds=(0, 4)),
        _sfp(label="41", serdes=48, phy_addr=0x48, leds=(1, 32)),
        _sfp(label="42", serdes=49, phy_addr=0x49, leds=(1, 30)),
        _sfp(label="43", serdes=50, phy_addr=0x4a, leds=(1, 28)),
        _sfp(label="44", serdes=51, phy_addr=0x4b, leds=(1, 26)),
        _sfp(label="45", serdes=44, phy_addr=0x4c, leds=(1, 40)),
        _sfp(label="46", serdes=45, phy_addr=0x4d, leds=(1, 38)),
        _sfp(label="47", serdes=46, phy_addr=0x4e, leds=(1, 36)),
        _sfp(label="48", serdes=47, phy_addr=0x4f, leds=(1, 34)),

        _qsfp(label="49", serdes=(36,37,38,39), phy_addrs=(0x54,0x55,0x56,0x57),
              leds=(1, 50)),
        _qsfp(label="50", serdes=(28,29,30,31), phy_addrs=(0x50,0x51,0x52,0x53),
              leds=(0, 2)),
        _qsfp(label="51", serdes=(56,57,58,59), phy_addrs=(0x5c,0x5d,0x5e,0x5f),
              leds=(1, 24)),
        _qsfp(label="52", serdes=(32,33,34,35), phy_addrs=(0x58,0x59,0x5a,0x5b),
              leds=(0, 0)),
    )


class Accton5652Board(cumulus.sensors.Board_Unit):
    name = 'Board'
    cpld_hwmon = ['system']

class Accton5652PSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class Accton5652PSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

Accton5652_9_29_path = '/sys/devices/soc.0/ff703000.i2c/i2c-0/i2c-9/9-0029'
Accton5652_9_18_path = '/sys/devices/soc.0/ff703000.i2c/i2c-0/i2c-9/9-0018'
Accton5652_9_1a_path = '/sys/devices/soc.0/ff703000.i2c/i2c-0/i2c-9/9-001a'
Accton5652_9_4c_path = '/sys/devices/soc.0/ff703000.i2c/i2c-0/i2c-9/9-004c'
Accton5652_pci_0100_path = '/sys/devices/pci0000:00/0000:00:00.0/0000:01:00.0'

# Temp sensor parameters provided by Accton seem conservative. In our lab and
# at customer sites, we see that fans are running at full speed for normal operation.
# Collecting some temperature sensor data from our lab at normal operation, this box
# just runs at a higher temperature than other boxes, and running the fan at full speed,
# doesnt seem to change much. Hence raising the limits for all sensors. (CM-4335)
class Accton5652Temp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 57
    temp_max = 77
    temp_crit = 90

    def __init__(self):
        self.name =  (self.__class__.__name__[10:])
        cumulus.sensors.Temp_Unit.__init__(self)

# Temp values added below are an approximation to the values
# provided by Accton. They have 2 models: B2F and F2B. I have taken
# the average of the values provided and avoided the extremes.
class Accton5652Temp1(Accton5652Temp):
    description = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path = Accton5652_pci_0100_path
    temp_fan_min = 65
    temp_max = 100
    temp_crit = 105

class Accton5652Temp2(Accton5652Temp):
    description = 'Near the CPU (Right)'
    driver_path = Accton5652_9_29_path

class Accton5652Temp3(Accton5652Temp):
    description = 'Top right corner'
    driver_path = Accton5652_9_29_path
    driver_hwmon = ['temp2']

class Accton5652Temp4(Accton5652Temp):
    description = 'Right side of Networking ASIC'
    driver_path = Accton5652_9_29_path
    driver_hwmon = ['temp3']

class Accton5652Temp5(Accton5652Temp):
    description = 'Middle of the board'
    driver_path = Accton5652_9_18_path

class Accton5652Temp6(Accton5652Temp):
    description = cumulus.sensors.Temp_Unit.p2020_cpu_sensor_name
    driver_path = Accton5652_9_18_path
    driver_hwmon = ['temp2']
    temp_fan_min = 65
    temp_max = 100
    #P2020 (Tj) Temperature Operating range is 0-125 C
    temp_crit = 120

class Accton5652Temp7(Accton5652Temp):
    description = 'Left side of the board'
    driver_path = Accton5652_9_1a_path

class Accton5652Temp8(Accton5652Temp):
    description = 'Left side of the board'
    driver_path = Accton5652_9_1a_path
    driver_hwmon = ['temp2']

class Accton5652Temp9(Accton5652Temp):
    description = 'Right side of the board'
    driver_path = Accton5652_9_4c_path

class Accton5652Temp10(Accton5652Temp):
    description = 'Right side of the board'
    driver_path = Accton5652_9_4c_path
    driver_hwmon = ['temp2']

class Accton5652Fan(cumulus.sensors.Fan_Unit):
    name = 'Fan'
    cpld_hwmon = ['system_fan']
    pwm_path = Accton5652_9_29_path

class Accton5652Sensors():
    units = ( Accton5652Board(),
              Accton5652PSU1(),
              Accton5652PSU2(),
              Accton5652Temp1(),
              Accton5652Temp2(),
              Accton5652Temp3(),
              Accton5652Temp4(),
              Accton5652Temp5(),
              Accton5652Temp6(),
              Accton5652Temp7(),
              Accton5652Temp8(),
              Accton5652Temp9(),
              Accton5652Temp10(),
              Accton5652Fan(),
              )
class Accton5652PWM1(cumulus.sensors.PWM_Unit):
    fans = (Accton5652Fan(),
            )
    temps = ( Accton5652Temp1(),
              Accton5652Temp2(),
              Accton5652Temp3(),
              Accton5652Temp4(),
              Accton5652Temp5(),
              Accton5652Temp6(),
              Accton5652Temp7(),
              Accton5652Temp8(),
              Accton5652Temp9(),
              Accton5652Temp10(),
              )

class Accton5652PWMs():
    units = (Accton5652PWM1(),
             )

class Accton5652PSU1LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU1'
    sources = ['PSU1']
    led_location = 'led_psu1'

class Accton5652PSU2LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU2'
    sources = ['PSU2']
    led_location = 'led_psu2'

class Accton5652FanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan'
    sources = ['Fan']
    led_location = 'led_fan'

class Accton5652SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_diag'

class Accton5652LEDs():
    units = (Accton5652PSU1LED(),
             Accton5652PSU2LED(),
             Accton5652FanLED(),
             Accton5652SystemLED(),
             )

class Accton5652(cumulus.platform.Platform):
    name = 'accton,es5652bt1'
    cpld = Accton5652CPLD()
    chip = cumulus.platform.TridentPlusChip()
    switch = Accton5652Switch(chip)
    sensors = Accton5652Sensors()
    pwms = Accton5652PWMs()
    statusleds = Accton5652LEDs()

Platform = Accton5652
