#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class CelBigStoneG100Fabric(cumulus.platform.Tomahawk100GPort):
    '''
    Fabric ports are 100G Tomahawk ports using KR4 serdes and
    not LEDs.
    '''
    is_fabric = True
    create_fabric = True
    def __init__(self, label, serdes, tx_polarity_flip,
                 rx_polarity_flip, tx_lane_map, rx_lane_map):
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=None,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=None,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   oversub=True,
                                                   unit=0)
        self._serdes_intf_map = { 100 : 'KR4' }

class CelBigStoneG100QSFP28(cumulus.platform.Tomahawk100GPort):
    '''
    QSFP28 ports have 12 LED bits per port.
    '''
    _set_led_common = '''
;;;
;;; common LED routines
;;;
set_port_led:
        port    a                   ; The port upon which we are operating

        push    1                   ; First bit (always on)
        pack

        pushst  PORTSTATUS_RX       ; Any RX or TX activity recently?
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)     ; If we are blinking, turn it off
        tand
        tinv
        pushst  PORTSTATUS_LINKEN   ; Is there any link on this port?
        tand
        tinv                        ; Invert for correct color.
        pack                        ; Green = 101, Off = 111

        push    1                   ; Last bit, (always on)
        pack

        ret

set_led_off:
        push    1
        pack
        push    1
        pack
        push    1
        pack
        ret
'''

    _set_led_1x = '''
;;;
;;; In 1x mode, only the first 3 bits of the LED chain set the LED color.
;;;
set_led:
        call    set_port_led        ; Set the first LED in the chain
        call    set_led_off         ; The rest are ignored
        call    set_led_off
        call    set_led_off
        ret
'''

    _set_led_2x = '''
;;;
;;; In 2x mode, the first and third 3 bits of the LED chain set the LED color.
;;;
set_led:
        call    set_port_led        ; Set the first LED in the chain
        call    set_led_off         ; These three bits are ignored
        add     a, 2
        call    set_port_led        ; Set the third LED in the chain
        call    set_led_off         ; These three bits are ignored
        ret
'''

    _set_led_4x = '''
;;;
;;; In 4x mode, the all 3-bit LED values in the LED chain must be set.
;;; indicate link/activity.
;;;
set_led:
        ld      b, 4
set_next_led:
        call    set_port_led        ; Set the 3-bit port LED
        inc     a
        dec     b
        jnz     set_next_led
        ret
'''

    def __init__(self, label, serdes, leds, tx_polarity_flip, 
                 rx_polarity_flip, tx_lane_map, rx_lane_map):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=leds,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        for led in led_index_list:
            set_calls[led] = call
            for x in range(1,12):
                set_calls[led + x] = None

        return set_calls

class CelBigStoneG100LED0(cumulus.ledup.TomahawkLEDController):
    led_functions = (CelBigStoneG100QSFP28._set_led_common)
    num_leds = 144

class CelBigStoneG100LED1(cumulus.ledup.TomahawkLEDController):
    led_functions = (CelBigStoneG100QSFP28._set_led_common)
    num_leds = 48

class CelBigStoneG100Switch(cumulus.platform.BCMSwitch):
    ledup = [ CelBigStoneG100LED0, CelBigStoneG100LED1 ]
    led_controllers = set([0, 1]) # 2 is unused
    _qsfp28 = CelBigStoneG100QSFP28
    _fab = CelBigStoneG100Fabric
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (

        # Tomahawk front panel ports
        _qsfp28(label="17", serdes=( 80,  81,  82,  83), leds=(1,   0),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0000,
              tx_lane_map=0x2031, rx_lane_map=0x2031),
        _qsfp28(label="18", serdes=( 84,  85,  86,  87), leds=(1,  12),
              tx_polarity_flip=0x0003, rx_polarity_flip=0x0000,
              tx_lane_map=0x1302, rx_lane_map=0x1302),
        _qsfp28(label="19", serdes=( 88,  89,  90,  91), leds=(1,  24),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0000,
              tx_lane_map=0x2031, rx_lane_map=0x2031),
        _qsfp28(label="20", serdes=( 92,  93,  94,  95), leds=(1,  36),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x1302),
        _qsfp28(label="21", serdes=( 96,  97,  98,  99), leds=(0,   0),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
              tx_lane_map=0x2031, rx_lane_map=0x2031),
        _qsfp28(label="22", serdes=(100, 101, 102, 103), leds=(0,  12),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x0000,
              tx_lane_map=0x1302, rx_lane_map=0x1302),
        _qsfp28(label="23", serdes=(104, 105, 106, 107), leds=(0,  24),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
              tx_lane_map=0x2031, rx_lane_map=0x2031),
        _qsfp28(label="24", serdes=(108, 109, 110, 111), leds=(0,  36),
              tx_polarity_flip=0x0003, rx_polarity_flip=0x0000,
              tx_lane_map=0x1302, rx_lane_map=0x1302),
        _qsfp28(label="25", serdes=(112, 113, 114, 115), leds=(0,  48),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
              tx_lane_map=0x0321, rx_lane_map=0x3012),
        _qsfp28(label="26", serdes=(116, 117, 118, 119), leds=(0,  60),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x3120),
        _qsfp28(label="27", serdes=(120, 121, 122, 123), leds=(0,  72),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x3120),
        _qsfp28(label="28", serdes=(124, 125, 126, 127), leds=(0,  84),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,
              tx_lane_map=0x2031, rx_lane_map=0x0213),
        _qsfp28(label="29", serdes=(  0,   1,   2,   3), leds=(0,  96),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x3120),
        _qsfp28(label="30", serdes=(  4,   5,   6,   7), leds=(0, 108),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x3120),
        _qsfp28(label="31", serdes=(  8,   9,  10,  11), leds=(0, 120),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x3120),
        _qsfp28(label="32", serdes=( 12,  13,  14,  15), leds=(0, 132),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x3120),

        # Line Card Tomahawk 2 backplane ports to fab #1
        _fab(label="0",  serdes=( 64,  65,  66,  67),
              tx_polarity_flip=0x0001, rx_polarity_flip=0x0000,
              tx_lane_map=0x1203, rx_lane_map=0x1023),
        _fab(label="1",  serdes=( 68,  69,  70,  71),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x0123),
        _fab(label="2",  serdes=( 72,  73,  74,  75),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x3210, rx_lane_map=0x0123),
        _fab(label="3",  serdes=( 76,  77,  78,  79),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3102, rx_lane_map=0x2013),

        # Line Card Tomahawk 2 backplane ports to fab #2
        _fab(label="4",  serdes=( 48,  49,  50,  51),
              tx_polarity_flip=0x0001, rx_polarity_flip=0x000f,
              tx_lane_map=0x2130, rx_lane_map=0x2310),
        _fab(label="5",  serdes=( 52,  53,  54,  55),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x0123),
        _fab(label="6",  serdes=( 56,  57,  58,  59),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x0123),
        _fab(label="7",  serdes=( 60,  61,  62,  63),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x0213),

        # Line Card Tomahawk 2 backplane ports to fab #3
        _fab(label="8",  serdes=( 32,  33,  34,  35),
              tx_polarity_flip=0x000e, rx_polarity_flip=0x0000,
              tx_lane_map=0x1203, rx_lane_map=0x2310),
        _fab(label="9",  serdes=( 36,  37,  38,  39),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _fab(label="10", serdes=( 40,  41,  42,  43),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x0123, rx_lane_map=0x0123),
        _fab(label="11", serdes=( 44,  45,  46,  47),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        
        # Line Card Tomahawk 2 backplane ports to fab #4
        _fab(label="12", serdes=( 16,  17,  18,  19),
              tx_polarity_flip=0x000e, rx_polarity_flip=0x0000,
              tx_lane_map=0x1203, rx_lane_map=0x2310),
        _fab(label="13", serdes=( 20,  21,  22,  23),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _fab(label="14", serdes=( 24,  25,  26,  27),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x0123, rx_lane_map=0x0123),
        _fab(label="15", serdes=( 28,  29,  30,  31),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
    )

class CelBigStoneG100(cumulus.platform.Platform):
    name = 'cel,bigstone_g_lc1r'
    chip = cumulus.platform.TomahawkChip()
    switch = CelBigStoneG100Switch(chip)
    cpld = None
    statusleds = None
    pwms = None
    sensors = None

Platform = CelBigStoneG100
