#
# Copyright 2016.  Cumulus Networks, Inc.
#

import subprocess
import json
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class CelRedstoneXpSFPp(cumulus.platform.TridentTwo10GPort):

    _set_led = '''
;;; CelRedstoneXpSFPp
;;
;;; Each port on the Redstone XP uses two bits (SFP and QSFP), they
;;;  behave as follows:
;;;
;;;                  bit 0
;;;                0        1
;;;            +--------+-------+
;;;          0 | green  |  off  |
;;;   bit 1    +--------+-------+
;;;          1 |  red   |  off  |
;;;            +--------+-------+
set_led:
set_any_led:
;;; The first bit will indicate link/activity (low active)
;;; The second bit will always be 0
;;;
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        port 1
        pushst  PORTSTATUS_ZERO ; disable yellow LED
        pack

        ret
'''
    def __init__(self, label, serdes, leds, rx_polarity_flip=None, tx_polarity_flip=None):
        cumulus.platform.TridentTwo10GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=None,
                                                 phy_addrs=None,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 rx_lane_map=0x3210,
                                                 tx_lane_map=0x1032,
                                                 rx_polarity_flip=rx_polarity_flip,
                                                 tx_polarity_flip=tx_polarity_flip,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None

        return set_calls

class CelRedstoneXpQSFPp(cumulus.platform.TridentTwo40GPort):
    _set_led = '''
;;; CelRedstoneXpQSFPp
;;;
;;; For 40G mode, we use only the first LED to indicate link/activity.
;;; The other 3 LEDs will be off.
;;;
set_led:
        call    set_any_led    ; set the first led green/off

        push    PORTSTATUS_ONE ; 1, 1 => led 2 off
        pack
        push    PORTSTATUS_ONE
        pack

        push    PORTSTATUS_ONE ; 1, 1 => led 3 off
        pack
        push    PORTSTATUS_ONE
        pack

        push    PORTSTATUS_ONE ; 1, 1 => led 4 off
        pack
        push    PORTSTATUS_ONE
        pack

        ret
'''
    _set_led_4x10 = '''
;;; CelRedstoneXpQSFPp_4x10
;;
;;  For 4x10G mode, we use an LED for each port.
;;
set_led:
        call    set_any_led
        inc     a
        call    set_any_led
        inc     a
        call    set_any_led
        inc     a
        call    set_any_led

        ret
'''
    def __init__(self, label, serdes, leds, tx_lane_map, rx_polarity_flip=None, tx_polarity_flip=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.TridentTwo40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=None,
                                                 phy_addrs=None,
                                                 phy_layer=sfp.type.upper(),
                                                 leds=leds,
                                                 rx_lane_map=0x3210,
                                                 rx_polarity_flip=rx_polarity_flip,
                                                 tx_lane_map=tx_lane_map,
                                                 tx_polarity_flip=tx_polarity_flip,
                                                 unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None


        return set_calls

class CelRedstoneXpLED(cumulus.ledup.TridentTwoLEDController):
    num_leds = 72

class CelRedstoneXpCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/cel_redstone_xp_cpld.0'

'''
   Celestica Redstone XP LEDs
       System:
         green - all systems ok
         green blinking - fan missing or broken, psu missing or broken
       PSU1:
         green - psu1 installed and working
         off - psu1 missing or broken
       PSU2:
         green - psu2 installed and working
         off - psu2 missing or broken
       Fan1:
         green - fan installed and working
         off - fan missing or broken
       Fan2:
         green - fan installed and working
         off - fan missing or broken
       Fan3:
         green - fan installed and working
         off - fan missing or broken
       Fan4:
         green - fan installed and working
         off - fan missing or broken
'''

class CelRedstoneXpSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_system'
    bad_led_color = 'green_blinking'

class CelRedstoneXpPsu1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Psu1'
    sources = ['PSU1']
    led_location = 'led_psu1'
    bad_led_color = 'off'

class CelRedstoneXpPsu2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Psu2'
    sources = ['PSU2']
    led_location = 'led_psu2'
    bad_led_color = 'off'

class CelRedstoneXpFanLED(cumulus.sensors.StatusLED_Unit):
    good_led_color = 'green'
    bad_led_color  = 'amber'
    green_colormap = {'green':'1',  'amber':'1',  'red':'0',  'off':'0'}
    red_colormap   = {'green':'0',  'amber':'1',  'red':'1',  'off':'0'}
    colormap       = {'10':'green', '11':'amber', '01':'red', '00':'off'}

    def led_update(self, color):
        self.set_gpio_value(self.led_location[0], self.green_colormap[color])
        self.set_gpio_value(self.led_location[1], self.red_colormap[color])

    def led_read(self):
        green_color = self.get_gpio_value(self.led_location[0])
        red_color = self.get_gpio_value(self.led_location[1])
        return self.name, self.colormap[green_color + red_color]

class CelRedstoneXpFan1LED(CelRedstoneXpFanLED):
    name = 'Fan Tray 1'
    sources = ['Fan1', 'Fan5']
    led_location = ['led_fan1_green', 'led_fan1_red']

class CelRedstoneXpFan2LED(CelRedstoneXpFanLED):
    name = 'Fan Tray 2'
    sources = ['Fan2', 'Fan6']
    led_location = ['led_fan2_green', 'led_fan2_red']

class CelRedstoneXpFan3LED(CelRedstoneXpFanLED):
    name = 'Fan Tray 3'
    sources = ['Fan3', 'Fan7']
    led_location = ['led_fan3_green', 'led_fan3_red']

class CelRedstoneXpFan4LED(CelRedstoneXpFanLED):
    name = 'Fan Tray 4'
    sources = ['Fan4', 'Fan8']
    led_location = ['led_fan4_green', 'led_fan4_red']

class CelRedstoneXpTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = 'Intel CPU external sensor'
    driver_path = '/sys/bus/i2c/devices/14-0048'
    temp_crit = 70
    temp_max = 65

class CelRedstoneXpTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Rear Outlet Air sensor'
    driver_path = '/sys/bus/i2c/devices/15-004e'
    temp_crit = 60
    temp_max = 48
    temp_min = -10
    temp_lcrit = -15

class CelRedstoneXpTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Front Outlet Air sensor'
    driver_path = '/sys/bus/i2c/devices/24-0049'
    temp_crit = 60
    temp_max = 48
    temp_min = -10
    temp_lcrit = -15

class CelRedstoneXpTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = '/sys/bus/i2c/devices/25-004a'
    temp_crit = 80
    temp_max = 75

CelRedstoneXp_core_path = '/sys/devices/platform/coretemp.0'
class CelRedstoneXpTemp5(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    description = 'Intel CPU die sensor'
    driver_path = CelRedstoneXp_core_path
    driver_hwmon = ['temp2']
    temp_crit = 98

class CelRedstoneXpTemp6(cumulus.sensors.Temp_Unit):
    name = 'Temp6'
    description = 'Intel CPU die sensor'
    driver_path = CelRedstoneXp_core_path
    driver_hwmon = ['temp3']
    temp_crit = 98

class CelRedstoneXpTemp7(cumulus.sensors.Temp_Unit):
    name = 'Temp7'
    description = 'Intel CPU die sensor'
    driver_path = CelRedstoneXp_core_path
    driver_hwmon = ['temp4']
    temp_crit = 98

class CelRedstoneXpTemp8(cumulus.sensors.Temp_Unit):
    name = 'Temp8'
    description = 'Intel CPU die sensor'
    driver_path = CelRedstoneXp_core_path
    driver_hwmon = ['temp5']
    temp_crit = 98

CelRedstoneXp_13_4d_path = '/sys/bus/i2c/devices/13-004d'
CelRedstoneXp_13_2e_path = '/sys/bus/i2c/devices/13-002e'

class CelRedstoneXpFan(cumulus.sensors.Fan_Unit):
    fan_min = 6300
    fan_max = 21000
    hwmon_search = False

    def find_and_add_attrs(self):
        if self.get_gpio_value(self.fan_present) == '1':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class CelRedstoneXpFan1(CelRedstoneXpFan):
    name = 'Fan1'
    description = 'Fan Tray 1'
    driver_path = CelRedstoneXp_13_4d_path
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    fan_present = 'fan1_present'

class CelRedstoneXpFan2(CelRedstoneXpFan):
    name = 'Fan2'
    description = 'Fan Tray 2'
    driver_path = CelRedstoneXp_13_4d_path
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    fan_present = 'fan2_present'

class CelRedstoneXpFan3(CelRedstoneXpFan):
    name = 'Fan3'
    description = 'Fan Tray 3'
    driver_path = CelRedstoneXp_13_4d_path
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']
    fan_present = 'fan3_present'

class CelRedstoneXpFan4(CelRedstoneXpFan):
    name = 'Fan4'
    description = 'Fan Tray 4'
    driver_path = CelRedstoneXp_13_4d_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_present = 'fan4_present'

class CelRedstoneXpFan5(CelRedstoneXpFan):
    name = 'Fan5'
    description = 'Fan Tray 1'
    fan_min = 5600
    fan_max = 17000
    driver_path = CelRedstoneXp_13_2e_path
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    fan_present = 'fan1_present'

class CelRedstoneXpFan6(CelRedstoneXpFan):
    name = 'Fan6'
    description = 'Fan Tray 2'
    fan_min = 5600
    fan_max = 17000
    driver_path = CelRedstoneXp_13_2e_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_present = 'fan2_present'

class CelRedstoneXpFan7(CelRedstoneXpFan):
    name = 'Fan7'
    description = 'Fan Tray 3'
    fan_min = 5600
    fan_max = 17000
    driver_path = CelRedstoneXp_13_2e_path
    driver_hwmon = ['fan5']
    pwm_hwmon = ['pwm5']
    fan_present = 'fan3_present'

class CelRedstoneXpFan8(CelRedstoneXpFan):
    name = 'Fan8'
    description = 'Fan Tray 4'
    fan_min = 5600
    fan_max = 17000
    driver_path = CelRedstoneXp_13_2e_path
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']
    fan_present = 'fan4_present'

class CelRedstoneXp_PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)
        self.is_hw_managed = self._check_hw_mgmt()

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = 'rmmod dps460 && modprobe dps460'
            subprocess.call(cmd, shell=True)
            self.init_done = True

    def find_and_add_attrs(self):
        if self.get_gpio_value(self.psu_present) == '1':
            self.attrs[self.psu_present] = 1
        else:
            self.attrs[self.psu_present] = 0
        if self.get_gpio_value(self.psu_all_ok) == '1':
            self.attrs[self.psu_all_ok] = 1
        else:
            self.attrs[self.psu_all_ok] = 0
        return 0

    def _check_hw_mgmt(self):
        try:
            data = json.loads(subprocess.check_output(['/usr/cumulus/bin/decode-syseeprom', '-j', '-t', self.name.lower()]))
            name = data.get('product_info').get('name').strip()
        except:
            name = ''
        if name in ['DPS550AB6A',]:
            return True
        return False

class CelRedstoneXpPSU1(CelRedstoneXp_PSU):
    name = 'PSU1'
    psu_present = 'psu_pwr1_present'
    psu_all_ok = 'psu_pwr1_all_ok'

class CelRedstoneXpPSU2(CelRedstoneXp_PSU):
    name = 'PSU2'
    psu_present = 'psu_pwr2_present'
    psu_all_ok = 'psu_pwr2_all_ok'

class CelRedstoneXpPSUTemp(cumulus.sensors.Temp_Unit):
    def find_and_add_attrs(self):
        if ((self.get_gpio_value(self.temp_present) == '1') and
            (self.get_gpio_value(self.temp_all_ok) == '1')):
            return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)
        else:
            return -1

class CelRedstoneXpTemp9(CelRedstoneXpPSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Inlet Temp Sensor'
    driver_path = '/sys/bus/i2c/devices/11-0059'
    driver_hwmon = ['temp1']
    temp_present = 'psu_pwr2_present'
    temp_all_ok = 'psu_pwr2_all_ok'
    temp_max = 60
    temp_crit = 100

class CelRedstoneXpTemp10(CelRedstoneXpPSUTemp):
    name = 'PSU2Temp2'
    description = 'PSU2 Max Temp Sensor'
    driver_path = '/sys/bus/i2c/devices/11-0059'
    driver_hwmon = ['temp2']
    temp_present = 'psu_pwr2_present'
    temp_all_ok = 'psu_pwr2_all_ok'
    temp_max = 60
    temp_crit = 100

class CelRedstoneXpTemp11(CelRedstoneXpPSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Inlet Temp Sensor'
    driver_path = '/sys/bus/i2c/devices/10-0058'
    driver_hwmon = ['temp1']
    temp_present = 'psu_pwr1_present'
    temp_all_ok = 'psu_pwr1_all_ok'
    temp_max = 60
    temp_crit = 100

class CelRedstoneXpTemp12(CelRedstoneXpPSUTemp):
    name = 'PSU1Temp2'
    description = 'PSU1 Max Temp Sensor'
    driver_path = '/sys/bus/i2c/devices/10-0058'
    driver_hwmon = ['temp2']
    temp_present = 'psu_pwr1_present'
    temp_all_ok = 'psu_pwr1_all_ok'
    temp_max = 60
    temp_crit = 100

class CelRedstoneXpPSUFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    minpwm = 102
    fan_max = 18000
    fan_min = 7000

    def find_and_add_attrs(self):
        if ((self.get_gpio_value(self.fan_present) == '1') and
            (self.get_gpio_value(self.fan_all_ok) == '1')):
            return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)
        else:
            return -1

class CelRedstoneXpFan9(CelRedstoneXpPSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = '/sys/bus/i2c/devices/11-0059'
    fan_present = 'psu_pwr2_present'
    fan_all_ok = 'psu_pwr2_all_ok'
    if CelRedstoneXpPSU2().is_hw_managed:
        fan_min = 4000

class CelRedstoneXpFan10(CelRedstoneXpPSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = '/sys/bus/i2c/devices/10-0058'
    fan_present = 'psu_pwr1_present'
    fan_all_ok = 'psu_pwr1_all_ok'
    if CelRedstoneXpPSU1().is_hw_managed:
        fan_min = 4000

class CelRedstoneXpPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (CelRedstoneXpFan1(),
            CelRedstoneXpFan2(),
            CelRedstoneXpFan3(),
            CelRedstoneXpFan4(),
            CelRedstoneXpFan5(),
            CelRedstoneXpFan6(),
            CelRedstoneXpFan7(),
            CelRedstoneXpFan8(),
    )
    temps = (CelRedstoneXpTemp1(),
             CelRedstoneXpTemp2(),
             CelRedstoneXpTemp3(),
             CelRedstoneXpTemp4(),
             CelRedstoneXpTemp5(),
             CelRedstoneXpTemp6(),
             CelRedstoneXpTemp7(),
             CelRedstoneXpTemp8(),
             )

class CelRedstoneXpPWM2(cumulus.sensors.PWM_Unit):
    name = 'PSU1 PWM'
    fans = (CelRedstoneXpFan10(),
            )
    temps = (CelRedstoneXpTemp11(),
             CelRedstoneXpTemp12(),
             )

class CelRedstoneXpPWM3(cumulus.sensors.PWM_Unit):
    name = 'PSU2 PWM'
    fans = (CelRedstoneXpFan9(),
            )
    temps = (CelRedstoneXpTemp9(),
             CelRedstoneXpTemp10(),
             )

class CelRedstoneXpSensors():
    units = (CelRedstoneXpPSU1(),
             CelRedstoneXpPSU2(),
             CelRedstoneXpTemp1(),
             CelRedstoneXpTemp2(),
             CelRedstoneXpTemp3(),
             CelRedstoneXpTemp4(),
             CelRedstoneXpTemp5(),
             CelRedstoneXpTemp6(),
             CelRedstoneXpTemp7(),
             CelRedstoneXpTemp8(),
             CelRedstoneXpTemp9(),
             CelRedstoneXpTemp10(),
             CelRedstoneXpTemp11(),
             CelRedstoneXpTemp12(),
             CelRedstoneXpFan1(),
             CelRedstoneXpFan2(),
             CelRedstoneXpFan3(),
             CelRedstoneXpFan4(),
             CelRedstoneXpFan5(),
             CelRedstoneXpFan6(),
             CelRedstoneXpFan7(),
             CelRedstoneXpFan8(),
             CelRedstoneXpFan9(),
             CelRedstoneXpFan10(),
            )

class CelRedstoneXpPWMs():

    units = (CelRedstoneXpPWM1(),)

    def __init__(self):
        # Some Redstone XP's (specially ones from SuperMicro), have PSUs where
        # the PSU fan speed in managed by hardware and cannot be managed by software elegantly.
        # More on this in CM-9627. So if the said PSU is there, dont manage the PSU fan from
        # PWM. In all other cases, the fan speed is managed by software.
        if not CelRedstoneXpPSU1().is_hw_managed:
            self.units = self.units + (CelRedstoneXpPWM2(),)

        if not CelRedstoneXpPSU2().is_hw_managed:
            self.units = self.units + (CelRedstoneXpPWM3(),)

class CelRedstoneXpLEDs():
    units = (CelRedstoneXpPsu1LED(),
             CelRedstoneXpPsu2LED(),
             CelRedstoneXpSystemLED(),
             CelRedstoneXpFan1LED(),
             CelRedstoneXpFan2LED(),
             CelRedstoneXpFan3LED(),
             CelRedstoneXpFan4LED(),
             )

class CelRedstoneXpSwitch(cumulus.platform.BCMSwitch):
    ledup = CelRedstoneXpLED
    _sfp = CelRedstoneXpSFPp
    _qsfp = CelRedstoneXpQSFPp

    has_phy_ucode_mdio = False # XXX unverified
    has_phy_ucode_spi_rom = False # XXX unverified

    ports = (
        _sfp(label="1",  serdes=(12,), leds=(0,  0), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="2",  serdes=(13,), leds=(0,  2)),
        _sfp(label="3",  serdes=(14,), leds=(0,  4), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="4",  serdes=(15,), leds=(0,  6)),
        _sfp(label="5",  serdes=(20,), leds=(0,  8), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="6",  serdes=(21,), leds=(0, 10)),
        _sfp(label="7",  serdes=(22,), leds=(0, 12), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="8",  serdes=(23,), leds=(0, 14)),
        _sfp(label="9",  serdes=(24,), leds=(0, 16), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="10", serdes=(25,), leds=(0, 18)),
        _sfp(label="11", serdes=(26,), leds=(0, 20), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="12", serdes=(27,), leds=(0, 22)),
        _sfp(label="13", serdes=(28,), leds=(0, 24), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="14", serdes=(29,), leds=(0, 26)),
        _sfp(label="15", serdes=(30,), leds=(0, 28), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="16", serdes=(31,), leds=(0, 30)),
        _sfp(label="17", serdes=(44,), leds=(0, 32), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="18", serdes=(45,), leds=(0, 34)),
        _sfp(label="19", serdes=(46,), leds=(0, 36), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="20", serdes=(47,), leds=(0, 38)),
        _sfp(label="21", serdes=(48,), leds=(0, 40), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="22", serdes=(49,), leds=(0, 42)),
        _sfp(label="23", serdes=(50,), leds=(0, 44), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="24", serdes=(51,), leds=(0, 46)),
        _sfp(label="25", serdes=(52,), leds=(0, 48), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="26", serdes=(53,), leds=(0, 50)),
        _sfp(label="27", serdes=(54,), leds=(0, 52), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="28", serdes=(55,), leds=(0, 54)),
        _sfp(label="29", serdes=(56,), leds=(0, 56), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="30", serdes=(57,), leds=(0, 58)),
        _sfp(label="31", serdes=(58,), leds=(0, 60), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="32", serdes=(59,), leds=(0, 62)),
        _sfp(label="33", serdes=(60,), leds=(0, 64), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="34", serdes=(61,), leds=(0, 66)),
        _sfp(label="35", serdes=(62,), leds=(0, 68), rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="36", serdes=(63,), leds=(0, 70)),
        _sfp(label="37", serdes=(64,), leds=(1,  0), rx_polarity_flip=True),
        _sfp(label="38", serdes=(65,), leds=(1,  2), tx_polarity_flip=True),
        _sfp(label="39", serdes=(66,), leds=(1,  4)),
        _sfp(label="40", serdes=(67,), leds=(1,  6), tx_polarity_flip=True),
        _sfp(label="41", serdes=(68,), leds=(1,  8)),
        _sfp(label="42", serdes=(69,), leds=(1, 10), tx_polarity_flip=True),
        _sfp(label="43", serdes=(70,), leds=(1, 12)),
        _sfp(label="44", serdes=(71,), leds=(1, 14), tx_polarity_flip=True),
        _sfp(label="45", serdes=(72,), leds=(1, 16)),
        _sfp(label="46", serdes=(73,), leds=(1, 18), tx_polarity_flip=True),
        _sfp(label="47", serdes=(74,), leds=(1, 20)),
        _sfp(label="48", serdes=(75,), leds=(1, 22), tx_polarity_flip=True),

        _qsfp(label="49", serdes=(76, 77, 78, 79), leds=(1, 24),
		tx_lane_map=0x3012, rx_polarity_flip=0xf, tx_polarity_flip=0x8),
        _qsfp(label="50", serdes=(80, 81, 82, 83), leds=(1, 32),
		tx_lane_map=0x3210, rx_polarity_flip=0xa, tx_polarity_flip=0xf),
        _qsfp(label="51", serdes=( 96, 97, 98, 99), leds=(1, 40),
		tx_lane_map=0x1032, rx_polarity_flip=0xf, tx_polarity_flip=0x0),
        _qsfp(label="52", serdes=( 100, 101, 102, 103), leds=(1, 48),
		tx_lane_map=0x3210, rx_polarity_flip=0xf, tx_polarity_flip=0xf),
        _qsfp(label="53", serdes=( 104, 105, 106, 107), leds=(1, 56),
		tx_lane_map=0x1032, rx_polarity_flip=0xf, tx_polarity_flip=0x0),
        _qsfp(label="54", serdes=( 108, 109, 110, 111), leds=(1, 64),
		tx_lane_map=0x1230, rx_polarity_flip=0xa, tx_polarity_flip=0x7),
    )


class CelRedstoneXp(cumulus.platform.Platform):
    name = 'cel,redstone_xp'
    chip = cumulus.platform.TridentTwo_56854_Chip()
    switch = CelRedstoneXpSwitch(chip)
    cpld = CelRedstoneXpCPLD()
    statusleds = CelRedstoneXpLEDs()
    pwms = CelRedstoneXpPWMs()
    sensors = CelRedstoneXpSensors()

Platform = CelRedstoneXp
