#
# Copyright 2016.  Cumulus Networks, Inc.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class CelSmallstoneXpQSFPp(cumulus.platform.TridentTwo40GPort):
    # _set_led_common defines LED functions common to both 40G and
    # 4x10G LED programs.
    _set_led_common = '''
set_any_led:
;;; The first bit will indicate link/activity (low active)
;;; The second bit will always be 0
;;;
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        port 1
        pushst  PORTSTATUS_ZERO
        pack

        ret
'''
    _set_led = '''
;;; CelSmallstoneXpQSFPp
;;; Each port on the Smallstone XP uses two bits, they
;;;  behave as follows:
;;;
;;;                  bit 0
;;;                0        1
;;;            +--------+-------+
;;;          0 | green  |  off  |
;;;   bit 1    +--------+-------+
;;;          1 | amber  |  off  |
;;;            +--------+-------+
;;;
;;; For 40G mode, we use only the first LED to indicate link/activity.
;;; The other 3 LEDs will be off.
;;;
set_led:
        call    set_any_led    ; set the first led green/off

        push    PORTSTATUS_ONE ; 1, 1 => led 2 off
        pack
        push    PORTSTATUS_ZERO
        pack

        push    PORTSTATUS_ONE ; 1, 1 => led 3 off
        pack
        push    PORTSTATUS_ZERO
        pack

        push    PORTSTATUS_ONE ; 1, 1 => led 4 off
        pack
        push    PORTSTATUS_ZERO
        pack

        ret
'''
    _set_led_4x10 = '''
;;; CelSmallstoneXpQSFPp_4x10
;;
;;  For 4x10G mode, we use an LED for each port.
;;
set_led:
        call    set_any_led
        inc     a
        call    set_any_led
        inc     a
        call    set_any_led
        inc     a
        call    set_any_led

        ret
'''

    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip=None, tx_polarity_flip=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.TridentTwo40GPort.__init__(self,
                                                    label=label,
                                                    serdes=serdes,
                                                    phy_types=None,
                                                    phy_addrs=None,
                                                    phy_layer=sfp.type.upper(),
                                                    leds=leds,
                                                    rx_lane_map=rx_lane_map,
                                                    rx_polarity_flip=rx_polarity_flip,
                                                    tx_lane_map=tx_lane_map,
                                                    tx_polarity_flip=tx_polarity_flip,
                                                    oversub=True,
                                                    unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None

        return set_calls

class CelSmallstoneXpLED(cumulus.ledup.TridentTwoLEDController):
    led_functions = (CelSmallstoneXpQSFPp._set_led_common)
    num_leds = 128

class CelSmallstoneXpCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/cel_smallstone_xp_cpld.0'

'''
   Celestica Smallstone XP LEDs
       System:
         green - all systems ok
         green blinking - fan missing or broken, psu missing or broken
       PSU1:
         green - psu1 installed and working
         off - psu1 missing or broken
       PSU2:
         green - psu2 installed and working
         off - psu2 missing or broken
       Fan1:
         green - fan installed and working
         off - fan missing or broken
       Fan2:
         green - fan installed and working
         off - fan missing or broken
       Fan3:
         green - fan installed and working
         off - fan missing or broken
       Fan4:
         green - fan installed and working
         off - fan missing or broken
'''

class CelSmallstoneXpSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'Status'
    led_location = 'led_status'
    bad_led_color = 'green_blinking'

class CelSmallstoneXpPsu1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Psu1'
    sources = ['PSU1']
    led_location = 'led_psu1'
    bad_led_color = 'off'

class CelSmallstoneXpPsu2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Psu2'
    sources = ['PSU2']
    led_location = 'led_psu2'
    bad_led_color = 'off'

class CelSmallstoneXpFanLED(cumulus.sensors.StatusLED_Unit):
    good_led_color = 'green'
    bad_led_color  = 'amber'
    green_colormap = {'green':'1',  'amber':'1',  'red':'0',  'off':'0'}
    red_colormap   = {'green':'0',  'amber':'1',  'red':'1',  'off':'0'}
    colormap       = {'10':'green', '11':'amber', '01':'red', '00':'off'}

    def led_update(self, color):
        self.set_gpio_value(self.led_location[0], self.green_colormap[color])
        self.set_gpio_value(self.led_location[1], self.red_colormap[color])

    def led_read(self):
        green_color = self.get_gpio_value(self.led_location[0])
        red_color = self.get_gpio_value(self.led_location[1])
        return self.name, self.colormap[green_color + red_color]

class CelSmallstoneXpFan1LED(CelSmallstoneXpFanLED):
    name = 'Fan Tray 1'
    sources = ['Fan1', 'Fan6']
    led_location = ['led_fan1_green', 'led_fan1_red']

class CelSmallstoneXpFan2LED(CelSmallstoneXpFanLED):
    name = 'Fan Tray 2'
    sources = ['Fan2', 'Fan7']
    led_location = ['led_fan2_green', 'led_fan2_red']

class CelSmallstoneXpFan3LED(CelSmallstoneXpFanLED):
    name = 'Fan Tray 3'
    sources = ['Fan3', 'Fan8']
    led_location = ['led_fan3_green', 'led_fan3_red']

class CelSmallstoneXpFan4LED(CelSmallstoneXpFanLED):
    name = 'Fan Tray 4'
    sources = ['Fan4', 'Fan9']
    led_location = ['led_fan4_green', 'led_fan4_red']

class CelSmallstoneXpFan5LED(CelSmallstoneXpFanLED):
    name = 'Fan Tray 5'
    sources = ['Fan5', 'Fan10']
    led_location = ['led_fan5_green', 'led_fan5_red']

# This sensor is giving low temperature (0-05C) readings at a customer site.
# Chances are this sensor is broken. So commenting it out of smond and pwmd.
# class CelSmallstoneXpTemp1(cumulus.sensors.Temp_Unit):
#     name = 'Temp1'
#     description = 'Intel CPU external sensor'
#     driver_path = '/sys/bus/i2c/devices/14-0048'
#     temp_crit = 70
#     temp_max = 65

class CelSmallstoneXpTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = 'Rear Outlet Air sensor'
    driver_path = '/sys/bus/i2c/devices/15-004e'
    temp_crit = 60
    temp_max = 48
    temp_min = -10
    temp_lcrit = -15

class CelSmallstoneXpTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Front Outlet Air sensor'
    driver_path = '/sys/bus/i2c/devices/24-0049'
    temp_crit = 60
    temp_max = 48
    temp_min = -10
    temp_lcrit = -15

class CelSmallstoneXpTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = '/sys/bus/i2c/devices/23-0048'
    temp_crit = 80
    temp_max = 75

CelSmallstoneXp_core_path = '/sys/devices/platform/coretemp.0'
class CelSmallstoneXpTemp5(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = 'Intel CPU die sensor'
    driver_path = CelSmallstoneXp_core_path
    driver_hwmon = ['temp2']
    temp_crit = 98

class CelSmallstoneXpTemp6(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    description = 'Intel CPU die sensor'
    driver_path = CelSmallstoneXp_core_path
    driver_hwmon = ['temp3']
    temp_crit = 98

class CelSmallstoneXpTemp7(cumulus.sensors.Temp_Unit):
    name = 'Temp6'
    description = 'Intel CPU die sensor'
    driver_path = CelSmallstoneXp_core_path
    driver_hwmon = ['temp4']
    temp_crit = 98

class CelSmallstoneXpTemp8(cumulus.sensors.Temp_Unit):
    name = 'Temp7'
    description = 'Intel CPU die sensor'
    driver_path = CelSmallstoneXp_core_path
    driver_hwmon = ['temp5']
    temp_crit = 98

CelSmallstoneXp_13_4d_path = '/sys/bus/i2c/devices/13-004d'
CelSmallstoneXp_13_2e_path = '/sys/bus/i2c/devices/13-002e'

class CelSmallstoneXpFan(cumulus.sensors.Fan_Unit):
    fan_min = 6200
    fan_max = 20000
    hwmon_search = False

    def find_and_add_attrs(self):
        if self.get_gpio_value(self.fan_present) == '1':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class CelSmallstoneXpFan1(CelSmallstoneXpFan):
    name = 'Fan1'
    description = 'Fan Tray 1'
    driver_path = CelSmallstoneXp_13_4d_path
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    fan_present = 'fan1_present'

class CelSmallstoneXpFan2(CelSmallstoneXpFan):
    name = 'Fan2'
    description = 'Fan Tray 2'
    driver_path = CelSmallstoneXp_13_4d_path
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    fan_present = 'fan2_present'

class CelSmallstoneXpFan3(CelSmallstoneXpFan):
    name = 'Fan3'
    description = 'Fan Tray 3'
    driver_path = CelSmallstoneXp_13_4d_path
    driver_hwmon = ['fan5']
    pwm_hwmon = ['pwm5']
    fan_present = 'fan3_present'

class CelSmallstoneXpFan4(CelSmallstoneXpFan):
    name = 'Fan4'
    description = 'Fan Tray 4'
    driver_path = CelSmallstoneXp_13_4d_path
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']
    fan_present = 'fan4_present'

class CelSmallstoneXpFan5(CelSmallstoneXpFan):
    name = 'Fan6'
    description = 'Fan Tray 1'
    fan_min = 5500
    fan_max = 17000
    driver_path = CelSmallstoneXp_13_2e_path
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    fan_present = 'fan1_present'

class CelSmallstoneXpFan6(CelSmallstoneXpFan):
    name = 'Fan7'
    description = 'Fan Tray 2'
    fan_min = 5500
    fan_max = 17000
    driver_path = CelSmallstoneXp_13_2e_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_present = 'fan2_present'

class CelSmallstoneXpFan7(CelSmallstoneXpFan):
    name = 'Fan8'
    description = 'Fan Tray 3'
    fan_min = 5500
    fan_max = 17000
    driver_path = CelSmallstoneXp_13_2e_path
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    fan_present = 'fan3_present'

class CelSmallstoneXpFan8(CelSmallstoneXpFan):
    name = 'Fan9'
    description = 'Fan Tray 4'
    fan_min = 5500
    fan_max = 17000
    driver_path = CelSmallstoneXp_13_2e_path
    driver_hwmon = ['fan5']
    pwm_hwmon = ['pwm5']
    fan_present = 'fan4_present'

class CelSmallstoneXpFan11(CelSmallstoneXpFan):
    name = 'Fan5'
    description = 'Fan Tray 5'
    driver_path = CelSmallstoneXp_13_4d_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_present = 'fan5_present'

class CelSmallstoneXpFan12(CelSmallstoneXpFan):
    name = 'Fan10'
    description = 'Fan Tray 5'
    fan_min = 5500
    fan_max = 17000
    driver_path = CelSmallstoneXp_13_2e_path
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']
    fan_present = 'fan5_present'

class CelSmallstoneXp_PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    fan_min = 4200
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = 'rmmod dps460 && modprobe dps460'
            subprocess.call(cmd, shell=True)
            self.init_done = True

    def find_and_add_attrs(self):
        if self.get_gpio_value(self.psu_present) == '1':
            self.attrs[self.psu_present] = 1
        else:
            self.attrs[self.psu_present] = 0
        if self.get_gpio_value(self.psu_all_ok) == '1':
            self.attrs[self.psu_all_ok] = 1
        else:
            self.attrs[self.psu_all_ok] = 0
        return 0

class CelSmallstoneXpPSU1(CelSmallstoneXp_PSU):
    name = 'PSU1'
    psu_present = 'psu_pwr1_present'
    psu_all_ok = 'psu_pwr1_all_ok'

class CelSmallstoneXpPSU2(CelSmallstoneXp_PSU):
    name = 'PSU2'
    psu_present = 'psu_pwr2_present'
    psu_all_ok = 'psu_pwr2_all_ok'

class CelSmallstoneXpPSUTemp(cumulus.sensors.Temp_Unit):
    def find_and_add_attrs(self):
        if ((self.get_gpio_value(self.temp_present) == '1') and
            (self.get_gpio_value(self.temp_all_ok) == '1')):
            return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)
        else:
            return -1

class CelSmallstoneXpTemp9(CelSmallstoneXpPSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Inlet Temp Sensor'
    driver_path = '/sys/bus/i2c/devices/11-0059'
    driver_hwmon = ['temp1']
    temp_present = 'psu_pwr2_present'
    temp_all_ok = 'psu_pwr2_all_ok'
    temp_max = 60
    temp_crit = 100

class CelSmallstoneXpTemp10(CelSmallstoneXpPSUTemp):
    name = 'PSU2Temp2'
    description = 'PSU2 Max Temp Sensor'
    driver_path = '/sys/bus/i2c/devices/11-0059'
    driver_hwmon = ['temp2']
    temp_present = 'psu_pwr2_present'
    temp_all_ok = 'psu_pwr2_all_ok'
    temp_max = 60
    temp_crit = 100

class CelSmallstoneXpTemp11(CelSmallstoneXpPSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Inlet Temp Sensor'
    driver_path = '/sys/bus/i2c/devices/10-0058'
    driver_hwmon = ['temp1']
    temp_present = 'psu_pwr1_present'
    temp_all_ok = 'psu_pwr1_all_ok'
    temp_max = 60
    temp_crit = 100

class CelSmallstoneXpTemp12(CelSmallstoneXpPSUTemp):
    name = 'PSU1Temp2'
    description = 'PSU1 Max Temp Sensor'
    driver_path = '/sys/bus/i2c/devices/10-0058'
    driver_hwmon = ['temp2']
    temp_present = 'psu_pwr1_present'
    temp_all_ok = 'psu_pwr1_all_ok'
    temp_max = 60
    temp_crit = 100

class CelSmallstoneXpPSUFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    minpwm = 64
    fan_max = 18000
    fan_min = 4300

    def find_and_add_attrs(self):
        if ((self.get_gpio_value(self.fan_present) == '1') and
            (self.get_gpio_value(self.fan_all_ok) == '1')):
            return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)
        else:
            return -1

class CelSmallstoneXpFan9(CelSmallstoneXpPSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = '/sys/bus/i2c/devices/11-0059'
    fan_present = 'psu_pwr2_present'
    fan_all_ok = 'psu_pwr2_all_ok'

class CelSmallstoneXpFan10(CelSmallstoneXpPSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = '/sys/bus/i2c/devices/10-0058'
    fan_present = 'psu_pwr1_present'
    fan_all_ok = 'psu_pwr1_all_ok'

class CelSmallstoneXpPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (CelSmallstoneXpFan1(),
            CelSmallstoneXpFan2(),
            CelSmallstoneXpFan3(),
            CelSmallstoneXpFan4(),
            CelSmallstoneXpFan11(),
            CelSmallstoneXpFan5(),
            CelSmallstoneXpFan6(),
            CelSmallstoneXpFan7(),
            CelSmallstoneXpFan8(),
            CelSmallstoneXpFan12(),
    )
    temps = (CelSmallstoneXpTemp2(),
             CelSmallstoneXpTemp3(),
             CelSmallstoneXpTemp4(),
             CelSmallstoneXpTemp5(),
             CelSmallstoneXpTemp6(),
             CelSmallstoneXpTemp7(),
             CelSmallstoneXpTemp8(),
             )

class CelSmallstoneXpPWM2(cumulus.sensors.PWM_Unit):
    name = 'PSU1 PWM'
    fans = (CelSmallstoneXpFan10(),
            )
    temps = (CelSmallstoneXpTemp11(),
             CelSmallstoneXpTemp12(),
             )

class CelSmallstoneXpPWM3(cumulus.sensors.PWM_Unit):
    name = 'PSU2 PWM'
    fans = (CelSmallstoneXpFan9(),
            )
    temps = (CelSmallstoneXpTemp9(),
             CelSmallstoneXpTemp10(),
             )

class CelSmallstoneXpSensors():
    units = (CelSmallstoneXpPSU1(),
             CelSmallstoneXpPSU2(),
             CelSmallstoneXpTemp2(),
             CelSmallstoneXpTemp3(),
             CelSmallstoneXpTemp4(),
             CelSmallstoneXpTemp9(),
             CelSmallstoneXpTemp5(),
             CelSmallstoneXpTemp6(),
             CelSmallstoneXpTemp7(),
             CelSmallstoneXpTemp8(),
             CelSmallstoneXpTemp10(),
             CelSmallstoneXpTemp11(),
             CelSmallstoneXpTemp12(),
             CelSmallstoneXpFan1(),
             CelSmallstoneXpFan2(),
             CelSmallstoneXpFan3(),
             CelSmallstoneXpFan4(),
             CelSmallstoneXpFan5(),
             CelSmallstoneXpFan6(),
             CelSmallstoneXpFan7(),
             CelSmallstoneXpFan8(),
             CelSmallstoneXpFan9(),
             CelSmallstoneXpFan10(),
             CelSmallstoneXpFan11(),
             CelSmallstoneXpFan12(),
            )

class CelSmallstoneXpPWMs():
    units = (CelSmallstoneXpPWM1(),
             CelSmallstoneXpPWM2(),
             CelSmallstoneXpPWM3(),
             )

class CelSmallstoneXpLEDs():
    units = (CelSmallstoneXpPsu1LED(),
             CelSmallstoneXpPsu2LED(),
             CelSmallstoneXpSystemLED(),
             CelSmallstoneXpFan1LED(),
             CelSmallstoneXpFan2LED(),
             CelSmallstoneXpFan3LED(),
             CelSmallstoneXpFan4LED(),
             CelSmallstoneXpFan5LED(),
             )

class CelSmallstoneXpSwitch(cumulus.platform.BCMSwitch):
    ledup = CelSmallstoneXpLED
    _qsfp = CelSmallstoneXpQSFPp

    has_phy_ucode_mdio = False # XXX unverified
    has_phy_ucode_spi_rom = False # XXX unverified

    ports = (
        _qsfp(label="1", serdes=(  0,   1,   2,   3), leds=(0, 0),
              tx_polarity_flip=0x0007, rx_polarity_flip=0x000f,
              rx_lane_map=0x3120, tx_lane_map=0x2031),
        _qsfp(label="2", serdes=(  4,   5,   6,   7), leds=(0, 8),
              tx_polarity_flip=0x0008, rx_polarity_flip=0x000f,
              rx_lane_map=0x2031, tx_lane_map=0x0213),
        _qsfp(label="3", serdes=(  8,   9,  10,  11), leds=(0, 16),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x000f,
              rx_lane_map=0x2031, tx_lane_map=0x3012),
        _qsfp(label="4", serdes=( 12,  13,  14,  15), leds=(0, 24),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x000f,
              rx_lane_map=0x2031, tx_lane_map=0x1203),
        _qsfp(label="5", serdes=( 16,  17,  18,  19), leds=(0, 32),
              tx_polarity_flip=0x0007, rx_polarity_flip=0x000b,
              rx_lane_map=0x3120, tx_lane_map=0x2031),
        _qsfp(label="6", serdes=( 20,  21,  22,  23), leds=(0, 40),
              tx_polarity_flip=0x0008, rx_polarity_flip=0x000f,
              rx_lane_map=0x2031, tx_lane_map=0x0213),
        _qsfp(label="7", serdes=( 24,  25,  26,  27), leds=(0, 48),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x000f,
              rx_lane_map=0x2031, tx_lane_map=0x3012),
        _qsfp(label="8", serdes=( 28,  29,  30,  31), leds=(0, 56),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x000f,
              rx_lane_map=0x2031, tx_lane_map=0x1203),
        _qsfp(label="9", serdes=( 32,  33,  34,  35), leds=(0, 64),
              tx_polarity_flip=0x0007, rx_polarity_flip=0x000b,
              rx_lane_map=0x3120, tx_lane_map=0x2031),
        _qsfp(label="10", serdes=( 36,  37,  38,  39), leds=(0, 72),
              tx_polarity_flip=0x0008, rx_polarity_flip=0x000f,
              rx_lane_map=0x2031, tx_lane_map=0x0213),
        _qsfp(label="11", serdes=( 40,  41,  42,  43), leds=(0, 80),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x000f,
              rx_lane_map=0x2031, tx_lane_map=0x3012),
        _qsfp(label="12", serdes=( 44,  45,  46,  47), leds=(0, 88),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x000f,
              rx_lane_map=0x2031, tx_lane_map=0x1203),
        _qsfp(label="13", serdes=( 48,  49,  50,  51), leds=(0, 96),
              tx_polarity_flip=0x0007, rx_polarity_flip=0x000b,
              rx_lane_map=0x3120, tx_lane_map=0x2031),
        _qsfp(label="14", serdes=( 52,  53,  54,  55), leds=(0, 104),
              tx_polarity_flip=0x000b, rx_polarity_flip=0x000f,
              rx_lane_map=0x2031, tx_lane_map=0x0213),
        _qsfp(label="15", serdes=( 56,  57,  58,  59), leds=(0, 112),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              rx_lane_map=0x2031, tx_lane_map=0x3012),
        _qsfp(label="16", serdes=( 60,  61,  62,  63), leds=(0, 120),
              tx_polarity_flip=0x000b, rx_polarity_flip=0x000f,
              rx_lane_map=0x2031, tx_lane_map=0x1203),
        _qsfp(label="17", serdes=( 64,  65,  66,  67), leds=(1, 0),
              tx_polarity_flip=0x0008, rx_polarity_flip=0x0005,
              rx_lane_map=0x3120, tx_lane_map=0x2031),
        _qsfp(label="18", serdes=( 68,  69,  70,  71), leds=(1, 8),
              tx_polarity_flip=0x0007,
              rx_lane_map=0x2031, tx_lane_map=0x0213),
        _qsfp(label="19", serdes=( 72,  73,  74,  75), leds=(1, 16),
              tx_polarity_flip=0x000a,
              rx_lane_map=0x2031, tx_lane_map=0x3012),
        _qsfp(label="20", serdes=( 76,  77,  78,  79), leds=(1, 24),
              tx_polarity_flip=0x000d,
              rx_lane_map=0x2031, tx_lane_map=0x0312),
        _qsfp(label="21", serdes=( 80,  81,  82,  83), leds=(1, 32),
              tx_polarity_flip=0x0008, rx_polarity_flip=0x0004,
              rx_lane_map=0x3120, tx_lane_map=0x3120),
        _qsfp(label="22", serdes=( 84,  85,  86,  87), leds=(1, 40),
              tx_polarity_flip=0x0007,
              rx_lane_map=0x2031, tx_lane_map=0x1302),
        _qsfp(label="23", serdes=( 88,  89,  90,  91), leds=(1, 48),
              tx_polarity_flip=0x000a,
              rx_lane_map=0x2031, tx_lane_map=0x2103),
        _qsfp(label="24", serdes=( 92,  93,  94,  95), leds=(1, 56),
              tx_polarity_flip=0x000d,
              rx_lane_map=0x2031, tx_lane_map=0x0312),
        _qsfp(label="25", serdes=( 96,  97,  98,  99), leds=(1, 64),
              tx_polarity_flip=0x0008, rx_polarity_flip=0x0004,
              rx_lane_map=0x3120, tx_lane_map=0x3120),
        _qsfp(label="26", serdes=(100, 101, 102, 103), leds=(1, 72),
              tx_polarity_flip=0x0007,
              rx_lane_map=0x2031, tx_lane_map=0x1302),
        _qsfp(label="27", serdes=(104, 105, 106, 107), leds=(1, 80),
              tx_polarity_flip=0x000a,
              rx_lane_map=0x2031, tx_lane_map=0x2103),
        _qsfp(label="28", serdes=(108, 109, 110, 111), leds=(1, 88),
              tx_polarity_flip=0x000d,
              rx_lane_map=0x2031, tx_lane_map=0x0312),
        _qsfp(label="29", serdes=(112, 113, 114, 115), leds=(1, 96),
              tx_polarity_flip=0x0008, rx_polarity_flip=0x0004,
              rx_lane_map=0x3120, tx_lane_map=0x3120),
        _qsfp(label="30", serdes=(116, 117, 118, 119), leds=(1, 104),
              tx_polarity_flip=0x0007,
              rx_lane_map=0x2031, tx_lane_map=0x0213),
        _qsfp(label="31", serdes=(120, 121, 122, 123), leds=(1, 112),
              tx_polarity_flip=0x000a,
              rx_lane_map=0x2031, tx_lane_map=0x3012),
        _qsfp(label="32", serdes=(124, 125, 126, 127), leds=(1, 120),
              tx_polarity_flip=0x000d,
              rx_lane_map=0x2031, tx_lane_map=0x1203),
    )

class CelSmallstoneXp(cumulus.platform.Platform):
    name = 'cel,smallstone_xp'
    chip = cumulus.platform.TridentTwo_56850_Chip()
    switch = CelSmallstoneXpSwitch(chip)
    cpld = CelSmallstoneXpCPLD()
    statusleds = CelSmallstoneXpLEDs()
    pwms = CelSmallstoneXpPWMs()
    sensors = CelSmallstoneXpSensors()

Platform = CelSmallstoneXp
