#
# Copyright 2016.  Cumulus Networks, Inc.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class DniAG9032V1QSFP28(cumulus.platform.Tomahawk100GPort):
    '''
;;;
;;; Each port has twelve bits of LED data controlling four multicolor leds
;;;
;;;  bit 0     #1 green
;;;  bit 1     #1 amber
;;;  bit 2     #1 white
;;;  bit 3     #2 white
;;;  bit 4     #1 blue
;;;  bit 5     #2 blue
;;;  bit 6     #3 blue
;;;  bit 7     #4 blue
;;;  bit 8     #1 purple
;;;  bit 9     #2 purple
;;;  bit 10    #3 purple
;;;  bit 11    #4 purple
;;;
;;;  we will use green for 1x modes
;;   and blue for 2x and 4x modes
;;;'''
    _set_led_common = '''
;;;
;;;
set_one_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv
        pushst  PORTSTATUS_LINKEN
        tand
        pack
        ret

pad_leds:
        pushst  PORTSTATUS_ZERO
        pack
        sub     b, 1
        jnz     pad_leds

        ret
'''

    _set_led_1x = '''
;;;
set_led:
        call   set_one_led
        ld     b, 11
        call   pad_leds
        ret
'''
    _set_led_2x = '''
;;;
;;; In 2x mode, the first two blue LEDs are used to
;;; indicate link/activity.
;;;
set_led:
        ld      b, 4
        call    pad_leds
        call    set_one_led
        inc     a
        inc     a
        call    set_one_led
        ld      b, 6
        call    pad_leds

        ret
'''

    _set_led_4x = '''
;;;
;;; In 4x mode all four blue LEDs are used to
;;; indicate link/activity.
;;;
set_led:
        ld      b, 4
        call    pad_leds
        call    set_one_led
        inc     a
        call    set_one_led
        inc     a
        call    set_one_led
        inc     a
        call    set_one_led
        ld      b, 4
        call    pad_leds

        ret
'''

    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        set_calls[led_index_list[0]] = call
        for i in range(1,12):
            set_calls[led_index_list[0]+i] = None

        return set_calls

class DniAG9032V1LED(cumulus.ledup.TomahawkLEDController):
    led_functions = (DniAG9032V1QSFP28._set_led_common)
    num_leds = 192


# DniAG9032V1 Switch Ports

class DniAG9032V1Switch(cumulus.platform.BCMSwitch):
    ledup = DniAG9032V1LED
    _qsfp28 = DniAG9032V1QSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp28(label="1", serdes=( 40, 41, 42, 43), leds=(1, 24),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x1032),
        _qsfp28(label="2", serdes=( 44, 45, 46, 47), leds=(1, 36),
              tx_polarity_flip=0x0001, rx_polarity_flip=0x0002,
              tx_lane_map=0x1032, rx_lane_map=0x0123),
        _qsfp28(label="3", serdes=( 48, 49, 50, 51), leds=(1, 48),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x0213, rx_lane_map=0x0213),
        _qsfp28(label="4", serdes=( 36, 37, 38, 39), leds=(1, 12),
              tx_polarity_flip=0x0001, rx_polarity_flip=0x0000,
              tx_lane_map=0x2031, rx_lane_map=0x2301),
        _qsfp28(label="5", serdes=( 32, 33, 34, 35), leds=(1, 0),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x0213, rx_lane_map=0x3120),
        _qsfp28(label="6", serdes=( 52, 53, 54, 55), leds=(1, 60),
              tx_polarity_flip=0x000e, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x1302),
        _qsfp28(label="7", serdes=( 56, 57, 58, 59), leds=(1, 72),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
                tx_lane_map=0x0213, rx_lane_map=0x0213),
        _qsfp28(label="8", serdes=( 60, 61, 62, 63), leds=(1, 84),
              tx_polarity_flip=0x000E, rx_polarity_flip=0x000F,
                tx_lane_map=0x1302, rx_lane_map=0x1302),
        _qsfp28(label="9", serdes=( 64, 65, 66, 67), leds=(1, 96),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
                tx_lane_map=0x3120, rx_lane_map=0x3120),
        _qsfp28(label="10", serdes=( 68, 69, 70, 71), leds=(1, 108),
              tx_polarity_flip=0x000E, rx_polarity_flip=0x0007,
                tx_lane_map=0x1302, rx_lane_map=0x2103),
        _qsfp28(label="11", serdes=( 72, 73, 74, 75), leds=(1, 120),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
                tx_lane_map=0x0213, rx_lane_map=0x0213),
        _qsfp28(label="12", serdes=( 76, 77, 78, 79), leds=(1, 132),
              tx_polarity_flip=0x000E, rx_polarity_flip=0x000F,
              tx_lane_map=0x0123, rx_lane_map=0x1032),
        _qsfp28(label="13", serdes=( 80, 81, 82, 83), leds=(1, 144),
              tx_polarity_flip=0x000F, rx_polarity_flip=0x000F,
              tx_lane_map=0x0213, rx_lane_map=0x2031),
        _qsfp28(label="14", serdes=( 84, 85, 86, 87), leds=(1, 156),
              tx_polarity_flip=0x000E, rx_polarity_flip=0x000F,
              tx_lane_map=0x2031, rx_lane_map=0x0213),
        _qsfp28(label="15", serdes=( 88, 89, 90, 91), leds=(1, 168),
              tx_polarity_flip=0x000F, rx_polarity_flip=0x000F,
              tx_lane_map=0x0213, rx_lane_map=0x3210),
        _qsfp28(label="16", serdes=( 92, 93, 94, 95), leds=(1, 180),
              tx_polarity_flip=0x000E, rx_polarity_flip=0x000F,
              tx_lane_map=0x2031, rx_lane_map=0x0213),

        _qsfp28(label="17", serdes=( 96, 97, 98, 99), leds=(0, 96),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x2031, rx_lane_map=0x0213),
        _qsfp28(label="18", serdes=(100, 101, 102, 103), leds=(0, 108),
              tx_polarity_flip=0x000F, rx_polarity_flip=0x0002,
              tx_lane_map=0x0213, rx_lane_map=0x2031),
        _qsfp28(label="19", serdes=(104, 105, 106, 107), leds=(0, 120),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x2031, rx_lane_map=0x0213),
        _qsfp28(label="20", serdes=(108, 109, 110, 111), leds=(0, 132),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000F,
              tx_lane_map=0x0213, rx_lane_map=0x2031),
        _qsfp28(label="21", serdes=(120, 121, 122, 123), leds=(0, 168),
              tx_polarity_flip=0x000F, rx_polarity_flip=0x000F,
              tx_lane_map=0x0312, rx_lane_map=0x1302),
        _qsfp28(label="22", serdes=(112, 113, 114, 115), leds=(0, 144),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x0123),
        _qsfp28(label="23", serdes=(  0,  1,  2,  3), leds=(0, 0),
              tx_polarity_flip=0x000F, rx_polarity_flip=0x000F,
              tx_lane_map=0x1302, rx_lane_map=0x1302),
        _qsfp28(label="24", serdes=(116, 117, 118, 119), leds=(0, 156),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x0213, rx_lane_map=0x0213),
        _qsfp28(label="25", serdes=(  4,  5,  6,  7), leds=(0, 12),
              tx_polarity_flip=0x000F, rx_polarity_flip=0x000F,
              tx_lane_map=0x1302, rx_lane_map=0x1302),
        _qsfp28(label="26", serdes=( 124, 125, 126, 127), leds=(0, 180),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0002,
              tx_lane_map=0x3120, rx_lane_map=0x3120),
        _qsfp28(label="27", serdes=( 28, 29, 30, 31), leds=(0, 84),
              tx_polarity_flip=0x000F, rx_polarity_flip=0x000F,
              tx_lane_map=0x1302, rx_lane_map=0x2031),
        _qsfp28(label="28", serdes=(  8,  9, 10, 11), leds=(0, 24),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x0213, rx_lane_map=0x0213),
        _qsfp28(label="29", serdes=( 12, 13, 14, 15), leds=(0, 36),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x1302, rx_lane_map=0x1302),
        _qsfp28(label="30", serdes=( 24, 25, 26, 27), leds=(0, 72),
              tx_polarity_flip=0x000F, rx_polarity_flip=0x000F,
              tx_lane_map=0x3120, rx_lane_map=0x0213),
        _qsfp28(label="31", serdes=( 16, 17, 18, 19), leds=(0, 48),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x1032, rx_lane_map=0x0123),
        _qsfp28(label="32", serdes=( 20, 21, 22, 23), leds=(0, 60),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x0213, rx_lane_map=0x1032),
    )

# ***** DniAG9032V1 Sensors *****

# DniAG9032V1 Power Supplies

def DniAg9032_i2c_path(bus, addr):
    return '/sys/bus/i2c/devices/%02u-%04x' % (bus, addr)

DniAG9032V1_i2c_30_58_path = DniAg9032_i2c_path(30, 0x58)
DniAG9032V1_i2c_32_58_path = DniAg9032_i2c_path(32, 0x58)
DniAG9032V1_i2c_10_4d_path = DniAg9032_i2c_path(10, 0x4d)
DniAG9032V1_i2c_26_4f_path = DniAg9032_i2c_path(26, 0x4f)
DniAG9032V1_i2c_15_4c_path = DniAg9032_i2c_path(15, 0x4c)
DniAG9032V1_i2c_15_4d_path = DniAg9032_i2c_path(15, 0x4d)
DniAG9032V1_i2c_15_4e_path = DniAg9032_i2c_path(15, 0x4e)
DniAG9032V1_i2c_30_58_path = DniAg9032_i2c_path(30, 0x58)
DniAG9032V1_i2c_32_58_path = DniAg9032_i2c_path(32, 0x58)
DniAG9032V1_i2c_25_2c_path = DniAg9032_i2c_path(25, 0x2c)
DniAG9032V1_i2c_25_2d_path = DniAg9032_i2c_path(25, 0x2d)
DniAG9032V1_i2c_31_40_path = DniAg9032_i2c_path(31, 0x40)
DniAG9032V1_i2c_33_40_path = DniAg9032_i2c_path(33, 0x40)

class DniAG9032V1PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = 'rmmod dps460 && modprobe dps460'
            subprocess.call(cmd, shell=True)
            self.init_done = True

class DniAG9032V1PSU1(DniAG9032V1PSU):
    name = 'PSU1'
    driver_path = DniAG9032V1_i2c_31_40_path
    cpld_hwmon = ['psu_pwr1']

class DniAG9032V1PSU2(DniAG9032V1PSU):
    name = 'PSU2'
    driver_path = DniAG9032V1_i2c_33_40_path
    cpld_hwmon = ['psu_pwr2']


# DniAG9032V1 Board Temperature Sensors

# Intel Rangeley on-die temperature sensors
DniAG9032V1_core_path = '/sys/devices/platform/coretemp.0'

class DniAG9032V1CT(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = DniAG9032V1_core_path
    temp_fan_min = 75
    temp_max = 93
    temp_crit = 98

class DniAG9032V1Temp1(DniAG9032V1CT):
    name = 'Temp1'
    driver_hwmon = ['temp2']

class DniAG9032V1Temp2(DniAG9032V1CT):
    name = 'Temp2'
    driver_hwmon = ['temp3']

class DniAG9032V1Temp3(DniAG9032V1CT):
    name = 'Temp3'
    driver_hwmon = ['temp4']

class DniAG9032V1Temp4(DniAG9032V1CT):
    name = 'Temp4'
    driver_hwmon = ['temp5']

class DniAG9032V1Temp5(cumulus.sensors.Temp_Unit):
     name = 'Temp5'
     description = 'CPU external sensor'
     driver_path = DniAG9032V1_i2c_10_4d_path
     temp_crit = 60
     temp_max = 50

class DniAG9032V1Temp6(cumulus.sensors.Temp_Unit):
     name = 'Temp6'
     description = 'Fan sensor'
     driver_path = DniAG9032V1_i2c_26_4f_path
     temp_crit = 65
     temp_max = 50

class DniAG9032V1Temp7(cumulus.sensors.Temp_Unit):
     name = 'Temp7'
     description = 'MAC sensor top'
     driver_path = DniAG9032V1_i2c_15_4c_path
     temp_crit = 80
     temp_max = 65

class DniAG9032V1Temp8(cumulus.sensors.Temp_Unit):
     name = 'Temp8'
     description = 'MAC sensor bottom'
     driver_path = DniAG9032V1_i2c_15_4d_path
     temp_crit = 75
     temp_max = 65

class DniAG9032V1Temp9(cumulus.sensors.Temp_Unit):
     name = 'Temp9'
     description = 'Front panel sensor'
     driver_path = DniAG9032V1_i2c_15_4e_path
     temp_crit = 65
     temp_max = 50


# DniAG9032V1 PSU Temp Sensors

class DniAG9032V1PSU1Temp1(cumulus.sensors.Temp_Unit):
    psu = 'PSU1'
    name = 'PSU1Temp1'
    description = 'PSU1 Inlet Temp Sensor'
    driver_path = DniAG9032V1_i2c_30_58_path
    driver_hwmon = ['temp1']
    temp_present = 'psu_pwr1'
    temp_all_ok = 'psu_pwr1'
    temp_max = 95

class DniAG9032V1PSU1Temp2(cumulus.sensors.Temp_Unit):
    psu = 'PSU1'
    name = 'PSU1Temp2'
    description = 'PSU1 Heatsink Temp Sensor'
    driver_path = DniAG9032V1_i2c_30_58_path
    driver_hwmon = ['temp2']
    temp_present = 'psu_pwr1'
    temp_all_ok = 'psu_pwr1'
    temp_max = 95

class DniAG9032V1PSU2Temp1(cumulus.sensors.Temp_Unit):
    psu = 'PSU2'
    name = 'PSU2Temp1'
    description = 'PSU2 Inlet Temp Sensor'
    driver_path = DniAG9032V1_i2c_32_58_path
    driver_hwmon = ['temp1']
    temp_present = 'psu_pwr2'
    temp_all_ok = 'psu_pwr2'
    temp_max = 95

class DniAG9032V1PSU2Temp2(cumulus.sensors.Temp_Unit):
    psu = 'PSU2'
    name = 'PSU2Temp2'
    description = 'PSU2 Heatsink Temp Sensor'
    driver_path = DniAG9032V1_i2c_32_58_path
    driver_hwmon = ['temp2']
    temp_present = 'psu_pwr2'
    temp_all_ok = 'psu_pwr2'
    temp_max = 95

class DniAG9032V1THTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 105
    temp_crit    = 110

class DniAG9032V1THTemp1(DniAG9032V1THTemp):
    name = 'Temp10'
    driver_hwmon = ['temp1']

class DniAG9032V1THTemp2(DniAG9032V1THTemp):
    name = 'Temp11'
    driver_hwmon = ['temp2']

class DniAG9032V1THTemp3(DniAG9032V1THTemp):
    name = 'Temp12'
    driver_hwmon = ['temp3']

class DniAG9032V1THTemp4(DniAG9032V1THTemp):
    name = 'Temp13'
    driver_hwmon = ['temp4']

class DniAG9032V1THTemp5(DniAG9032V1THTemp):
    name = 'Temp14'
    driver_hwmon = ['temp5']

class DniAG9032V1THTemp6(DniAG9032V1THTemp):
    name = 'Temp15'
    driver_hwmon = ['temp6']

class DniAG9032V1THTemp7(DniAG9032V1THTemp):
    name = 'Temp16'
    driver_hwmon = ['temp7']

class DniAG9032V1THTemp8(DniAG9032V1THTemp):
    name = 'Temp17'
    driver_hwmon = ['temp8']


# DniAG9032V1 Fans
#
#         25-002c    25-002d
#  tray1   fan5       fan5
#  tray2   fan4       fan4
#  tray3   fan3       fan3
#  tray4   fan2       fan2
#  tray5   fan1       fan1

class DniAG9032V1Fan(cumulus.sensors.Fan_Unit):
    fan_min = 5000
    fan_max = 23000
    hwmon_search = False

    def find_and_add_attrs(self):
        if self.get_gpio_value(self.fan_absent) == '0':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class DniAG9032V1Fan1(DniAG9032V1Fan):
    name = 'Fan1'
    description = 'Fan Tray 1'
    driver_path = DniAG9032V1_i2c_25_2c_path
    driver_hwmon = ['fan5']
    pwm_hwmon = ['pwm5']
    fan_absent = 'fan_tray1_absent'

class DniAG9032V1Fan2(DniAG9032V1Fan):
    name = 'Fan2'
    description = 'Fan Tray 1'
    driver_path = DniAG9032V1_i2c_25_2d_path
    driver_hwmon = ['fan5']
    pwm_hwmon = ['pwm5']
    fan_absent = 'fan_tray1_absent'

class DniAG9032V1Fan3(DniAG9032V1Fan):
    name = 'Fan3'
    description = 'Fan Tray 2'
    driver_path = DniAG9032V1_i2c_25_2c_path
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    fan_absent = 'fan_tray2_absent'

class DniAG9032V1Fan4(DniAG9032V1Fan):
    name = 'Fan4'
    description = 'Fan Tray 2'
    driver_path = DniAG9032V1_i2c_25_2d_path
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    fan_absent = 'fan_tray2_absent'

class DniAG9032V1Fan5(DniAG9032V1Fan):
    name = 'Fan5'
    description = 'Fan Tray 3'
    driver_path = DniAG9032V1_i2c_25_2c_path
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']
    fan_absent = 'fan_tray3_absent'

class DniAG9032V1Fan6(DniAG9032V1Fan):
    name = 'Fan6'
    description = 'Fan Tray 3'
    driver_path = DniAG9032V1_i2c_25_2d_path
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']
    fan_absent = 'fan_tray3_absent'

class DniAG9032V1Fan7(DniAG9032V1Fan):
    name = 'Fan7'
    description = 'Fan Tray 4'
    driver_path = DniAG9032V1_i2c_25_2c_path
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    fan_absent = 'fan_tray4_absent'

class DniAG9032V1Fan8(DniAG9032V1Fan):
    name = 'Fan8'
    description = 'Fan Tray 4'
    driver_path = DniAG9032V1_i2c_25_2d_path
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    fan_absent = 'fan_tray4_absent'

class DniAG9032V1Fan9(DniAG9032V1Fan):
    name = 'Fan9'
    description = 'Fan Tray 5'
    driver_path = DniAG9032V1_i2c_25_2c_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_absent = 'fan_tray5_absent'

class DniAG9032V1Fan10(DniAG9032V1Fan):
    name = 'Fan10'
    description = 'Fan Tray 5'
    driver_path = DniAG9032V1_i2c_25_2d_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_absent = 'fan_tray5_absent'

# DniAG9032V1 PSU Fans

class DniAG9032V1PSUFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_max = 19000
    fan_min = 3500

class DniAG9032V1PSU1Fan(DniAG9032V1PSUFan):
    name = 'PSU1Fan'
    description = 'PSU1 Fan'
    driver_path = DniAG9032V1_i2c_30_58_path

class DniAG9032V1PSU2Fan(DniAG9032V1PSUFan):
    name = 'PSU2Fan'
    description = 'PSU2 Fan'
    driver_path = DniAG9032V1_i2c_32_58_path

class DniAG9032V1Sensors():
    units = (DniAG9032V1PSU1(),
             DniAG9032V1PSU2(),
             DniAG9032V1Temp1(),
             DniAG9032V1Temp2(),
             DniAG9032V1Temp3(),
             DniAG9032V1Temp4(),
             DniAG9032V1Temp5(),
             DniAG9032V1Temp6(),
             DniAG9032V1Temp7(),
             DniAG9032V1Temp8(),
             DniAG9032V1Temp9(),
             DniAG9032V1THTemp1(),
             DniAG9032V1THTemp2(),
             DniAG9032V1THTemp3(),
             DniAG9032V1THTemp4(),
             DniAG9032V1THTemp5(),
             DniAG9032V1THTemp6(),
             DniAG9032V1THTemp7(),
             DniAG9032V1THTemp8(),
             DniAG9032V1PSU1Temp1(),
             DniAG9032V1PSU1Temp2(),
             DniAG9032V1PSU2Temp1(),
             DniAG9032V1PSU2Temp2(),
             DniAG9032V1Fan1(),
             DniAG9032V1Fan2(),
             DniAG9032V1Fan3(),
             DniAG9032V1Fan4(),
             DniAG9032V1Fan5(),
             DniAG9032V1Fan6(),
             DniAG9032V1Fan7(),
             DniAG9032V1Fan8(),
             DniAG9032V1Fan9(),
             DniAG9032V1Fan10(),
             DniAG9032V1PSU1Fan(),
             DniAG9032V1PSU2Fan(),
            )


# DniAG9032V1 PWM Controllers

class DniAG9032V1PWM(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (DniAG9032V1Fan1(),
             DniAG9032V1Fan2(),
             DniAG9032V1Fan3(),
             DniAG9032V1Fan4(),
             DniAG9032V1Fan5(),
             DniAG9032V1Fan6(),
             DniAG9032V1Fan7(),
             DniAG9032V1Fan8(),
             DniAG9032V1Fan9(),
             DniAG9032V1Fan10(),
    )
    temps = (DniAG9032V1Temp1(),
             DniAG9032V1Temp2(),
             DniAG9032V1Temp3(),
             DniAG9032V1Temp4(),
             DniAG9032V1Temp5(),
             DniAG9032V1Temp6(),
             DniAG9032V1Temp7(),
             DniAG9032V1Temp8(),
             DniAG9032V1Temp9(),
             DniAG9032V1THTemp1(),
             DniAG9032V1THTemp2(),
             DniAG9032V1THTemp3(),
             DniAG9032V1THTemp4(),
             DniAG9032V1THTemp5(),
             DniAG9032V1THTemp6(),
             DniAG9032V1THTemp7(),
             DniAG9032V1THTemp8(),
            )

class DniAG9032V1PSU1PWM(cumulus.sensors.PWM_Unit):
    name = 'PSU1 PWM'
    fans = (DniAG9032V1PSU1Fan(),
            )
    temps = (DniAG9032V1PSU1Temp1(),
             DniAG9032V1PSU1Temp2(),
             )

class DniAG9032V1PSU2PWM(cumulus.sensors.PWM_Unit):
    name = 'PSU2 PWM'
    fans = (DniAG9032V1PSU2Fan(),
            )
    temps = (DniAG9032V1PSU2Temp1(),
             DniAG9032V1PSU2Temp2(),
             )

class DniAG9032V1PWMs():
    units = (DniAG9032V1PWM(),
             DniAG9032V1PSU1PWM(),
             DniAG9032V1PSU2PWM(),
             )


# ***** DniAG9032V1 CPLD *****

class DniAG9032V1CPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/dni_ag9032v1_cpld.0'

'''
    DniAG9032V1 LEDs

    Front Panel LEDs
        System:
          green - all systems ok
          red - fan missing or broken, psu missing or broken
        Fan:
          green - all fans ok
          amber - one or more fans missing or broken
        PSU1:
          green - psu1 installed and working
          amber blinking - psu1 missing or broken
        PSU2:
          green - psu2 installed and working
          amber blinking - psu2 missing or broken

    Fan Tray LEDs
        Fan_Tray_1 - Fan_Tray_5:
          green - fan installed and working
          red - fan missing or broken
'''
class DniAG9032V1SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System LED'
    led_location = 'led_system'
    bad_led_color = 'red'

class DniAG9032V1PSU1LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU1 LED'
    sources = ['PSU1']
    led_location = 'led_psu1'
    bad_led_color = 'amber_blinking'

class DniAG9032V1PSU2LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU2 LED'
    sources = ['PSU2']
    led_location = 'led_psu2'
    bad_led_color = 'amber_blinking'

class DniAG9032V1FanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan LED'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5',
               'Fan6', 'Fan7', 'Fan8', 'Fan9', 'Fan10']
    led_location = 'led_fan'
    bad_led_color  = 'amber'

class DniAG9032V1FanTrayLED(cumulus.sensors.StatusLED_Unit):
    bad_led_color = 'red'

class DniAG9032V1FanTray1LED(DniAG9032V1FanTrayLED):
    name = 'Fan Tray 1 LED'
    sources = ['Fan1', 'Fan2']
    led_location = 'led_fan_tray1'

class DniAG9032V1FanTray2LED(DniAG9032V1FanTrayLED):
    name = 'Fan Tray 2 LED'
    sources = ['Fan3', 'Fan4']
    led_location = 'led_fan_tray2'

class DniAG9032V1FanTray3LED(DniAG9032V1FanTrayLED):
    name = 'Fan Tray 3 LED'
    sources = ['Fan5', 'Fan6']
    led_location = 'led_fan_tray3'

class DniAG9032V1FanTray4LED(DniAG9032V1FanTrayLED):
    name = 'Fan Tray 4 LED'
    sources = ['Fan7', 'Fan8']
    led_location = 'led_fan_tray4'

class DniAG9032V1FanTray5LED(DniAG9032V1FanTrayLED):
    name = 'Fan Tray 5 LED'
    sources = ['Fan9', 'Fan10']
    led_location = 'led_fan_tray5'

class DniAG9032V1LEDs():
    units = (DniAG9032V1SystemLED(),
             DniAG9032V1PSU1LED(),
             DniAG9032V1PSU2LED(),
             DniAG9032V1FanLED(),
             DniAG9032V1FanTray1LED(),
             DniAG9032V1FanTray2LED(),
             DniAG9032V1FanTray3LED(),
             DniAG9032V1FanTray4LED(),
             DniAG9032V1FanTray5LED(),
             )

# ***** DniAG9032V1 Class *****

class DniAG9032V1(cumulus.platform.Platform):
    name = 'dni,ag9032v1'
    chip = cumulus.platform.TomahawkChip()
    switch = DniAG9032V1Switch(chip)
    cpld = DniAG9032V1CPLD()
    statusleds = DniAG9032V1LEDs()
    pwms = DniAG9032V1PWMs()
    sensors = DniAG9032V1Sensors()

Platform = DniAG9032V1
