#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class DNIC7448NQSFPp(cumulus.platform.Trident40GPort):
    _set_led = '''
; DNIC7448NQSFP -
;   Three bits per port:
;    1st: activity
;    2nd: 1 (constant)
;    3rd: link
;
set_led:
; 1st activity
        port    a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor

        push    (BLINK_STATE_P)
        tand
        tinv
        pack
; 2nd 1
        pushst PORTSTATUS_ONE
        pack
; 3rd link
        pushst PORTSTATUS_LINKEN
        tinv
        pack

        ret
'''
    _set_led_4x10 = '''
set_led:
; 1st activity
        call   get_p_act

        call   get_p_act
        tor

        call   get_p_act
        tor

        call   get_p_act
        tor
        push   (BLINK_STATE_P)
        tand
        tinv
        pack
; 2nd 1
        pushst PORTSTATUS_ONE
        pack
; 3rd link
        sub    a,4
        call   get_p_lnk

        call   get_p_lnk
        tor

        call   get_p_lnk
        tor

        call   get_p_lnk
        tor
        tinv
        pack
        ret

get_p_act:
        port   a

        pushst PORTSTATUS_RX
        pushst PORTSTATUS_TX
        tor
        inc    a
        ret

get_p_lnk:
        port   a

        pushst PORTSTATUS_LINKEN
        inc    a
        ret
'''

    def __init__(self, label, serdes, phy_addrs, leds):
        cumulus.platform.Trident40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=("84740",) * 4,
                                                 phy_addrs=phy_addrs,
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMBCM(phy_addrs[0], 'qsfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.hw_port_num(mode, 0)
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call

        return set_calls


class DNIC7448N10GT(cumulus.platform.Trident10GPort):
    _set_led = '''
; DNIC7448N10GT - three bits are allocated for each 10GT port
;                  but they are ignored by the hardware (the 10GT
;                  port LEDs are controlled by the phys). Just
;                  pack whatever
set_led:
        pack
        pack
        pack
        ret
'''

    def __init__(self, label, serdes, phy_addr, leds):

        cumulus.platform.Trident10GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_type="84740",
                                                 phy_addr=phy_addr,
                                                 phy_layer='10GBaseT',
                                                 leds=leds,
                                                 unit=0)

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.hw_port_num(mode, 0))
        for led in range(self.num_physical_ports(mode)):
            set_calls[self.leds[1] + led] = call

        return set_calls


class DNIC7448NCPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/ffe05000.localbus/ffdf0000.cpld'


class DNIC7448NLED(cumulus.ledup.TridentLEDController):
    num_leds = 36


class DNIC7448NSwitch(cumulus.platform.BCMSwitch):
    ledup = DNIC7448NLED
    _10GT = DNIC7448N10GT
    _qsfp = DNIC7448NQSFPp
    has_phy_ucode_spi_rom = True
    has_phy_ucode_mdio = True
    ports = (
        _10GT(label="1",  serdes=0,  phy_addr=0x4,  leds=(0, 0)),
        _10GT(label="2",  serdes=1,  phy_addr=0x5,  leds=(0, 1)),
        _10GT(label="3",  serdes=2,  phy_addr=0x6,  leds=(0, 2)),
        _10GT(label="4",  serdes=3,  phy_addr=0x7,  leds=(0, 3)),
        _10GT(label="5",  serdes=4,  phy_addr=0x8,  leds=(0, 4)),
        _10GT(label="6",  serdes=5,  phy_addr=0x9,  leds=(0, 5)),
        _10GT(label="7",  serdes=6,  phy_addr=0xa,  leds=(0, 6)),
        _10GT(label="8",  serdes=7,  phy_addr=0xb,  leds=(0, 7)),
        _10GT(label="9",  serdes=8,  phy_addr=0xc,  leds=(0, 8)),
        _10GT(label="10", serdes=9,  phy_addr=0xd,  leds=(0, 9)),
        _10GT(label="11", serdes=10, phy_addr=0xe,  leds=(0, 10)),
        _10GT(label="12", serdes=11, phy_addr=0xf,  leds=(0, 11)),
        _10GT(label="13", serdes=12, phy_addr=0x10, leds=(0, 12)),
        _10GT(label="14", serdes=13, phy_addr=0x11, leds=(0, 13)),
        _10GT(label="15", serdes=14, phy_addr=0x12, leds=(0, 14)),
        _10GT(label="16", serdes=15, phy_addr=0x13, leds=(0, 15)),
        _10GT(label="17", serdes=20, phy_addr=0x14, leds=(0, 16)),
        _10GT(label="18", serdes=21, phy_addr=0x15, leds=(0, 17)),
        _10GT(label="19", serdes=22, phy_addr=0x16, leds=(0, 18)),
        _10GT(label="20", serdes=23, phy_addr=0x17, leds=(0, 19)),
        _10GT(label="21", serdes=24, phy_addr=0x18, leds=(0, 20)),
        _10GT(label="22", serdes=25, phy_addr=0x19, leds=(0, 21)),
        _10GT(label="23", serdes=26, phy_addr=0x1a, leds=(0, 22)),
        _10GT(label="24", serdes=27, phy_addr=0x1b, leds=(0, 23)),
        _10GT(label="25", serdes=28, phy_addr=0x24, leds=(0, 24)),
        _10GT(label="26", serdes=29, phy_addr=0x25, leds=(0, 25)),
        _10GT(label="27", serdes=30, phy_addr=0x26, leds=(0, 26)),
        _10GT(label="28", serdes=31, phy_addr=0x27, leds=(0, 27)),
        _10GT(label="29", serdes=32, phy_addr=0x28, leds=(0, 28)),
        _10GT(label="30", serdes=33, phy_addr=0x29, leds=(0, 29)),
        _10GT(label="31", serdes=34, phy_addr=0x2a, leds=(0, 30)),
        _10GT(label="32", serdes=35, phy_addr=0x2b, leds=(0, 31)),
        _10GT(label="33", serdes=36, phy_addr=0x2c, leds=(0, 32)),
        _10GT(label="34", serdes=37, phy_addr=0x2d, leds=(0, 33)),
        _10GT(label="35", serdes=38, phy_addr=0x2e, leds=(0, 34)),
        _10GT(label="36", serdes=39, phy_addr=0x2f, leds=(0, 35)),
        _10GT(label="37", serdes=40, phy_addr=0x30, leds=(1, 0)),
        _10GT(label="38", serdes=41, phy_addr=0x31, leds=(1, 1)),
        _10GT(label="39", serdes=42, phy_addr=0x32, leds=(1, 2)),
        _10GT(label="40", serdes=43, phy_addr=0x33, leds=(1, 3)),
        _10GT(label="41", serdes=44, phy_addr=0x34, leds=(1, 4)),
        _10GT(label="42", serdes=45, phy_addr=0x35, leds=(1, 5)),
        _10GT(label="43", serdes=46, phy_addr=0x36, leds=(1, 10)),
        _10GT(label="44", serdes=47, phy_addr=0x37, leds=(1, 11)),
        _10GT(label="45", serdes=48, phy_addr=0x38, leds=(1, 12)),
        _10GT(label="46", serdes=49, phy_addr=0x39, leds=(1, 13)),
        _10GT(label="47", serdes=50, phy_addr=0x3a, leds=(1, 14)),
        _10GT(label="48", serdes=51, phy_addr=0x3b, leds=(1, 15)),

        _qsfp(label="49", serdes=(68,69,70,71), phy_addrs=(0x50,0x51,0x52,0x53), leds=(1, 8)),
        _qsfp(label="50", serdes=(56,57,58,59), phy_addrs=(0x44,0x45,0x46,0x47), leds=(1, 6)),
        _qsfp(label="51", serdes=(64,65,66,67), phy_addrs=(0x54,0x55,0x56,0x57), leds=(1, 9)),
        _qsfp(label="52", serdes=(60,61,62,63), phy_addrs=(0x48,0x49,0x4a,0x4b), leds=(1, 7)),
    )

''' tmp75: path for temp sensor #1 '''
DNIC7448N_i2c_1_49_path = '/sys/bus/i2c/devices/1-0049'

''' tmp75: path for temp sensor #2 '''
DNIC7448N_i2c_1_4a_path = '/sys/bus/i2c/devices/1-004a'

''' tmp75: path for temp sensor #3 '''
DNIC7448N_i2c_1_4b_path = '/sys/bus/i2c/devices/1-004b'

''' tmp75: path for temp sensor #4 '''
DNIC7448N_i2c_1_4c_path = '/sys/bus/i2c/devices/1-004c'

''' dni_dps460: path for the PSU1 PMBus driver - Fan5, temp5, temp6 '''
DNIC7448N_i2c_1_59_path = '/sys/bus/i2c/devices/1-0059'

''' dni_dps460: path for the PSU2 PMBus driver - Fan6, temp7, temp8 '''
DNIC7448N_i2c_1_58_path = '/sys/bus/i2c/devices/1-0058'

''' path for MAC temp sensor - temp #9 '''
DNIC7448N_pci_0100_path = '/sys/devices/pci0000:00/0000:00:00.0/0000:01:00.0'

''' max6620: path for fan #1 - #4 '''
DNIC7448N_i2c_1_29_path = '/sys/bus/i2c/devices/1-0029'

''' ltc4215: path for PSU1 '''
DNIC7448N_i2c_1_42_path = '/sys/bus/i2c/devices/1-0042'

''' ltc4215: path for PSU2 '''
DNIC7448N_i2c_1_40_path = '/sys/bus/i2c/devices/1-0040'

class DNIC7448NSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_system'
    good_led_color = 'blue'
    bad_led_color = 'blue_blinking'

class DNIC7448NFan1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan1'
    sources = ['Fan1', 'Fan2']
    led_location = 'led_fan_tray_0'
    bad_led_color = 'off'

class DNIC7448NFan2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan2'
    sources = ['Fan3', 'Fan4']
    led_location = 'led_fan_tray_1'
    bad_led_color = 'off'

class DNIC7448NPSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = 'rmmod dni_dps460 && modprobe dni_dps460'
            subprocess.call(cmd, shell=True)
            self.init_done = True

class DNIC7448NPSU1(DNIC7448NPSU):
    name = 'PSU1'
    driver_path = DNIC7448N_i2c_1_42_path
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr1']

class DNIC7448NPSU2(DNIC7448NPSU):
    name = 'PSU2'
    driver_path = DNIC7448N_i2c_1_40_path
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr2']

class DNIC7448NTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = 'CPU Sensor'
    driver_path = DNIC7448N_i2c_1_49_path

class DNIC7448NTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'MAC Sensor'
    driver_path = DNIC7448N_i2c_1_4a_path

class DNIC7448NTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Left of Phy'
    driver_path = DNIC7448N_i2c_1_4b_path

class DNIC7448NTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = 'Right of Phy'
    driver_path = DNIC7448N_i2c_1_4c_path

class DNIC7448NPSUTemp(cumulus.sensors.Temp_Unit):
    temp_crit = 85
    temp_max = 50
    temp_fan_min = 25
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class DNIC7448NTemp5(DNIC7448NPSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    driver_path = DNIC7448N_i2c_1_59_path
    driver_hwmon = ['temp1']
    psu = 'PSU1'

class DNIC7448NTemp6(DNIC7448NPSUTemp):
    name = 'PSU1Temp2'
    description = 'PSU1 Temp Sensor'
    driver_path = DNIC7448N_i2c_1_59_path
    driver_hwmon = ['temp2']
    psu = 'PSU1'

class DNIC7448NTemp7(DNIC7448NPSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    driver_path = DNIC7448N_i2c_1_58_path
    driver_hwmon = ['temp1']
    psu = 'PSU2'

class DNIC7448NTemp8(DNIC7448NPSUTemp):
    name = 'PSU2Temp2'
    description = 'PSU2 Temp Sensor'
    driver_path = DNIC7448N_i2c_1_58_path
    driver_hwmon = ['temp2']
    psu = 'PSU2'

class DNIC7448NTemp9(cumulus.sensors.Temp_Unit):
    name = 'Temp9'
    description = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path = DNIC7448N_pci_0100_path
    driver_hwmon = ['temp1']
    temp_max = 100
    temp_crit = 105

class DNIC7448NFan(cumulus.sensors.Fan_Unit):
    rpmmode = 1
    minpwm = 136
    maxpwm = 221
    fan_min = 4500
    fan_max = 18000
    driver_path = DNIC7448N_i2c_1_29_path

class DNIC7448NPSUFan(cumulus.sensors.Fan_Unit):
    rpmmode = 1
    fan_max = 18000
    fan_min = 7000
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class DNIC7448NFan1(DNIC7448NFan):
    name = 'Fan1'
    description = 'Tray 1, Fan 1'
    driver_hwmon = ['fan3']
    cpld_hwmon = ['fan_0']

class DNIC7448NFan2(DNIC7448NFan):
    name = 'Fan2'
    description = 'Tray 1, Fan 2'
    driver_hwmon = ['fan4']
    cpld_hwmon = ['fan_0']

class DNIC7448NFan3(DNIC7448NFan):
    name = 'Fan3'
    description = 'Tray 2, Fan 1'
    driver_hwmon = ['fan1']
    cpld_hwmon = ['fan_1']

class DNIC7448NFan4(DNIC7448NFan):
    name = 'Fan4'
    description = 'Tray 2, Fan 2'
    driver_hwmon = ['fan2']
    cpld_hwmon = ['fan_1']

class DNIC7448NFan5(DNIC7448NPSUFan):
    name = 'PSU1Fan1'
    description = "PSU1 Fan"
    driver_path = DNIC7448N_i2c_1_59_path
    driver_hwmon = ['fan1']
    psu = 'PSU1'

class DNIC7448NFan6(DNIC7448NPSUFan):
    name = 'PSU2Fan1'
    description = "PSU2 Fan"
    driver_path = DNIC7448N_i2c_1_58_path
    driver_hwmon = ['fan1']
    psu = 'PSU2'

class DNIC7448NLEDs():
    units = (
        DNIC7448NSystemLED(),
        DNIC7448NFan1LED(),
        DNIC7448NFan2LED(),
    )

class DNIC7448NPWM1(cumulus.sensors.PWM_Unit):
    fans = (DNIC7448NFan1(),
            DNIC7448NFan2(),
            DNIC7448NFan3(),
            DNIC7448NFan4(),
            )
    temps = (DNIC7448NTemp1(),
             DNIC7448NTemp2(),
             DNIC7448NTemp3(),
             DNIC7448NTemp4(),
             DNIC7448NTemp9(),
             )

class DNIC7448NPWM2(cumulus.sensors.PWM_Unit):
    name = 'PSU1 PWM'
    fans = (DNIC7448NFan5(),
            )
    temps = (DNIC7448NTemp5(),
             DNIC7448NTemp6(),
             )

class DNIC7448NPWM3(cumulus.sensors.PWM_Unit):
    name = 'PSU2 PWM'
    fans = (DNIC7448NFan6(),
            )
    temps = (DNIC7448NTemp7(),
             DNIC7448NTemp8(),
             )

class DNIC7448NSensors():
    units = (DNIC7448NPSU1(),
             DNIC7448NPSU2(),
             DNIC7448NFan1(),
             DNIC7448NFan2(),
             DNIC7448NFan3(),
             DNIC7448NFan4(),
             DNIC7448NFan5(),
             DNIC7448NFan6(),
             DNIC7448NTemp1(),
             DNIC7448NTemp2(),
             DNIC7448NTemp3(),
             DNIC7448NTemp4(),
             DNIC7448NTemp5(),
             DNIC7448NTemp6(),
             DNIC7448NTemp7(),
             DNIC7448NTemp8(),
             DNIC7448NTemp9(),
        )

class DNIC7448NPWMs():
    units = (DNIC7448NPWM1(),
             DNIC7448NPWM2(),
             DNIC7448NPWM3(),
             )

class DNIC7448N(cumulus.platform.Platform):
    name = 'dni,c7448n'
    cpld = DNIC7448NCPLD()
    chip = cumulus.platform.TridentChip()
    switch = DNIC7448NSwitch(chip)
    statusleds = DNIC7448NLEDs()
    pwms = DNIC7448NPWMs()
    sensors = DNIC7448NSensors()

Platform = DNIC7448N
