#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class QuantaLB9SFPp(cumulus.platform.Triumph10GPort):
    def __init__(self, label, hw_port, phy_addr, leds):
        cumulus.platform.Triumph10GPort.__init__(self,
                                                 label=label,
                                                 hw_port=hw_port,
                                                 phy_type="8727",
                                                 phy_addr=phy_addr,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')
    _set_led_link = '''
; QuantaLB9SFPp - link
set_led:
        port a

        pushst  PORTSTATUS_LINKEN
        tinv                    ; LEDs are active low
        pack

        ret
'''
    _set_led_activity = '''
; QuantaLB9SFPp - activity
set_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv ; LEDs active low
        pack

        ret
    '''
    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call_link = (fn_name + '_link', self._set_led_link, self.hw_port_num(mode, 0))
        call_activity = (fn_name + '_activity', self._set_led_activity, self.hw_port_num(mode, 0))
        set_calls[led_index_list[0]] = call_link
        set_calls[led_index_list[0] + 2] = call_activity

        return set_calls

class QuantaLB9GigE(cumulus.platform.Triumph1GPort):
    def __init__(self, label, hw_port, phy_addr):
        cumulus.platform.Triumph1GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type="54680",
                                                phy_addr=phy_addr,
                                                phy_layer='1GBaseT',
                                                leds=None,
                                                unit=0)

class QuantaLB9LED(cumulus.ledup.TriumphLEDController):
    num_leds = 8

class QuantaLB9Switch(cumulus.platform.BCMSwitch):
    '''
    led chain starting from bit 1 and ending on bit 40....
     1 : sfp_45_act, sfp_45_link_g, sfp_45_link_y,
     4 : sfp_46_act, sfp_46_link_g, sfp_46_link_y,
     7 : sfp_47_act, sfp_47_link_g, sfp_47_link_y,
    10 : sfp_48_act, sfp_48_link_g, sfp_48_link_y,
    13 : rj_45_act, 1, rj_45_link_g, rj_45_link_y,
    17 : rj_46_act, 1, rj_46_link_g, rj_46_link_y,
    21 : rj_47_act, 1, rj_47_link_g, rj_47_link_y,
    25 : rj_48_act, 1, rj_48_link_g, rj_48_link_y,
    29 : mod_a_0, mod_a_1, mod_a_2,
    32 : mod_b_0, mod_b_1, mod_b_2,
    35 : 1, 1, 1,
    38 : 1, 1, 1
    '''
    ledup = QuantaLB9LED
    _gige = QuantaLB9GigE
    _sfp = QuantaLB9SFPp
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = True

    ports = (
        _gige(label="1",  hw_port='ge0',  phy_addr=0x1),
        _gige(label="2",  hw_port='ge1',  phy_addr=0x2),
        _gige(label="3",  hw_port='ge2',  phy_addr=0x3),
        _gige(label="4",  hw_port='ge3',  phy_addr=0x4),
        _gige(label="5",  hw_port='ge4',  phy_addr=0x5),
        _gige(label="6",  hw_port='ge5',  phy_addr=0x6),
        _gige(label="7",  hw_port='ge6',  phy_addr=0x7),
        _gige(label="8",  hw_port='ge7',  phy_addr=0x8),
        _gige(label="9",  hw_port='ge8',  phy_addr=0x9),
        _gige(label="10", hw_port='ge9' , phy_addr=0xa),
        _gige(label="11", hw_port='ge10', phy_addr=0xb),
        _gige(label="12", hw_port='ge11', phy_addr=0xc),
        _gige(label="13", hw_port='ge12', phy_addr=0xd),
        _gige(label="14", hw_port='ge13', phy_addr=0xe),
        _gige(label="15", hw_port='ge14', phy_addr=0xf),
        _gige(label="16", hw_port='ge15', phy_addr=0x10),
        _gige(label="17", hw_port='ge16', phy_addr=0x11),
        _gige(label="18", hw_port='ge17', phy_addr=0x12),
        _gige(label="19", hw_port='ge18', phy_addr=0x13),
        _gige(label="20", hw_port='ge19', phy_addr=0x14),
        _gige(label="21", hw_port='ge20', phy_addr=0x15),
        _gige(label="22", hw_port='ge21', phy_addr=0x16),
        _gige(label="23", hw_port='ge22', phy_addr=0x17),
        _gige(label="24", hw_port='ge23', phy_addr=0x18),
        _gige(label="25", hw_port='ge24', phy_addr=0x21),
        _gige(label="26", hw_port='ge25', phy_addr=0x22),
        _gige(label="27", hw_port='ge26', phy_addr=0x23),
        _gige(label="28", hw_port='ge27', phy_addr=0x24),
        _gige(label="29", hw_port='ge28', phy_addr=0x25),
        _gige(label="30", hw_port='ge29', phy_addr=0x26),
        _gige(label="31", hw_port='ge30', phy_addr=0x27),
        _gige(label="32", hw_port='ge31', phy_addr=0x28),
        _gige(label="33", hw_port='ge32', phy_addr=0x29),
        _gige(label="34", hw_port='ge33', phy_addr=0x2a),
        _gige(label="35", hw_port='ge34', phy_addr=0x2b),
        _gige(label="36", hw_port='ge35', phy_addr=0x2c),
        _gige(label="37", hw_port='ge36', phy_addr=0x2d),
        _gige(label="38", hw_port='ge37', phy_addr=0x2e),
        _gige(label="39", hw_port='ge38', phy_addr=0x2f),
        _gige(label="40", hw_port='ge39', phy_addr=0x30),
        _gige(label="41", hw_port='ge40', phy_addr=0x31),
        _gige(label="42", hw_port='ge41', phy_addr=0x32),
        _gige(label="43", hw_port='ge42', phy_addr=0x33),
        _gige(label="44", hw_port='ge43', phy_addr=0x34),
        _gige(label="45", hw_port='ge44', phy_addr=0x35),
        _gige(label="46", hw_port='ge45', phy_addr=0x36),
        _gige(label="47", hw_port='ge46', phy_addr=0x37),
        _gige(label="48", hw_port='ge47', phy_addr=0x38),
        _sfp(label="49",  hw_port='hg0',  phy_addr=0x42, leds=(0,0)),
        _sfp(label="50",  hw_port='hg1',  phy_addr=0x43, leds=(0,1)),
        _sfp(label="51",  hw_port='hg2',  phy_addr=0x46, leds=(0,4)),
        _sfp(label="52",  hw_port='hg3',  phy_addr=0x47, leds=(0,5)),
    )

class QuantaLB9_PSU(cumulus.sensors.PSU_Unit):
    def find_and_add_attrs(self):
        if self.get_gpio_value(self.psu_present) == '0':
            self.attrs['psu_pwr1_present'] = 1
        else:
            self.attrs['psu_pwr1_present'] = 0
        if self.get_gpio_value(self.psu_all_ok) == '1':
            self.attrs['psu_pwr1_all_ok'] = 1
        else:
            self.attrs['psu_pwr1_all_ok'] = 0
        return 0

QuantaLB9_5_2c_path = '/sys/devices/soc8541.0/e0003000.i2c/i2c-0/i2c-5/5-002c'
QuantaLB9_6_2f_path = '/sys/devices/soc8541.0/e0003000.i2c/i2c-0/i2c-6/6-002f'

class QuantaLB9Fan(cumulus.sensors.Fan_Unit):
    minpwm = 64
    maxpwm = 255
    fan_min = 2500
    fan_max = 14000
    def __init__(self):
        off = int(self.__class__.__name__[12:])%4 + 1
        self.name = 'Fan' + str(off)
        self.driver_hwmon = ['fan' + str(off)]
        self.pwm_hwmon = ['pwm' + str(off)]

        cumulus.sensors.Fan_Unit.__init__(self)


class QuantaLB9Fan1(QuantaLB9Fan):
    driver_path = QuantaLB9_5_2c_path

class QuantaLB9Fan2(QuantaLB9Fan):
    driver_path = QuantaLB9_5_2c_path

class QuantaLB9Fan3(QuantaLB9Fan):
    driver_path = QuantaLB9_5_2c_path

class QuantaLB9Fan4(QuantaLB9Fan):
    driver_path = QuantaLB9_5_2c_path

class QuantaLB9Fan5(QuantaLB9Fan):
    driver_path = QuantaLB9_6_2f_path

class QuantaLB9Fan6(QuantaLB9Fan):
    driver_path = QuantaLB9_6_2f_path

class QuantaLB9Fan7(QuantaLB9Fan):
    driver_path = QuantaLB9_6_2f_path

class QuantaLB9Fan8(QuantaLB9Fan):
    driver_path = QuantaLB9_6_2f_path

class QuantaLB9PSU1(QuantaLB9_PSU):
    name = 'PSU1'
    psu_present = 'gpio96'
    psu_all_ok = 'gpio97'

class QuantaLB9PSU2(QuantaLB9_PSU):
    name = 'PSU2'
    psu_present = 'gpio112'
    psu_all_ok = 'gpio113'

class QuantaLB9SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'gpio246'
    good_led_color = '1'  # Off
    bad_led_color = '0'   # Yellow

    def led_update(self, color):
        self.set_gpio_value(self.led_location, color)
        #print 'loc: %s color %s' %(self.led_location, color)

    def led_read(self):
        color = self.get_gpio_value(self.led_location)
        if color == '0':
            color = 'yellow'
        else:
            color = 'off'
        return self.name, color

class QuantaLB9Temp(cumulus.sensors.Temp_Unit):
    def __init__(self):
        self.name = (self.__class__.__name__[9:])
        self.driver_path = QuantaLB9_5_2c_path
        off = int(self.__class__.__name__[13:])
        self.driver_hwmon = ['temp' + str(off)]

        cumulus.sensors.Temp_Unit.__init__(self)

class QuantaLB9Temp1(QuantaLB9Temp):
    description = "Fan outlet Sensor"
    temp_fan_min = 35
    temp_max = 50
    temp_crit = 55

class QuantaLB9Temp2(QuantaLB9Temp):
    description = "MPC8541(CPU) Sensor"

class QuantaLB9Temp3(QuantaLB9Temp):
    description = "Front Right Side"
    temp_fan_min = 35
    temp_max = 58
    temp_crit = 65

class QuantaLB9Temp4(QuantaLB9Temp):
    description = "Front Left Side"
    temp_fan_min = 35
    temp_max = 58
    temp_crit = 65

class QuantaLB9Temp5(QuantaLB9Temp):
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name

def QuantaLB9FindFans():
    # Quick hack to prevent other platforms from running this. CM-2908
    # Need to fix this elegantly.
    plat = os.popen('platform-detect -m').read().strip()
    if plat != 'quanta,lb9':
        return ()

    # Logic taken from /usr/share/platform-config/quanta/lb9/hw_init.d/S20fan_config.sh
    f2b_fans = (QuantaLB9Fan1(),
                QuantaLB9Fan2(),
                QuantaLB9Fan3(),
                QuantaLB9Fan4(),
    )
    b2f_fans = (QuantaLB9Fan5(),
                QuantaLB9Fan6(),
                QuantaLB9Fan7(),
                QuantaLB9Fan8(),
    )
    f_name = subprocess.check_output("decode-syseeprom |grep 'Part Number'|awk '{print $5}'", shell=True)

    found = 0
    if f_name == "1LB9BZZ0STQ":
        return f2b_fans
    elif f_name == "1LB9BZZ0STR":
        return b2f_fans

    b2f_count = 0
    f2b_count = 0
    for fan in b2f_fans:
        if fan.detect_state() != 'ABSENT':
            b2f_count = b2f_count + 1
    for fan in f2b_fans:
        if fan.detect_state() != 'ABSENT':
            f2b_count = f2b_count + 1
    if f2b_count > b2f_count:
        return f2b_fans

    return b2f_fans

class QuantaLB9PWM1(cumulus.sensors.PWM_Unit):
    temps = (QuantaLB9Temp1(),
             QuantaLB9Temp2(),
             QuantaLB9Temp3(),
             QuantaLB9Temp4(),
             QuantaLB9Temp5(),
    )
    fans = QuantaLB9FindFans()

class QuantaLB9Sensors():
    psus = (QuantaLB9PSU1(),
            QuantaLB9PSU2(),
    ) 
    temps = (QuantaLB9Temp1(),
             QuantaLB9Temp2(),
             QuantaLB9Temp3(),
             QuantaLB9Temp4(),
             QuantaLB9Temp5(),
    )
    fans = QuantaLB9FindFans()

    units = psus + fans + temps

class QuantaLB9PWMs():
    units = (QuantaLB9PWM1(),
             )

class QuantaLB9LEDs():
    units = (QuantaLB9SystemLED(),
             )

class QuantaLB9(cumulus.platform.Platform):
    name = 'quanta,lb9'
    chip = cumulus.platform.TriumphChip()
    switch = QuantaLB9Switch(chip)
    statusleds = QuantaLB9LEDs()
    pwms = QuantaLB9PWMs()
    sensors = QuantaLB9Sensors()

Platform = QuantaLB9
