#
# Copyright 2015.  Cumulus Networks, Inc.
#
# sysctlconfig.py --
#     Generate sysctl.d configuration fragments.
#

import sys
from collections import OrderedDict

class Config:
    '''
    Base class for sysctl configuration.  Understands sysctl format.
    '''
    def __init__(self, platform, verbose=False):
        self.platform = platform
        self.verbose = verbose

        # neigh_gc_thresh_ratios{} -- refers to the ratio of sysctl
        # parameters neigh.default.gc_thresh2 to neigh.default.gc_thresh3.
        #
        #   neigh.default.gc_thresh2 - Threshold when garbage collector
        #                              becomes more aggressive about
        #                              purging entries.
        #
        #   neigh.default.gc_thresh3 - Maximum number of neighbor entries
        #                              allowed.
        #
        # See the Linux kernel documentation for more info:
        #   Documentation/networking/ip-sysctl.txt
        #
        # neigh_gc_thresh_ratios[2] - fraction of ipv4_host_table_size to use for gc_thresh2
        # neigh_gc_thresh_ratios[3] - fraction of ipv4_host_table_size to use for gc_thresh3

        self.neigh_gc_thresh_ratios = { 2 : 0.875, #gc_thresh2
                                        3 : 1.0,   #gc_thresh3
        }

        # neigh_ip_entry_ratio[4] - fraction of ipv4_host_table_size to use ipv4 entries
        # neigh_ip_entry_ratio[6] - fraction of ipv4_host_table_size to use ipv6 entries
        self.neigh_ip_entry_ratios = { 4 : 1.0, #ipv4
                                       6 : 0.5, #ipv6
        }

        self.neigh_dict = OrderedDict()

        if self.platform.chip is not None:
            for ip in (4, 6):
                for thresh in (2, 3):
                    key = 'net.ipv%d.neigh.default.gc_thresh%d' % (ip, thresh)
                    value = int(self.platform.chip.ipv4_host_table_size *
                                self.neigh_ip_entry_ratios[ip] *
                                self.neigh_gc_thresh_ratios[thresh])
                    self.neigh_dict[key] = value

        # set non-chip specific global cumulus
        # default neigh base reachable time
        for ip in (4, 6):
            key = 'net.ipv%d.neigh.default.base_reachable_time_ms' % (ip)
            self.neigh_dict[key] = 14400000

        # See Linux kernel documentation for
        #  Documentation/networking/ip-sysctl.txt
        self.igmp_dict = OrderedDict()
        self.igmp_dict['net.ipv4.igmp_max_memberships'] = 1000

        self.mld_dict = OrderedDict()
        self.mld_dict['net.ipv6.mld_max_msf'] = 512

    @property
    def neigh(self):
        neigh = []

        for k, v in self.neigh_dict.iteritems():
            neigh.append('%s=%s' % (k,v))

        return '\n'.join(neigh)

    @property
    def igmp(self):
        igmp = []

        for k, v in self.igmp_dict.iteritems():
            igmp.append('%s=%s' % (k,v))

        return '\n'.join(igmp)

    @property
    def mld(self):
        mld = []

        for k, v in self.mld_dict.iteritems():
            mld.append('%s=%s' % (k,v))

        return '\n'.join(mld)

    def output(self, configfile):
        if configfile == 'neigh':
            out = [
                '\n# neigh',
                self.neigh,
            ]
        elif configfile == 'igmp':
            out = [
                '\n# igmp',
                self.igmp,
            ]
        elif configfile == 'mld':
            out = [
                '\n# mld',
                self.mld,
            ]

        return '\n'.join(out)
