#
# Copyright (C) 2017 Cumulus Networks, Inc.
#
# vhwmon.py --
#
#     Interface to the vhwmon driver
#

import fcntl
import struct
import ctypes
import os

class VHWMon:
    '''
    Interface to the virtual hwmon driver.

    The vhwmon driver is a way to create sysfs hwmon nodes
    that look and feel enough like real hwmon to fool
    the lm-sensors suite.

    The concepts used here are
        device - a vhwmon device, corresponding to a sysfs directory
            Ours look like /sys/class/hwmon/hwmon<n>.
        attribute - a single file in sysfs that contains a value
            The file name and file content make up the attribute
            name and value.
            A device contains a number of attributes.  They look
            like this: /sys/class/hwmon/<n>/<name>.

    Because there are many types of sensors and each sensor
    type has many optional parts, the vhwmon interface doesn't
    try to impose any structure on the attribute names and values
    it manages.  It's up to the caller to make sure they make
    sense as sensors.  What the driver does provides is a way
    to put them in the right place in /sys/class/hwmon.

    An example of a vhwmon device:
    A device with a single temperature sensor would show up
    as a directory, say, /sys/class/hwmon/hwmon23.
    In the directory are two files:
        name - the name of the sensor device
            It can be any string, like "psu1".
        temp1_input - the temperature in millidegrees C
            The value is a string, like "23100" for 23.1 degrees.
    A device can have many sensors.  If "psu1" also has a voltage
    sensor, it might be called "in0_input" with value "11900"
    for 11.9 volts.  To recap, this is what the hwmon directory
    looks like for this device:
        /sys/class/hwmon:
            hwmon23:
                name: psu1
                temp1_input: 23100
                in0_input: 11900

    There are three ioctl calls:
        alloc(name, attrNames, attrValues) -
            Create a device with specified name and attributes
            (names and values).  The example above would be
                alloc("psu1",
                      ["temp1_input", "in0_input"],
                      ["23100", "11900"])
            The return value is a numeric ID.
        free(id) -
            Free the device identified by id.
        set(id, values) -
            Set attribute values.  All attribute values of the
            device must be given, even if they don't change.
            For example, to change temp1_input to 23.5 degrees,
            do set(id, ["23500", "11900"]).
    '''

    #
    # Specifics of the vhwmon ioctl interface, transliterated
    # from vhwmon.h.
    #

    # size limits, some are also in the packers below
    MAX_DEVICES = 512		# number of devices
    MAX_ATTRS = 32		# number of attributes per device
    DEVICE_NAME_LEN = 64	# device name length
    ATTR_NAME_LEN = 32		# attribute name length
    ATTR_VALUE_LEN = 16		# attribute value length

    # ioctl numbers
    ALLOC = 1
    FREE = 2
    SET = 3

    #
    # These define the ioctl arguments
    # The code below that uses them also makes some assumptions
    # about alignment within a structure.  For example, the argument
    # to alloc is actally composed by appending an _alloc_packer object
    # with a number of _attr_packer objects.
    #
    _alloc_packer = struct.Struct("64si")
    _attr_packer = struct.Struct("32s16s")
    _free_packer = struct.Struct("i")
    _set_packer = struct.Struct("i")
    _value_packer = struct.Struct("16s")

    def __init__(self):
        self.fd = open("/dev/vhwmon", "r+")
        size = max(self._alloc_packer.size +
                        self._attr_packer.size * self.MAX_ATTRS,
                   self._free_packer.size,
                   self._set_packer.size +
                        self._value_packer.size * self.MAX_ATTRS)
        # XXX not reentrant
        self._argbuf = ctypes.create_string_buffer(size)
        self._nattrs = [None] * self.MAX_DEVICES

    def alloc(self, name, attrNames, attrValues):
        '''
        Create hwmon directory with specified name, containing
        specified attribute names and values.
        '''
        if len(name) > self.DEVICE_NAME_LEN:
            raise RuntimeError("Group name too long (%d max)" %
                               self.DEVICE_NAME_LEN)
        nattrs = len(attrNames)
        if nattrs > self.MAX_ATTRS:
            raise RuntimeError("Too many attributes (%d max)" % self.MAX_ATTRS)
        if len(attrValues) != nattrs:
            raise RuntimeError("Wrong number of attribute values")
        self._alloc_packer.pack_into(self._argbuf, 0, name, nattrs)
        for i in range(nattrs):
            if len(attrNames[i]) >= self.ATTR_NAME_LEN:
                raise RuntimeError("Attribute name \"%s\" too long (%d max)" %
                                   (attrNames[i], self.ATTR_NAME_LEN))
            if len(attrValues[i]) >= self.ATTR_VALUE_LEN:
                raise RuntimeError("Attribute value \"%s\" too long (%d max)" %
                                   (attrValues[i], self.ATTR_VALUE_LEN))
            self._attr_packer.pack_into(self._argbuf,
                        self._alloc_packer.size + self._attr_packer.size * i,
                        attrNames[i], attrValues[i])
        id = fcntl.ioctl(self.fd, self.ALLOC, self._argbuf)
        self._nattrs[id] = nattrs
        return id

    def free(self, id):
        '''
        Destroy a previously created directory.
        '''
        self._nattrs[id] = None
        self._free_packer.pack_into(self._argbuf, 0, id)
        return fcntl.ioctl(self.fd, self.FREE, self._argbuf)

    def set(self, id, values):
        '''
        Set the values of all attributes in a directory.
        '''
        if len(values) != self._nattrs[id]:
            raise RuntimeError("Wrong number of values (should be %d)" %
                               self._nattrs[id])
        self._set_packer.pack_into(self._argbuf, 0, id)
        for i in range(len(values)):
            self._value_packer.pack_into(self._argbuf,
                        self._set_packer.size + self._value_packer.size * i,
                        values[i])
        return fcntl.ioctl(self.fd, self.SET, self._argbuf)

    def device_name(self, id, base):
        return "vhwmon%d:%s" % (id, base)
