#!/usr/bin/env python
# Copyright 2017 Cumulus Networks, Inc.
#
# Neighbor manager cmdline handler

import json
import logging
import os
import select
import socket
import struct

import stream_lib


'''
1. neighmgrctl communicates with NeighMgrCliT to configure and display
databases maintained by neighmgrd.
2. NeighMgrCliT creates a unix-domain-stream server socket that is advertised
as - /var/run/neighmgrd/uds
3. neighmgrctl connects to this socket and sends commands to NeighMgrCliT.
4. Responses are sent to neighmgrctl as JSON strings.

'''

logger = logging.getLogger("neighmgrd")

class NeighMgrCliHandler():
    '''
    start a server socket. accepts connections on it and processes
    commands rxed on those connections.

    '''
    NEIGHMGRD_ADDR = "/var/run/neighmgrd"
    NM_UDS_ADDR = NEIGHMGRD_ADDR + "/uds"
    def __init__(self, stopEvent, nlm, arpsnoop):
        self.stopEvent = stopEvent
        self.nlm = nlm
        self.arpsnoop = arpsnoop

        # server socket
        self.uds = None

        # Accepted client sockets
        # key: clientSock, data: StreamInfo
        self.clients = {}

        # check and create /var/run/neighmgrd/
        if not os.path.exists(self.NEIGHMGRD_ADDR):
            try:
                os.makedirs(self.NEIGHMGRD_ADDR)
            except OSError:
                pass

        machine = os.uname()[4]
        if "ppc" in machine or "powerpc" in machine:
            # XXX - why even bother with ppc
            self._SO_PASSCRED = 20
            self._SO_PEERCRED = 21
        else:
            self._SO_PASSCRED = 16
            self._SO_PEERCRED = 17

    def _show_snoop_link(self, args):
        return str(self.arpsnoop)

    def _show_nl_link(self, args):
        return self.nlm.ifinfo_cache_json_str()

    def _show_nl_neigh(self, args):
        return self.nlm.neigh_cache_json_str()

    def _get_log_level(self):
        level = logging.getLevelName(logger.getEffectiveLevel())
        level = level.lower() if level else None
        return json.dumps({"logLevel": level}, indent=4)

    def _set_log_level(self, args):
        if not args:
            return self._get_log_level()

        statusMsg = "success"
        level = args[0]
        try:
            logger.setLevel(level.upper())
        except:
            statusMsg = "invalid log level"

        info = {"cmdStatus": statusMsg}
        return json.dumps(info, indent=4)

    def _get_pid_uid_gid(self, cliSock):
        pid = uid = gid = -1

        if cliSock.family != socket.AF_UNIX:
            return (pid, uid, gid)

        try:
            credentials = cliSock.getsockopt(socket.SOL_SOCKET, self._SO_PEERCRED, struct.calcsize('3i'))
            pid, uid, gid = struct.unpack('3i', credentials)
        except Exception, e:
            logger.error("socket get credentials failed: %r" % e)

        return (pid, uid, gid)

    def _handle_cmd(self, sockInfo, cmd):
        nmCtlCmds = {
            #Command            : (Function(args), rootOnly)
            "shownllink"        : (self._show_nl_link, False),
            "shownlneigh"       : (self._show_nl_neigh, False),
            "showsnooplink"     : (self._show_snoop_link, False),
            "loglevel"          : (self._set_log_level, True),
        }

        logger.info("Rxed %s from the other side" % cmd)
        cmd = cmd.split()
        key = cmd[0]
        args = cmd[1:] if len(cmd) > 1 else []
        fn, rootOnly = nmCtlCmds.get(key)
        info = {}
        if rootOnly:
            # only root user can run this cmd
            pid, uid, gid = self._get_pid_uid_gid(sockInfo.sock)
            if uid or gid:
                fn = None
                info = {"cmdStatus": "user does not have permission for this command"}
        if fn:
            jout = fn(args)
        else:
            jout = json.dumps(info, indent=4)
        sockInfo.build_write_data(jout)

    def _client_sock_shut(self, sock):
        logger.info("client sock %r shut" % sock)
        if sock in self.clients:
            del self.clients[sock]
            sock.close()

    def _server_sock_shut(self):
        if not self.uds:
            return
        logger.info("server sock %r shut" % self.uds)
        self.uds.close()
        os.remove(self.NM_UDS_ADDR)
        self.uds = None

        # close all previously accepted connections as well
        cliSocks = [sock for sock in self.clients]
        for sock in cliSocks:
            sock.close()
            del self.clients[sock]

    def _sock_shut(self, sock):
        if sock == self.uds:
            self._server_sock_shut()
        else:
            self._client_sock_shut(sock)

    def _client_sock_create(self):
        (sock, addr) = self.uds.accept()
        logger.info("client sock %r %s created" % (sock, addr))
        sock.setblocking(0)
        if sock in self.clients:
            # if something alreadys exist just remove it and start over
            del self.clients[sock]
        sockInfo = stream_lib.StreamInfo(sock, addr)
        self.clients[sock] = sockInfo

    def _server_sock_create(self):
        if os.path.exists(self.NM_UDS_ADDR):
            os.remove(self.NM_UDS_ADDR)

        self.uds = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self.uds.setblocking(0)
        try:
            self.uds.bind(self.NM_UDS_ADDR)
            self.uds.setsockopt(socket.SOL_SOCKET, self._SO_PASSCRED, 1)
            os.chmod(self.NM_UDS_ADDR, 0777)
            self.uds.listen(4)
            logger.info("server sock %r create" % self.uds)
        except socket.error:
            self.uds.close()
            try:
                os.remove(self.NM_UDS_ADDR)
            except:
                pass
            self.uds = None
            logger.error("Failed to create neighmgr unix-domain-socket")

    def uds_proc(self):
        # Setup an unix domain socket that is used by neighmgrctl to issue
        # config/show commands neighmgrd
        self._server_sock_create()
        if not self.uds:
            return

        # accept and process connections
        while self.uds and not self.stopEvent.is_set():
            inputs = [self.uds]
            outputs = []
            for sock in self.clients:
                inputs.append(sock)
                sockInfo = self.clients.get(sock, None)
                if sockInfo.writeBuf:
                    outputs.append(sock)
            # XXX - replace select timeout with a wakeup fd. the timeout will
            # just make the thread wakeup unnecessarily.
            timeout = 1
            try:
                (readable, writeable, exceptional) = select.select(inputs, outputs, inputs, timeout)
            except Exception as e:
                logger.info("select failed: %r" % e)

            for errSock in exceptional:
                self._sock_shut(errSock)

            for readSock in readable:
                # got something on server sock
                if readSock is self.uds:
                    self._client_sock_create()
                    continue
                # got something on a client sock
                sockInfo = self.clients.get(readSock, None)
                if sockInfo:
                    # get the number of bytes we want to read from this socket
                    readlen = sockInfo.get_read_data_len()
                    data = readSock.recv(readlen)
                    if data:
                        cmd = sockInfo.proc_read_data(data)
                        if cmd:
                            self._handle_cmd(sockInfo, cmd)
                    else:
                        self._sock_shut(readSock)

            for writeSock in writeable:
                sockInfo = self.clients.get(readSock, None)
                if sockInfo and sockInfo.writeBuf:
                    try:
                        numSent = writeSock.send(sockInfo.writeBuf)
                        sockInfo.remove_written_data(numSent)
                    except:
                        self._sock_shut(writeSock)

        # we broke out of the processing loop. tear everything down
        self._server_sock_shut()
