#!/usr/bin/env python
# Copyright 2017 Cumulus Networks, Inc.
#
# ARP snooper utility -
#
# 1. Snoops on IPv4 ARP and IPv6 NS/ND packets. And installs kernel entries
#    for misc applications (EVPN, rdnbrd).
# 2. EVPN arp suppress -
#    If the packet is rxed on a bridge access port which is member of a
#    bridge that has network ports with arp_suppress enabled then a local
#    neigh entry is installed on the corresponding SVI or bridge intf.
# 3. rdnbrd -
#    Coming soon....

import ctypes
import dpkt
import ipaddr
import json
import logging
import os
import select
import socket
import struct
import threading

import nlmanager.nlpacket
import nlmanager.nlmanager
import neighmgr_const as nm_const
import neighmgr_pkt as nm_pkt

# from include/linux/socket.h
_SOL_PACKET = 263
_SO_ATTACH_FILTER = 26

# from include/uapi/linux/if_packet.h
_PACKET_AUXDATA = 8
_PACKET_OUTGOING = 4
_TP_STATUS_VLAN_VALID = (1 << 4)

# from include/uapi/linux/if_ether.h
_ETH_P_ALL = 3

# from include/uapi/linux/filter.h
class struct_sock_filter(ctypes.Structure):
    _fields_ = [('code', ctypes.c_ushort),
                ('jt', ctypes.c_ubyte),
                ('jf', ctypes.c_ubyte),
                ('k', ctypes.c_uint)]

# This goes as arg with SO_ATTACH_FILTER
class struct_sock_fprog(ctypes.Structure):
    _fields_ = [('len', ctypes.c_ushort),
                ('filter', ctypes.POINTER(struct_sock_filter))]

# from include/uapi/linux/if_packet.h
class struct_sockaddr_ll(ctypes.Structure):
    _fields_ = [('sll_family', ctypes.c_ushort),
                ('sll_protocol', ctypes.c_short),
                ('sll_ifindex', ctypes.c_int),
                ('sll_hatype', ctypes.c_ushort),
                ('sll_pkttype', ctypes.c_ubyte),
                ('sll_halen', ctypes.c_ubyte),
                ('sll_addr', ctypes.c_ulong)]

# from include/linux/socket.h (len=16)
class struct_cmsghdr(ctypes.Structure):
    _fields_ = [('cmsg_len', ctypes.c_size_t),
                ('cmsg_level', ctypes.c_int),
                ('cmsg_type', ctypes.c_int)]

# from include/uapi/linux/if_packet.h (len=20; NOT 8 byte aligned)
class struct_tpacket_auxdata(ctypes.Structure):
    _fields_ = [('tp_status', ctypes.c_uint),
                ('tp_len', ctypes.c_uint),
                ('tp_snaplen', ctypes.c_uint),
                ('tp_mac', ctypes.c_ushort),
                ('tp_net', ctypes.c_ushort),
                ('tp_vlan_tci', ctypes.c_ushort),
                ('tp_padding', ctypes.c_ushort)]

# from linux recvmsg manpage - data buf will passed in via this
class struct_iovec(ctypes.Structure):
    _fields_ = [("iov_base", ctypes.c_void_p),
                ("iov_len", ctypes.c_size_t)]

# from linux recvmsg manpage - fromaddr, iovec  and control_buf will be passed
# via this
class struct_msghdr(ctypes.Structure):
    _fields_ = [('msg_name', ctypes.c_void_p),
                ('msg_namelen', ctypes.c_uint32),
                ('msg_iov', ctypes.POINTER(struct_iovec)),
                ('msg_iovlen', ctypes.c_size_t),
                ('msg_control', ctypes.c_void_p),
                ('msg_controllen', ctypes.c_size_t),
                ('msg_flags', ctypes.c_int)]

# per-interface stats
INTF_STAT_IDX_RX_ARP = 0
INTF_STAT_IDX_RX_ND = 1
INTF_STAT_IDX_ADD_NEIGH_v4 = 2
INTF_STAT_IDX_ADD_NEIGH_v6 = 3
# update INTF_STAT_IDX_NUM and intfStatStr everytime you add a new stat
INTF_STAT_IDX_NUM = 4

intfStatStr = {INTF_STAT_IDX_RX_ARP: 'rxArpCnt',
               INTF_STAT_IDX_RX_ND: 'rxNdCnt',
               INTF_STAT_IDX_ADD_NEIGH_v4: 'addNeighV4Cnt',
               INTF_STAT_IDX_ADD_NEIGH_v6: 'addNeighV6Cnt'}

logger = logging.getLogger("neighmgrd")

class ArpSnoopIntfEntry(object):
    def __init__(self, ifIndex, ifName, bpf_filter):
        self.ifIndex = ifIndex
        self.ifName = ifName

        # processing stats
        self.stats = [0] * INTF_STAT_IDX_NUM

        # create a socket for all ethertypes - 0x3
        self.sock = socket.socket(socket.PF_PACKET, socket.SOCK_RAW, socket.htons(_ETH_P_ALL))

        try:
            # bind socket to br port by ifName
            self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self.sock.bind((ifName, 0))

            # attach bpf filter to only get arp packets
            self.sock.setsockopt(socket.SOL_SOCKET, _SO_ATTACH_FILTER, bpf_filter)

            # enable aux data for getting the vlan id (kernel strips the
            # vlan tag for raw sockets)
            self.sock.setsockopt(_SOL_PACKET, _PACKET_AUXDATA, 1)

        except socket.error:
            self.sock.close()
            self.sock = None

    def __str__(self):
        return '  %s (%d): sock: %d' % (self.ifName, self.ifIndex, self.sock)

    def cleanup(self):
        # immediate cleanup - can't wait for garbage collection
        if self.sock:
            self.sock.close()
            self.sock = None

class ArpSnoop:
    # output of - <tcpdump -dd '(arp or (icmp6 and (ip6[40] == 135 or ip6[40] == 136))) and inbound'>
    # { 0x28, 0, 0, 0x0000000c },
    # { 0x15, 9, 0, 0x00000806 },
    # { 0x15, 0, 11, 0x000086dd },
    # { 0x30, 0, 0, 0x00000014 },
    # { 0x15, 3, 0, 0x0000003a },
    # { 0x15, 0, 8, 0x0000002c },
    # { 0x30, 0, 0, 0x00000036 },
    # { 0x15, 0, 6, 0x0000003a },
    # { 0x30, 0, 0, 0x00000036 },
    # { 0x15, 1, 0, 0x00000087 },
    # { 0x15, 0, 3, 0x00000088 },
    # { 0x28, 0, 0, 0xfffff004 },
    # { 0x15, 1, 0, 0x00000004 },
    # { 0x6, 0, 0, 0x00040000 },
    # { 0x6, 0, 0, 0x00000000 }

    # (code, jt, jf, k)
    BPF_INS_IDX_CODE = 0
    BPF_INS_IDX_JT = 1
    BPF_INS_IDX_JF = 2
    BPF_INS_IDX_K = 3
    arp_bpf_instructions = [(0x28, 0, 0, 0x0000000c),
                            (0x15, 9, 0, 0x00000806),
                            (0x15, 0, 11, 0x000086dd),
                            (0x30, 0, 0, 0x00000014),
                            (0x15, 3, 0, 0x0000003a),
                            (0x15, 0, 8, 0x0000002c),
                            (0x30, 0, 0, 0x00000036),
                            (0x15, 0, 6, 0x0000003a),
                            (0x30, 0, 0, 0x00000036),
                            (0x15, 1, 0, 0x00000087),
                            (0x15, 0, 3, 0x00000088),
                            (0x28, 0, 0, 0xfffff004),
                            (0x15, 1, 0, 0x00000004),
                            (0x6, 0, 0, 0x00040000),
                            (0x6, 0, 0, 0x00000000)]

    def __init__(self, main_nlm, stopEvent, localLearn):
        self.stopEvent = stopEvent
        # this nlm belongs to the neighmgrd main thread we cannot use it
        # for sending netlink requests from other threads. only use for
        # accessing the main thread's local cache
        self.main_nlm = main_nlm

        # learn neigh on access ports of vxlan bridges that have arp suppress
        # enabled on any of their network ports
        self.localLearn = localLearn

        # key: ifIndex data: ArpSnoopIntfEntry
        self.intfs = {}
        # sockets created by the main thread are used by the ArpRxT for
        # select. so the main thread cannot close out a socket right away.
        # instead it adds it to garbage collection list that is closed
        # out by the ArpRxT
        self.gc_socks = []
        # the intfLock is used for safe access of intfs and gc_socks
        self.intfLock = threading.Lock()

        self.libc_handle = ctypes.cdll.LoadLibrary('libc.so.6')

        # create a wake-up eventfd that can be used by the main thread to
        # wakeup the ArpRx thread to add new sockets. Otherwise ArpRx is
        # blocked on select.
        # XXX - change this to native python wake_up - see CCR-5974
        self.rxWkupFd = self.libc_handle.eventfd(0, 0)
        # if set the ArpRxT will re-evaluate the sock list that it listens
        # on when woken up via rxWkupFd
        self.evalInputs = False

        # setup a netlink manager for ArpRxT's exclusive use
        self.snoop_nlm = nlmanager.nlmanager.NetlinkManager(nm_const.NM_NLM_PID_OFFSET_SNOOPER)

        # ARP receive thread
        self.rxT = threading.Thread(None, self.ArpRxT, 'ArpRxT')
        self.rxT.daemon = True
        self.rxT.start()

        # generate bpf fprog using the code provided by tcpdump -dd
        self.arp_bpf_filters, self.arp_bpf_fprog = self.gen_bpf_filter(self.arp_bpf_instructions)

        # we need to get aux data and that is not possible via native py sock
        # apis. instead we use recvmsg from cdll.
        # setup data buf by allocating a mutable memory block -
        self.bufsize = 1024
        self.buf = ctypes.create_string_buffer(self.bufsize)

        # setup control buf for aux data -
        # Reference:
        # CMSG_ALIGN(len)=>( ((len)+sizeof(long)-1) & ~(sizeof(long)-1) )
        # CMSG_SPACE(len)=>(CMSG_ALIGN(sizeof(struct cmsghdr)) + CMSG_ALIGN(len))
        # auxdata_len=20; add 4 to 8byte align it
        self.ctrl_bufsize = ctypes.sizeof(struct_cmsghdr) + ctypes.sizeof(struct_tpacket_auxdata) + 4
        self.ctrl_buf = ctypes.create_string_buffer(self.ctrl_bufsize)

    def __del__(self):
        self.wakeup_arprx()

    def __str__(self):
        intfs = []
        self.intfLock.acquire()
        for intf in self.intfs:
            intfInfo = self.intfs[intf]
            intfJInfo = {}
            intfJInfo['ifName'] = intfInfo.ifName
            intfJInfo['ifIndex'] = intfInfo.ifIndex
            for idx in range(INTF_STAT_IDX_NUM):
                intfJInfo[intfStatStr[idx]] = intfInfo.stats[idx]
            intfs.append(intfJInfo)
        self.intfLock.release()
        return json.dumps(intfs, indent=4)

    def intf_stat_increment(self, ifIndex, statIdx):
        self.intfLock.acquire()
        intf = self.intfs.get(ifIndex)
        if intf:
            intf.stats[statIdx] += 1
        self.intfLock.release()

    def join_threads(self):
        # creator of arpsnoop will join the arp_snoop threads and wait
        # for their shutdown before exiting
        self.rxT.join()

    def gen_bpf_filter(self, instructions):
        filter_array = struct_sock_filter * len(instructions)
        filters = filter_array()
        idx = 0
        for line in instructions:
            filters[idx].code = line[self.BPF_INS_IDX_CODE]
            filters[idx].jt = line[self.BPF_INS_IDX_JT]
            filters[idx].jf = line[self.BPF_INS_IDX_JF]
            filters[idx].k = line[self.BPF_INS_IDX_K]
            idx += 1

        fprog = struct_sock_fprog()
        fprog.len = len(instructions)
        fprog.filter = ctypes.pointer(filters[0])
        return filters, ctypes.string_at(ctypes.addressof(fprog), ctypes.sizeof(fprog))

    def intf_exists(self, ifIndex):
        self.intfLock.acquire()
        exists = True if ifIndex in self.intfs else False
        self.intfLock.release()
        return exists

    def add_intf(self, ifIndex, ifName):
        if not self.localLearn:
            return

        if not ifIndex or self.intf_exists(ifIndex):
            return

        logger.info("ARP snooper intf %s (%d) add" % (ifName, ifIndex))
        intf = ArpSnoopIntfEntry(ifIndex, ifName, self.arp_bpf_fprog)
        # create a socket and start listening to it
        if not intf.sock:
            logger.warning("ARP snoop rx socket create failed for %s" % ifName)
            return
        self.intfLock.acquire()
        self.intfs[ifIndex] = intf
        self.intfLock.release()

        # wakeup the rx thread to add new sockets to its select
        self.wakeup_arprx()

    def del_intf(self, ifIndex):
        if not ifIndex:
            return

        wkup = False
        self.intfLock.acquire()
        intf = self.intfs.get(ifIndex)
        if intf:
            ifName = intf.ifName
            if intf.sock:
                # only ArpRx thread can close sockets as it maybe actively
                # using the fd. so simply add the socket to a garbage
                # collection list and wakeup ArpRxT.
                self.gc_socks.append(intf.sock)
                intf.sock = None
            del self.intfs[ifIndex]
            wkup = True
        self.intfLock.release()
        if wkup:
            logger.info("ARP snooper intf %s (%d) del" % (ifName, ifIndex))
            # wakeup the rx thread to update sockets in its select list
            self.wakeup_arprx()

    def add_neigh(self, ifIndex, addr, mac):
        debug = nlmanager.nlpacket.RTM_NEWNEIGH in self.snoop_nlm.debug
        neigh = nlmanager.nlpacket.Neighbor(nlmanager.nlpacket.RTM_NEWNEIGH, debug, use_color=self.snoop_nlm.use_color)
        neigh.flags = nlmanager.nlpacket.NLM_F_REQUEST |\
                      nlmanager.nlpacket.NLM_F_CREATE |\
                      nlmanager.nlpacket.NLM_F_REPLACE
        neigh.family = socket.AF_INET if addr.version is 4 else socket.AF_INET6
        neigh.body = struct.pack(neigh.PACK, neigh.family, ifIndex, neigh.NUD_STALE, 0, nlmanager.nlpacket.Route.RTN_UNICAST)
        neigh.add_attribute(neigh.NDA_DST, addr)
        neigh.add_attribute(neigh.NDA_LLADDR, mac)
        neigh.build_message(self.snoop_nlm.sequence.next(), self.snoop_nlm.pid)

        logger.info("neigh add %d %s %s" % (ifIndex, addr, mac))
        self.snoop_nlm.tx_nlpacket_raw(neigh.message)

    def ifIndex2Name(self, ifIndex):
        ifName = None
        self.intfLock.acquire()
        intf = self.intfs.get(ifIndex)
        if intf:
            ifName = intf.ifName
        self.intfLock.release()
        return ifName

    def del_all_intfs(self):
        self.intfLock.acquire()
        for ifIndex in self.intfs.keys():
            intf = self.intfs.get(ifIndex)
            intf.cleanup()
            del self.intfs[ifIndex]
        self.intfLock.release()

    def close_gc_socks(self):
        self.intfLock.acquire()
        # release sockets in the garbage collection list
        for sock in self.gc_socks:
            logger.info("ARP rxT closed %r" % sock)
            sock.close()
        del self.gc_socks[:]
        self.intfLock.release()

    def rebuild_arp_inputs(self):
        self.evalInputs = False
        inputs = [self.rxWkupFd]
        self.intfLock.acquire()
        for ifIndex in self.intfs:
            intf = self.intfs[ifIndex]
            if intf.sock:
                inputs.append(intf.sock)
        self.intfLock.release()
        logger.info("ARP snooper inputs %r rebuilt" % inputs)

        # clear gc list
        self.close_gc_socks()
        return inputs

    def wakeup_arprx(self):
        self.evalInputs = True
        os.write(self.rxWkupFd, ctypes.c_ulonglong(2013))

    def neigh_learn_proc(self, ifIndex, vlan, addr, mac):
        '''
        snooper heard of a ipv4/ipv6 address to lladdr mapping
        '''
        logger.debug("arp check ifIndex %d vlan %d addr %s mac %s\n" % (ifIndex, vlan, addr, mac))
        if not ifIndex:
            logger.info('arp packet ignored; unknown intf')
            return

        if addr.version is 4:
            self.intf_stat_increment(ifIndex, INTF_STAT_IDX_RX_ARP)
        else:
            self.intf_stat_increment(ifIndex, INTF_STAT_IDX_RX_ND)

        # check if the arp snooper is actually interested in that interfacce
        ifName = self.ifIndex2Name(ifIndex)
        if not ifName:
            logger.info('arp packet ignored; unknown intf %d' % ifIndex)
            return

        # XXX - may need to check for other reserved values and ignore
        if addr.is_unspecified:
            logger.debug('arp packet on %s ignored; unspec address' % ifName)
            return

        # get SVI (from the main thread's if cache) for adding neigh
        neighIfIndex, brPortkind, opState = self.main_nlm.get_svi_from_br_port_and_vlan(ifIndex, vlan)
        if not neighIfIndex:
            logger.debug('neigh add for %s on %s skipped; no svi' % (addr, ifName))
            return

        if not opState:
            logger.debug('neigh add for %s on %s skipped; svi is down' % (addr, ifName))
            return

        if brPortkind == 'vxlan':
            # XXX - this is bit of a hack. there are windows where the arp_sup
            # flag is NOT set on the network port and I don't want to learn
            # arp on it during those windows. ref: CCR-5974
            logger.info('arp packet on %d ignored; network port' % ifIndex)
            return

        # if neigh entry doesn't exist (in the main thread's neigh cache)
        # or is from a remote VTEP update it
        (neighMac, isExt) = self.main_nlm.get_neigh_cache_entry(neighIfIndex, addr)
        if not neighMac or isExt:
            # add neigh entry
            if addr.version is 4:
                self.intf_stat_increment(ifIndex, INTF_STAT_IDX_ADD_NEIGH_v4)
            else:
                self.intf_stat_increment(ifIndex, INTF_STAT_IDX_ADD_NEIGH_v6)
            self.add_neigh(neighIfIndex, addr, mac)

    def recvmsg(self, sock):
        # reference: libpcap:pcap_read_packet
        sockaddr = struct_sockaddr_ll()

        iov = struct_iovec()
        iov.iov_base = ctypes.cast(self.buf, ctypes.c_void_p)
        iov.iov_len = self.bufsize

        msghdr = struct_msghdr()
        msghdr.msg_name = ctypes.cast(ctypes.pointer(sockaddr), ctypes.c_void_p)
        msghdr.msg_namelen = ctypes.sizeof(sockaddr)
        msghdr.msg_iov = ctypes.pointer(iov)
        msghdr.msg_iovlen = 1
        msghdr.msg_control = ctypes.cast(self.ctrl_buf, ctypes.c_void_p)
        msghdr.msg_controllen = self.ctrl_bufsize
        msghdr.msg_flags = 0

        vlan = 0
        packetlen = self.libc_handle.recvmsg(sock.fileno(), ctypes.pointer(msghdr), 0)
        if packetlen < 14:
            logger.info("recvmsg packetlen too small %d\n" % packetlen)
            return

        if msghdr.msg_controllen < (ctypes.sizeof(struct_cmsghdr) + ctypes.sizeof(struct_tpacket_auxdata)):
            logger.info("control length too small: %d\n" % msghdr.msg_controllen)
            return

        # get cmsghdr and auxdata from the ctrl_buf
        cmsghdr = struct_cmsghdr.from_buffer(self.ctrl_buf)
        if cmsghdr.cmsg_level != _SOL_PACKET or cmsghdr.cmsg_type != _PACKET_AUXDATA:
            logger.info("cmsg level %d type: %d incorrect\n" % (cmsghdr.cmsg_level, cmsghdr.cmsg_type))
            return

        aux = struct_tpacket_auxdata.from_buffer(self.ctrl_buf, ctypes.sizeof(struct_cmsghdr))

        # XXX: should I always check if _TP_STATUS_VLAN_VALID is set?
        # ref: CCR-5974
        if aux.tp_vlan_tci != 0 or aux.tp_status & _TP_STATUS_VLAN_VALID:
            vlan = aux.tp_vlan_tci & 0xfff

        try:
            eth = dpkt.ethernet.Ethernet(self.buf.raw[:packetlen])
        except Exception as e:
            logger.info("ignore malformed %r packet on %d\n" % (e, sockaddr.sll_ifindex))
            return

        if sockaddr.sll_pkttype == _PACKET_OUTGOING:
            # the BPF filter should only send inbound packets to the app. if we
            # get outbound packets too something went wrong
            logger.info("ignore outbound packet on %d\n" % sockaddr.sll_ifindex)
            return

        logger.debug("Sock addr: %d %d; vlan %d\n" % (sockaddr.sll_ifindex, sockaddr.sll_pkttype, vlan))
        logger.debug("Ethernet: %r\n" % eth)
        if eth.type == dpkt.ethernet.ETH_TYPE_ARP:
            arp = eth.data
            spa = socket.inet_ntop(socket.AF_INET, arp.spa)
            spa = ipaddr.IPAddress(spa)
            sha = nm_pkt.macBytes2Dotted(arp.sha)
            self.neigh_learn_proc(sockaddr.sll_ifindex, vlan, spa, sha)
        elif isinstance(eth.data, dpkt.ip6.IP6):
            addr, lladdr = nm_pkt.nd_pkt_to_learnt_addr(eth)
            if addr:
                self.neigh_learn_proc(sockaddr.sll_ifindex, vlan, addr, lladdr)

    def ArpRx(self):
        # XXX - we should only start rxing after neigh cache is populated.
        # temporarily use a hack to acheive this. ref: CCR-5974
        self.stopEvent.wait(10)
        inputs = self.rebuild_arp_inputs()
        while not self.stopEvent.is_set():
            try:
                readyFds = select.select(inputs, [], inputs)
                readable = readyFds[0]
                exceptional = readyFds[2]
            except Exception as e:
                logger.info('select error: %r' % e)
                if self.evalInputs:
                    inputs = self.rebuild_arp_inputs()
                readable = exceptional = []

            for errSock in exceptional:
                # XXX - what kind of RAW exceptions can we get
                logger.warning('exceptSock: %r exception ignored' % errSock)

            for readSock in readable:
                if readSock == self.rxWkupFd:
                    val = os.read(self.rxWkupFd, 8)
                    logger.info("RX wakeup call %r" % val)
                    if self.stopEvent.is_set():
                        #break out of the loop if somebody told us to stop
                        break
                    if self.evalInputs:
                        inputs = self.rebuild_arp_inputs()
                    continue

                # check if readsock is still in our socklist. if it is not
                # it is possible that the main thread closed it.
                # XXX: Will this be caught as an exception?
                if not readSock in inputs:
                    continue
                self.recvmsg(readSock)


    def ArpRxT(self):
        try:
            self.ArpRx()
            self.del_all_intfs()
            self.close_gc_socks()
        except Exception as e:
            logger.exception(e)
            # Tell all the threads in neighmgrd to stop
            self.stopEvent.set()
            self.main_nlm.listener.shutdown_event.set()
            self.main_nlm.listener.join()
